import java.awt.EventQueue;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;
import javax.swing.JLabel;
import javax.swing.JOptionPane;

import java.awt.Font;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.text.ParseException;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.JComboBox;
import javax.swing.JButton;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTabbedPane;

import ReferenceData.ACTION_RECORD;
import ReferenceData.TYPE_NAME_FORMAT;
import ReferenceData.URI_IDENTIFIER_CODE;
import NDEF.Helper;
import NDEF.Ndef.EncodeNdef;
import NDEF.Ndef.NdefMessage;
import NDEF.Ndef.NdefRecord;
import NDEF.VCard.IdentificationType;
import NDEF.*;
import java.awt.Color;
import javax.swing.JTextField;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JRadioButton;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.KeyAdapter;

@SuppressWarnings("serial")
public class Ntag203Ndef extends JFrame implements ReaderEvents.TransmitApduHandler,ActionListener, KeyListener{

	// Controls
	private JPanel _contentPane;
	private JPanel _PanelSmartPoster;
	private JTextField _TextFieldSmartPosterTitle;
	private JTextField _TextFieldSmartPosterUri;
	private JTextField _TextFieldUrlUri;
	private JTextField _TextFieldGivenName;
	private JTextField _TextFieldMiddleName;
	private JTextField _TextFieldSurname;
	
	private JButton _ButtonEncodeSmartPoster;
	private JButton _ButtonEncodeText;
	private JButton _ButtonEncodeUrl;
	private JButton _ButtonInitialize;
	private JButton _ButtonConnect;
	private JButton _ButtonEncodeVcard;
	private JButton _ButtonWriteToCard;
	private JButton _ButtonReadFromCard;
	private JButton _ButtonClear;
	private JButton _ButtonReset;
	private JButton _ButtonQuit;
	
	private JTabbedPane _TabbedPaneNfcType;
	
	private JComboBox<String> _ComboBoxReaderList;
	private JComboBox<String> _ComboBoxSmartPosterTitle;
	private JComboBox<String> _ComboBoxSmartPosterUri;
	private JComboBox<String> _ComboBoxSmartPosterAction;
	private JComboBox<String> _ComboBoxTextLanguage;
	private JComboBox<String> _ComboBoxUriPrefix;
	
	private JTextArea _TextAreaTextMessage;
	private JTextArea _TextAreaApduLogs;
	private JTextArea _TextAreaNdefMessage;
	
	private JRadioButton _RadioButtonUtf8;
	private JRadioButton _RadioButtonUtf16;
	
	// Global variables
	private PcscReader _pcscReader;
	private Ntag203 _ntag203;
    private ReaderFunctions _readerFunctions;
    private String _cardName; 
    private int _maximumInputLength = 141;
    
    private String _telNumberPattern = "^[\\d-(+)]+$";  
    //private String _namePattern = "[^_\\W[-.]]+$";
    private String _namePattern = "^[a-zA-Z-. ]+$";
    
    Pattern _pattern;
	Matcher _matcher;
    
    final byte BLOCK_3 = 0x03;
    final byte WRITEREAD_LEN = 0x10;
    final byte TERMINATOR_TLV = (byte)0xFE;
    
	
	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					Ntag203Ndef frame = new Ntag203Ndef();
					frame.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Create the frame.
	 * @throws ParseException 
	 */
	public Ntag203Ndef() throws ParseException {
		JLabel lblNewLabel;
	    JLabel lblApduLogs;
	    JLabel lblTitle;
	    JLabel lbluri;
	    JLabel lblAction;
	    JLabel lblRequiredFields;
	    JLabel lblNewLabel_1;
	    JLabel lblRequiredFields_1;
	    JLabel lblLanguage;
	    JLabel lblmessage;
	    JLabel lblRequiredFields_2;
	    JLabel lblNdefMessage;
	    JLabel lblNewLabel_2;
	    JLabel lblMiddleName;
	    JLabel lblSurname;
	    JLabel lblRequiredFields_3;
	    
		_pcscReader = new PcscReader(); //instantiate an event handler object
    	_pcscReader.setEventHandler(new ReaderEvents());
    	_pcscReader.getEventHandler().addEventListener(this);
    	_readerFunctions = new ReaderFunctions(_pcscReader);
    	
		setFont(new Font("Verdana", Font.PLAIN, 10));
		setTitle("NTAG203 NDEF Sample Code");
		setResizable(false);
		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		setBounds(100, 100, 843, 475);
		_contentPane = new JPanel();
		_contentPane.setBorder(new EmptyBorder(5, 5, 5, 5));
		setContentPane(_contentPane);
		_contentPane.setLayout(null);
		
		lblNewLabel = new JLabel("Select Reader:");
		lblNewLabel.setFont(new Font("Verdana", Font.PLAIN, 10));
		lblNewLabel.setBounds(10, 11, 106, 14);
		_contentPane.add(lblNewLabel);
		
		_ComboBoxReaderList = new JComboBox<String>();
		_ComboBoxReaderList.setFont(new Font("Verdana", Font.PLAIN, 10));
		_ComboBoxReaderList.setBounds(10, 26, 412, 20);
		_contentPane.add(_ComboBoxReaderList);
		
		_ButtonInitialize = new JButton("Initialize");
		_ButtonInitialize.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				initialize();
			}
		});
		
		_ButtonInitialize.setFont(new Font("Verdana", Font.PLAIN, 10));
		_ButtonInitialize.setBounds(10, 57, 130, 23);
		_contentPane.add(_ButtonInitialize);
		
		_ButtonConnect = new JButton("Connect");
		_ButtonConnect.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				connect();
			}
		});
		
		_ButtonConnect.setEnabled(false);
		_ButtonConnect.setFont(new Font("Verdana", Font.PLAIN, 10));
		_ButtonConnect.setBounds(292, 57, 130, 23);
		_contentPane.add(_ButtonConnect);
		
		lblApduLogs = new JLabel("APDU Logs:");
		lblApduLogs.setFont(new Font("Verdana", Font.PLAIN, 10));
		lblApduLogs.setBounds(430, 11, 135, 14);
		_contentPane.add(lblApduLogs);
		
		JScrollPane scrollPane = new JScrollPane();
		scrollPane.setBounds(432, 26, 398, 381);
		_contentPane.add(scrollPane);
		
		_TextAreaApduLogs = new JTextArea();
		_TextAreaApduLogs.setFont(new Font("Verdana", Font.PLAIN, 10));
		_TextAreaApduLogs.setEditable(false);
		scrollPane.setViewportView(_TextAreaApduLogs);
		
		_TabbedPaneNfcType = new JTabbedPane(JTabbedPane.TOP);
		_TabbedPaneNfcType.setFont(new Font("Verdana", Font.PLAIN, 10));
		_TabbedPaneNfcType.setBounds(10, 107, 412, 170);
		_contentPane.add(_TabbedPaneNfcType);
		
		_PanelSmartPoster = new JPanel();
		_TabbedPaneNfcType.addTab("Smart Poster", null, _PanelSmartPoster, null);
		_PanelSmartPoster.setLayout(null);
		
		lblTitle = new JLabel("Title");
		lblTitle.setFont(new Font("Verdana", Font.PLAIN, 10));
		lblTitle.setBounds(10, 11, 36, 19);
		_PanelSmartPoster.add(lblTitle);
		
		_ComboBoxSmartPosterTitle = new JComboBox<String>();
		_ComboBoxSmartPosterTitle.setEnabled(false);
		_ComboBoxSmartPosterTitle.setModel(new DefaultComboBoxModel<String>(new String[] {"en", "zn-CN", "de"}));
		_ComboBoxSmartPosterTitle.setFont(new Font("Verdana", Font.PLAIN, 10));
		_ComboBoxSmartPosterTitle.setBounds(56, 10, 110, 20);
		_PanelSmartPoster.add(_ComboBoxSmartPosterTitle);
		
		lbluri = new JLabel("*URI");
		lbluri.setFont(new Font("Verdana", Font.PLAIN, 10));
		lbluri.setBounds(10, 41, 46, 14);
		_PanelSmartPoster.add(lbluri);
		
		lblAction = new JLabel("Action");
		lblAction.setFont(new Font("Verdana", Font.PLAIN, 10));
		lblAction.setBounds(10, 73, 46, 14);
		_PanelSmartPoster.add(lblAction);
		
		_ComboBoxSmartPosterUri = new JComboBox<String>();
		_ComboBoxSmartPosterUri.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				_TextFieldSmartPosterUri.setText("");
			}
		});
		_ComboBoxSmartPosterUri.setEnabled(false);
		_ComboBoxSmartPosterUri.setModel(new DefaultComboBoxModel<String>(new String[] {"None", "http://www.", "https://www.", "http://", "https://", "tel:", "mailto:"}));
		_ComboBoxSmartPosterUri.setFont(new Font("Verdana", Font.PLAIN, 10));
		_ComboBoxSmartPosterUri.setBounds(56, 41, 110, 20);
		_PanelSmartPoster.add(_ComboBoxSmartPosterUri);
		
		_ComboBoxSmartPosterAction = new JComboBox<String>();
		_ComboBoxSmartPosterAction.setEnabled(false);
		_ComboBoxSmartPosterAction.setModel(new DefaultComboBoxModel<String>(new String[] {"No Action", "Do the action", "Save For Later", "Open for editing"}));
		_ComboBoxSmartPosterAction.setFont(new Font("Verdana", Font.PLAIN, 10));
		_ComboBoxSmartPosterAction.setBounds(56, 70, 341, 20);
		_PanelSmartPoster.add(_ComboBoxSmartPosterAction);
		
		_TextFieldSmartPosterTitle = new JTextField();
		_TextFieldSmartPosterTitle.setEnabled(false);		
		_TextFieldSmartPosterTitle.setFont(new Font("Verdana", Font.PLAIN, 10));
		_TextFieldSmartPosterTitle.setBounds(171, 10, 226, 20);
		_PanelSmartPoster.add(_TextFieldSmartPosterTitle);
		_TextFieldSmartPosterTitle.setColumns(10);
		_TextFieldSmartPosterTitle.setDocument(new JTextFieldLimit(141, this));
		_TextFieldSmartPosterTitle.setText("Advanced Card Systems Ltd.");
		
		_TextFieldSmartPosterUri = new JTextField();
		_TextFieldSmartPosterUri.addKeyListener(this);
		_TextFieldSmartPosterUri.addFocusListener(new FocusAdapter() {
			@Override
			public void focusLost(FocusEvent arg0) {
				if(_TextFieldSmartPosterUri.getText().trim().startsWith(_ComboBoxSmartPosterUri.getSelectedItem().toString()))
					_TextFieldSmartPosterUri.setText(_TextFieldSmartPosterUri.getText().substring(_ComboBoxSmartPosterUri.getSelectedItem().toString().length()));
				
			}
		});
		_TextFieldSmartPosterUri.setEnabled(false);		
		_TextFieldSmartPosterUri.setFont(new Font("Verdana", Font.PLAIN, 10));
		_TextFieldSmartPosterUri.setBounds(171, 41, 226, 20);
		_PanelSmartPoster.add(_TextFieldSmartPosterUri);
		_TextFieldSmartPosterUri.setColumns(10);
		_TextFieldSmartPosterUri.setDocument(new JTextFieldLimit(141, this));
		_TextFieldSmartPosterUri.setText("http://www.acs.com.hk");
		
		lblRequiredFields = new JLabel("* Required fields");
		lblRequiredFields.setFont(new Font("Verdana", Font.PLAIN, 10));
		lblRequiredFields.setBounds(10, 117, 121, 14);
		_PanelSmartPoster.add(lblRequiredFields);
		
		_ButtonEncodeSmartPoster = new JButton("Encode Smart Poster");
		_ButtonEncodeSmartPoster.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				try {
					encodeToNdefSmartPoster();
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		});
		_ButtonEncodeSmartPoster.setEnabled(false);
		_ButtonEncodeSmartPoster.setFont(new Font("Verdana", Font.PLAIN, 10));
		_ButtonEncodeSmartPoster.setBounds(237, 108, 160, 23);
		_PanelSmartPoster.add(_ButtonEncodeSmartPoster);
		
		JPanel PanelUrl = new JPanel();
		_TabbedPaneNfcType.addTab("URI", null, PanelUrl, null);
		PanelUrl.setLayout(null);
		
		lblNewLabel_1 = new JLabel("*Uniform Resource Identifier (URI):");
		lblNewLabel_1.setFont(new Font("Verdana", Font.PLAIN, 10));
		lblNewLabel_1.setBounds(10, 24, 211, 14);
		PanelUrl.add(lblNewLabel_1);
		
		_ComboBoxUriPrefix = new JComboBox<String>();
		_ComboBoxUriPrefix.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				_TextFieldUrlUri.setText("");
			}
		});
		_ComboBoxUriPrefix.setEnabled(false);
		_ComboBoxUriPrefix.setFont(new Font("Verdana", Font.PLAIN, 10));
		_ComboBoxUriPrefix.setModel(new DefaultComboBoxModel<String>(new String[] {"None", "http://www.", "https://www.", "http://", "https://", "tel:", "mailto:"}));
		_ComboBoxUriPrefix.setBounds(10, 49, 110, 20);
		PanelUrl.add(_ComboBoxUriPrefix);
		
		_TextFieldUrlUri = new JTextField();
		_TextFieldUrlUri.addKeyListener(this);
		_TextFieldUrlUri.addFocusListener(new FocusAdapter() {
			@Override
			public void focusLost(FocusEvent e) {
				if(_TextFieldUrlUri.getText().trim().startsWith(_ComboBoxUriPrefix.getSelectedItem().toString()))
					_TextFieldUrlUri.setText(_TextFieldUrlUri.getText().substring(_ComboBoxUriPrefix.getSelectedItem().toString().length()));
			}
		});
		_TextFieldUrlUri.setEnabled(false);
		_TextFieldUrlUri.setFont(new Font("Verdana", Font.PLAIN, 10));		
		_TextFieldUrlUri.setBounds(128, 49, 269, 20);
		PanelUrl.add(_TextFieldUrlUri);
		_TextFieldUrlUri.setColumns(10);
		_TextFieldUrlUri.setDocument(new JTextFieldLimit(136, this));
		_TextFieldUrlUri.setText("http://www.acs.com.hk");
		
		lblRequiredFields_1 = new JLabel("* Required fields");
		lblRequiredFields_1.setFont(new Font("Verdana", Font.PLAIN, 10));
		lblRequiredFields_1.setBounds(10, 117, 110, 14);
		PanelUrl.add(lblRequiredFields_1);
		
		_ButtonEncodeUrl = new JButton("Encode URI");
		_ButtonEncodeUrl.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				try {
					encodeToNdefUrl();
				} catch (Exception e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
			}
		});
		_ButtonEncodeUrl.setEnabled(false);
		_ButtonEncodeUrl.setFont(new Font("Verdana", Font.PLAIN, 10));
		_ButtonEncodeUrl.setBounds(237, 108, 160, 23);
		PanelUrl.add(_ButtonEncodeUrl);
		
		JPanel PanelText = new JPanel();
		_TabbedPaneNfcType.addTab("Text", null, PanelText, null);
		PanelText.setLayout(null);
		
		lblLanguage = new JLabel("Language");
		lblLanguage.setFont(new Font("Verdana", Font.PLAIN, 10));
		lblLanguage.setBounds(10, 21, 63, 14);
		PanelText.add(lblLanguage);
		
		lblmessage = new JLabel("*Message");
		lblmessage.setFont(new Font("Verdana", Font.PLAIN, 10));
		lblmessage.setBounds(10, 46, 63, 14);
		PanelText.add(lblmessage);
		
		_ComboBoxTextLanguage = new JComboBox<String>();
		_ComboBoxTextLanguage.setEnabled(false);
		_ComboBoxTextLanguage.setFont(new Font("Verdana", Font.PLAIN, 10));
		_ComboBoxTextLanguage.setModel(new DefaultComboBoxModel<String>(new String[] {"en", "zn-CN", "de"}));
		_ComboBoxTextLanguage.setBounds(75, 18, 110, 20);
		PanelText.add(_ComboBoxTextLanguage);
		
		JScrollPane scrollPane_3 = new JScrollPane();
		scrollPane_3.setBounds(75, 46, 322, 53);
		PanelText.add(scrollPane_3);
		
		_TextAreaTextMessage = new JTextArea();
		_TextAreaTextMessage.setEnabled(false);
		_TextAreaTextMessage.setFont(new Font("Verdana", Font.PLAIN, 10));		
		scrollPane_3.setViewportView(_TextAreaTextMessage);
		_TextAreaTextMessage.setDocument(new JTextFieldLimit(134, this));
		_TextAreaTextMessage.setText("Hello World!");
		
		lblRequiredFields_2 = new JLabel("* Required fields");
		lblRequiredFields_2.setFont(new Font("Verdana", Font.PLAIN, 10));
		lblRequiredFields_2.setBounds(10, 117, 129, 14);
		PanelText.add(lblRequiredFields_2);
		
		_ButtonEncodeText = new JButton("Encode Text");
		_ButtonEncodeText.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				try {
					encodeToNdefMessage();
				} catch (Exception e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
			}
		});
		_ButtonEncodeText.setEnabled(false);
		_ButtonEncodeText.setFont(new Font("Verdana", Font.PLAIN, 10));
		_ButtonEncodeText.setBounds(237, 108, 160, 23);
		PanelText.add(_ButtonEncodeText);
		
		_RadioButtonUtf8 = new JRadioButton("UTF 8");
		_RadioButtonUtf8.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(_RadioButtonUtf8.isSelected())
				{
					_RadioButtonUtf8.setSelected(true);
					_RadioButtonUtf16.setSelected(false);
				}
			}
		});
		
		_RadioButtonUtf8.setEnabled(false);
		_RadioButtonUtf8.setSelected(true);
		_RadioButtonUtf8.setFont(new Font("Verdana", Font.PLAIN, 10));
		_RadioButtonUtf8.setBounds(195, 17, 63, 23);
		PanelText.add(_RadioButtonUtf8);
		
		_RadioButtonUtf16 = new JRadioButton("UTF 16");
		_RadioButtonUtf16.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(_RadioButtonUtf16.isSelected())
				{
					_RadioButtonUtf16.setSelected(true);
					_RadioButtonUtf8.setSelected(false);
				}
			}
		});
		_RadioButtonUtf16.setEnabled(false);
		_RadioButtonUtf16.setFont(new Font("Verdana", Font.PLAIN, 10));
		_RadioButtonUtf16.setBounds(275, 17, 73, 23);
		PanelText.add(_RadioButtonUtf16);
		
		JPanel PanelVCard = new JPanel();
		_TabbedPaneNfcType.addTab("vCard", null, PanelVCard, null);
		PanelVCard.setLayout(null);
		
		lblNewLabel_2 = new JLabel("*Given Name");
		lblNewLabel_2.setFont(new Font("Verdana", Font.PLAIN, 10));
		lblNewLabel_2.setBounds(10, 14, 78, 14);
		PanelVCard.add(lblNewLabel_2);
		
		lblMiddleName = new JLabel("Middle Name");
		lblMiddleName.setFont(new Font("Verdana", Font.PLAIN, 10));
		lblMiddleName.setBounds(10, 42, 78, 14);
		PanelVCard.add(lblMiddleName);
		
		lblSurname = new JLabel("*Surname");
		lblSurname.setFont(new Font("Verdana", Font.PLAIN, 10));
		lblSurname.setBounds(10, 71, 78, 14);
		PanelVCard.add(lblSurname);
		
		_TextFieldGivenName = new JTextField();
		_TextFieldGivenName.setEnabled(false);
		_TextFieldGivenName.addKeyListener(this);
		_TextFieldGivenName.setFont(new Font("Verdana", Font.PLAIN, 10));		
		_TextFieldGivenName.setBounds(98, 11, 299, 20);
		PanelVCard.add(_TextFieldGivenName);
		_TextFieldGivenName.setColumns(10);
		_TextFieldGivenName.setDocument(new JTextFieldLimit(20, this));
		_TextFieldGivenName.setText("Given Name");
		
		_TextFieldMiddleName = new JTextField();
		_TextFieldMiddleName.setEnabled(false);
		_TextFieldMiddleName.addKeyListener(this);
		_TextFieldMiddleName.setFont(new Font("Verdana", Font.PLAIN, 10));		
		_TextFieldMiddleName.setBounds(98, 39, 299, 20);
		PanelVCard.add(_TextFieldMiddleName);
		_TextFieldMiddleName.setColumns(10);
		_TextFieldMiddleName.setDocument(new JTextFieldLimit(20, this));
		_TextFieldMiddleName.setText("Middle Name");
		
		_TextFieldSurname = new JTextField();
		_TextFieldSurname.setEnabled(false);
		_TextFieldSurname.addKeyListener(this);
		_TextFieldSurname.setFont(new Font("Verdana", Font.PLAIN, 10));		
		_TextFieldSurname.setBounds(98, 68, 299, 20);
		PanelVCard.add(_TextFieldSurname);
		_TextFieldSurname.setColumns(10);
		_TextFieldSurname.setDocument(new JTextFieldLimit(20, this));
		_TextFieldSurname.setText("Surname");
		
		lblRequiredFields_3 = new JLabel("* Required fields");
		lblRequiredFields_3.setFont(new Font("Verdana", Font.PLAIN, 10));
		lblRequiredFields_3.setBounds(10, 117, 131, 14);
		PanelVCard.add(lblRequiredFields_3);
		
		_ButtonEncodeVcard = new JButton("Encode vCard");
		_ButtonEncodeVcard.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				try {
					encodeVCard();
				} catch (Exception e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
			}
		});
		_ButtonEncodeVcard.setEnabled(false);
		_ButtonEncodeVcard.setFont(new Font("Verdana", Font.PLAIN, 10));
		_ButtonEncodeVcard.setBounds(237, 108, 160, 23);
		PanelVCard.add(_ButtonEncodeVcard);
		
		lblNdefMessage = new JLabel("NDEF Message:");
		lblNdefMessage.setFont(new Font("Verdana", Font.PLAIN, 10));
		lblNdefMessage.setBounds(10, 288, 106, 14);
		_contentPane.add(lblNdefMessage);
		
		JScrollPane scrollPane_1 = new JScrollPane();
		scrollPane_1.setBounds(10, 305, 412, 102);
		_contentPane.add(scrollPane_1);
		
		_TextAreaNdefMessage = new JTextArea();
		_TextAreaNdefMessage.setLineWrap(true);
		_TextAreaNdefMessage.setEditable(false);
		_TextAreaNdefMessage.setFont(new Font("Verdana", Font.PLAIN, 10));
		scrollPane_1.setViewportView(_TextAreaNdefMessage);
		
		_ButtonClear = new JButton("Clear");
		_ButtonClear.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				_TextAreaApduLogs.setText("");
			}
		});
		_ButtonClear.setFont(new Font("Verdana", Font.PLAIN, 10));
		_ButtonClear.setBounds(430, 418, 130, 23);
		_contentPane.add(_ButtonClear);
		
		_ButtonReset = new JButton("Reset");
		_ButtonReset.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				resetFields();
			}
		});
		_ButtonReset.setFont(new Font("Verdana", Font.PLAIN, 10));
		_ButtonReset.setBounds(565, 418, 130, 23);
		_contentPane.add(_ButtonReset);
		
		_ButtonQuit = new JButton("Quit");
		_ButtonQuit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				dispose();	
			}
		});
		_ButtonQuit.setFont(new Font("Verdana", Font.PLAIN, 10));
		_ButtonQuit.setBounds(700, 418, 130, 23);
		_contentPane.add(_ButtonQuit);
		
		_ButtonWriteToCard = new JButton("Write to Card");
		_ButtonWriteToCard.setBounds(10, 418, 130, 23);
		_contentPane.add(_ButtonWriteToCard);
		_ButtonWriteToCard.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
					writeToCard();
			}
		});
		_ButtonWriteToCard.setEnabled(false);
		_ButtonWriteToCard.setFont(new Font("Verdana", Font.PLAIN, 10));
		
		_ButtonReadFromCard = new JButton("Read from Card");
		_ButtonReadFromCard.setBounds(292, 418, 130, 23);
		_contentPane.add(_ButtonReadFromCard);
		_ButtonReadFromCard.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				readFromCard();				
			}
		});
		_ButtonReadFromCard.setEnabled(false);
		_ButtonReadFromCard.setFont(new Font("Verdana", Font.PLAIN, 10));
		
		addMsgToLog("Program Ready", 2);
	}
	
	private void initialize()
	{
		String[] readerList = null;
		
	    try
	    {
		    readerList = _readerFunctions.getPcscConnection().listTerminals();					
			if (readerList.length == 0)
			{
				addMsgToLog("No PC/SC reader detected", 1);
				return;
			}
			
			_ComboBoxReaderList.removeAllItems();
				
			for (int i = 0; i < readerList.length; i++)
			{
				if (!readerList.equals(""))	
					_ComboBoxReaderList.addItem(readerList[i]);
				else
					break;
			}
			
			_ComboBoxReaderList.setSelectedIndex(0);
			_ButtonConnect.setEnabled(true);
			_ButtonReset.setEnabled(true);
	    }
	    catch (Exception ex)
	    {
	    	addMsgToLog("No Reader Available", 1);
		    showErrorMessage("No Reader Available");
	    }	
	}
	
	private void connect()
	{
		byte[] atr = null;
		CardSelector cardSelector;
		String readerConnected;
				
		try
		{
			if(_readerFunctions.getPcscConnection().isConnectionActive())	
				_readerFunctions.getPcscConnection().disconnect();
			
			readerConnected = (String)_ComboBoxReaderList.getSelectedItem();
			
			_readerFunctions.getPcscConnection().connect(readerConnected, "*");				
			addMsgToLog("Successfully connected to " + readerConnected, 2);	
			
			// Initialize MifareClassic and CardSelector class
			_ntag203 = new Ntag203(_readerFunctions.getPcscConnection());
			cardSelector = new CardSelector(_readerFunctions.getPcscConnection());
			atr = _readerFunctions.getPcscConnection().getAtr();
		    
            _cardName = cardSelector.readCardType(atr);
                            
            if(!_cardName.equals("NTAG203"))
            {
            	showErrorMessage("Card not supported.\r\nPlease present NTAG203 card.");
            	// disable controls  
                enableControls(false);
            	return;
            }
            
            addMsgToLog("Chip Type: " + _cardName, 2);
            
            // enable controls  
            enableControls(true);
			
		}
		catch (Exception ex)
		{
			addMsgToLog(ex.getMessage().toString(), 2);
			showErrorMessage(ex.getMessage().toString());
		}
	}
	
	//////////////////////////////////////////////////////////////
	// NDEF Conversion
	//////////////////////////////////////////////////////////////
	public void encodeToNdefSmartPoster() throws Exception
	{
		ACTION_RECORD actionRecord = null;
		String title = "";
		byte[] buffer;
		int excessLength = 0;
		
		if(_TextFieldSmartPosterUri.getText().trim().equals(""))
		{
			showErrorMessage("Please key-in Smart Poster URI.");
			_TextFieldSmartPosterUri.grabFocus();
			return;
		}
		
		if(_ComboBoxSmartPosterUri.getSelectedItem().equals("tel:"))
		{
			_pattern = Pattern.compile(_telNumberPattern);
			_matcher = _pattern.matcher(_TextFieldSmartPosterUri.getText().trim());
			
			if(!_matcher.find())
			{
				showErrorMessage("Please key-in correct tel number.");
				_TextFieldSmartPosterUri.grabFocus();
				return;
			}
		}
		
		switch (_ComboBoxSmartPosterAction.getSelectedIndex())
		{
			case 1: actionRecord = ACTION_RECORD.DO_THE_ACTION; break;
			case 2: actionRecord = ACTION_RECORD.SAVE_FOR_LATER; break;
			case 3: actionRecord = ACTION_RECORD.OPEN_FOR_EDITING; break;
			default: actionRecord = null; break;
		}
		
		try
		{
			title = (String)_ComboBoxSmartPosterTitle.getSelectedItem();
			
			buffer = EncodeNdef.encodeSmartPoster(title,
											 _TextFieldSmartPosterTitle.getText(),
											 (byte)(URI_IDENTIFIER_CODE.values()[_ComboBoxSmartPosterUri.getSelectedIndex()]).getURIIdentifierCode(),
											 _TextFieldSmartPosterUri.getText(),
											 actionRecord);
			
			if (buffer == null || buffer.length < 1)
			{
				_TextAreaNdefMessage.setText("");
				return;
			}
			
			excessLength = buffer.length - _maximumInputLength;
			
			if(buffer.length > _maximumInputLength)
			{
				showErrorMessage("Input characters exceeds maximum input length. Please remove " + Integer.toString(excessLength) + " characters from Title or URI.");
				return;
			}
			
			_TextAreaNdefMessage.setText(Helper.byteAsString(buffer, false));
		}
		catch (Exception ex)
		{
			addMsgToLog(ex.getMessage() + "\r\n", 2);
		}
	}
	
	public void encodeToNdefMessage() throws Exception
	{
		NdefMessage ndefMessage;
		NdefRecord tmpRecord;
		byte statusByte = 0;
		byte[] buffer;
		int excessLength = 0;
		
		if(_TextAreaTextMessage.getText().trim().equals(""))
		{
			showErrorMessage("Please key-in Message.");
			_TextAreaTextMessage.grabFocus();
			return;
		}
		
		if (_RadioButtonUtf16.isSelected())
			statusByte = (byte)0x80;
		
		// Length of the Language (ISO/IANA language code)
		// ex. "en-US"
		statusByte |= (byte)_ComboBoxTextLanguage.getSelectedItem().toString().length();
		
		tmpRecord = new NdefRecord(new Ndef.NdefRecordType(TYPE_NAME_FORMAT.NFC_FORM_WELL_KNOWN_TYPE, "T"));
		tmpRecord.setMessageBegin(true);
		tmpRecord.setMessageEnd(true);
		tmpRecord.appendPayload(_TextAreaTextMessage.getText());
		
		tmpRecord.setPayLoad(new byte[] { statusByte });
		tmpRecord.setPayLoad(Helper.appendArrays(tmpRecord.getPayLoad(), _ComboBoxTextLanguage.getSelectedItem().toString().getBytes("ASCII")));
		tmpRecord.setPayLoad(Helper.appendArrays(tmpRecord.getPayLoad(), _TextAreaTextMessage.getText().getBytes("ASCII")));
		
		ndefMessage = new NdefMessage();
		ndefMessage.appendRecord(tmpRecord);
		
		buffer = ndefMessage.toByteArray();
		
		excessLength = buffer.length - _maximumInputLength;
		
		if(buffer.length > _maximumInputLength)
		{
			showErrorMessage("Input characters exceeds maximum input length. Please remove " + Integer.toString(excessLength) + " characters from Message.");
			return;
		}
		
		_TextAreaNdefMessage.setText(Helper.byteAsString(buffer, false));
	}
	
	public void encodeToNdefUrl() throws Exception
	{
		NdefMessage ndefMessage;
		NdefRecord tmpRecord;
		byte[] buffer;
		byte urlPrefix = 0x00;
		int excessLength = 0;
		
		if(_TextFieldUrlUri.getText().trim().equals(""))
		{
			showErrorMessage("Please key-in URI.");
			_TextFieldUrlUri.grabFocus();
			return;
		}
		
		if(_ComboBoxUriPrefix.getSelectedItem().equals("tel:"))
		{
			_pattern = Pattern.compile(_telNumberPattern);
			_matcher = _pattern.matcher(_TextFieldUrlUri.getText().trim());
			
			if(!_matcher.find())
			{
				showErrorMessage("Please key-in correct tel number.");
				_TextFieldUrlUri.grabFocus();
				return;
			}
		}
		
		// Encode URI
		tmpRecord = new NdefRecord(new Ndef.NdefRecordType(TYPE_NAME_FORMAT.NFC_FORM_WELL_KNOWN_TYPE, "U"));
		
		tmpRecord.setMessageBegin(true);
		tmpRecord.setMessageEnd(true);
		
		// URI Prefix
		// See URI Record Type Definition 
		// http://www.nfc-forum.org
		if(_ComboBoxUriPrefix.getSelectedIndex() > 0)
			urlPrefix = (byte)(URI_IDENTIFIER_CODE.values()[_ComboBoxUriPrefix.getSelectedIndex()]).getURIIdentifierCode();
		
		tmpRecord.appendPayload(urlPrefix);
		tmpRecord.appendPayload(_TextFieldUrlUri.getText());
		
		ndefMessage = new NdefMessage();
		ndefMessage.appendRecord(tmpRecord);
		
		buffer = ndefMessage.toByteArray();
		
		excessLength = buffer.length - _maximumInputLength;
		
		if(buffer.length > _maximumInputLength)
		{
			showErrorMessage("Input characters exceeds maximum input length. Please remove " + Integer.toString(excessLength) + " characters from URI.");
			return;
		}
		
		_TextAreaNdefMessage.setText(Helper.byteAsString(buffer, false));
	}
	
	public void encodeVCard() throws Exception
	{
		String formattedString = "BEGIN:VCARD\r\nVERSION:3.0";
		byte[] uBuffer;
		byte[] buffer;
		NdefRecord ndef;
		NdefMessage ndefMessage;
		int excessLength = 0;
		IdentificationType identificationType;
		
		if(_TextFieldGivenName.getText().trim().equals(""))
		{
			showErrorMessage("Please key-in Given Name.");
			_TextFieldGivenName.grabFocus();
			return;
		}
		
		_pattern = Pattern.compile(_namePattern);
		_matcher = _pattern.matcher(_TextFieldGivenName.getText().trim());
		
		if(!_matcher.find())
		{
			showErrorMessage("Please key-in correct Given Name.");
			_TextFieldGivenName.grabFocus();
			return;
		}
		
		if(!_TextFieldMiddleName.getText().trim().equals(""))
		{
			_matcher = _pattern.matcher(_TextFieldMiddleName.getText().trim());
			
			if(!_matcher.find())
			{
				showErrorMessage("Please key-in correct Middle Name.");
				_TextFieldMiddleName.grabFocus();
				return;
			}
		}
		
		if(_TextFieldSurname.getText().trim().equals(""))
		{
			showErrorMessage("Please key-in Surname.");
			_TextFieldSurname.grabFocus();
			return;
		}
		
		_matcher = _pattern.matcher(_TextFieldSurname.getText().trim());
		
		if(!_matcher.find())
		{
			showErrorMessage("Please key-in correct Surname.");
			_TextFieldSurname.grabFocus();
			return;
		}
		
		identificationType = new IdentificationType();
		identificationType.setGivenName(_TextFieldGivenName.getText());
		identificationType.setFamilyName(_TextFieldSurname.getText());
		identificationType.setAdditionalName(_TextFieldMiddleName.getText());
		
		formattedString += "\r\n" + identificationType.getFormattedString();
		
		formattedString += "\r\nEND:VCARD\r\n\r\n";
		
		uBuffer = formattedString.getBytes("ASCII");
		
		ndef = new Ndef.NdefRecord(new Ndef.NdefRecordType(TYPE_NAME_FORMAT.MEDIA_TYPE, "text/x-vCard"));
		ndef.appendPayload(uBuffer);
		
		ndefMessage = new NdefMessage();
		ndefMessage.appendRecord(ndef);
		
		buffer = ndefMessage.toByteArray();
		
		excessLength = buffer.length - _maximumInputLength;
		
		if(buffer.length > _maximumInputLength)
		{
			showErrorMessage("Input characters exceeds maximum input length. Please remove " + Integer.toString(excessLength) + " characters from Name.");
			return;
		}
		
		addTitleToLog(formattedString);
		_TextAreaNdefMessage.setText(Helper.byteAsString(ndefMessage.toByteArray(), false));
	}
	
	private void writeToCard()
	{
		byte[] dataToWrite = new byte[0];
        byte[] arrLength;
        byte[] buffer;
        byte[] tmpArray;
        byte numberOfWritesNeeded = 0;
        byte blockNumber = 0x04;
        int dataLength;
        int lenToCopy = 4;
        final int DATA_LENGTH = 255;
        final byte BLOCK_DATA_LENGTH = 4;
        final double BLOCK_LENGTH = 4.00;
        
        try
        {
        	if (_TextAreaNdefMessage.getText().trim().equals(""))
            {
                showErrorMessage("There is no NDEF message to write. \r\nPlease encode NDEF message first.");
                return;
            }

            dataToWrite = Helper.stringToByteArray(_TextAreaNdefMessage.getText());                                    

            //Form the data to be written to the card
            buffer = new byte[0];
            if (dataToWrite.length < DATA_LENGTH)
            {
                //If data is less than or equal to 254 (0xFE) use one byte format
                //Refere to NXP Type NTAG203 Operation document
                //http://www.nxp.com/
                buffer = Helper.appendArrays(buffer, new byte[] { (byte)NfcForum.TAGs.NDEF_MESSAGE_TLV.getTags(), (byte)dataToWrite.length });
            }
            else
            {
                //If data is greater than 254 (0xFE) use 3 bytes format
                dataLength = dataToWrite.length;

                //In 3 bytes format the last 2 bytes is for 00FE - FFFF                    
                dataLength -= (DATA_LENGTH - 1);
                
                arrLength = new byte[3];

                //Indicates that two more bytes folow
                arrLength[0] = (byte)0xFF;
                arrLength[1] = (byte)((dataLength & 0xFF00) >> 8);
                arrLength[2] = (byte)(dataLength & 0xFF);

                //NDEF Message TAG
                buffer = Helper.appendArrays(buffer, (byte)NfcForum.TAGs.NDEF_MESSAGE_TLV.getTags());

                //length
                buffer = Helper.appendArrays(buffer, arrLength);

            }

            buffer = Helper.appendArrays(buffer, dataToWrite);            

            //Append Terminator TLV
            buffer = Helper.appendArrays(buffer, (byte)NfcForum.TAGs.TERMINATOR_TLV.getTags());            

            //Write the data to NTAG203
            //Start from NFC Form Sector 1 Block 0

            addTitleToLog("Write NDEF Message To Card");

            //Get mumber of blocks needed
            numberOfWritesNeeded = (byte)((double)Math.ceil(buffer.length / BLOCK_LENGTH));
            blockNumber = 0x04; //Start to write at sector 1 block 1 (Physical Block 0x04)

            for (int indx = 0; indx < numberOfWritesNeeded; indx++, blockNumber += (BLOCK_DATA_LENGTH/4))
            {
            	tmpArray = new byte[BLOCK_DATA_LENGTH];
                lenToCopy = BLOCK_DATA_LENGTH;

                //Check if remaining data to be copied is less than 16
                if ((buffer.length - (indx * BLOCK_DATA_LENGTH)) > BLOCK_DATA_LENGTH)
                    lenToCopy = BLOCK_DATA_LENGTH;
                else
                    lenToCopy = buffer.length - (indx * BLOCK_DATA_LENGTH);

                System.arraycopy(buffer, (indx * BLOCK_DATA_LENGTH), tmpArray, 0, lenToCopy);

                addTitleToLog("Write in block " + Integer.toString(blockNumber));

                _ntag203.updateBinary(blockNumber, tmpArray, BLOCK_DATA_LENGTH);

            }
        }        
        catch (Exception ex)
        {
        	if(ex.getCause() != null)
        	{
	    		addMsgToLog(ex.getCause().getMessage().toString(), 2);        	
	        	showErrorMessage(ex.getCause().getMessage().toString());
        	}
        	else
        	{
        		addMsgToLog(ex.getMessage().toString(), 2);        	
	        	showErrorMessage(ex.getMessage().toString());
        	}
        }
        
	}
	
	private void readFromCard()
	{
		byte blockNumber = 0x04; //starting block
        byte[] buffer = new byte[0];
        byte[] tmpArray;
        byte[] arrayBuffer;
        byte[] payload;             
        int endIndex = 0;
        int temporaryLen;
        int temporaryPayLoadLen;
        int numberOfBlocks = 0;
        String payLoadTypeNameStr = "";
        String delimeter = "\r\n";
        String[] vCard;
        String[] vCardInfo;
        NdefMessage ndefMessage;
        NdefMessage temporaryNdefMessage;
        List<NdefRecord> ndefRecords;
        NdefRecord temporaryNdefRecord;
        final double BLOCK_LENGTH = 16.00;
        final int BLOCK_COUNT = 4;
        final int BLOCK_DATA_LENGTH = 16;
        final int THREE_BYTE_MESSAGE_TAG = 4;
        final int TWO_BYTE_MESSAGE_TAG = 2;
        final int THREE_BYTE_FORMAT_ADDTL = 5;
        final int NON_THREE_BYTE_FORMAT_ADDTL = 3;

         try
         {                          
             _TextAreaNdefMessage.setText("");

             buffer = new byte[0];

             tmpArray = _ntag203.readBinary(blockNumber, (byte)0x10);
             buffer = Helper.appendArrays(buffer, tmpArray);
             
             System.out.println(Helper.byteAsString(buffer, true));

             if(buffer[1] == (byte)0xFF)
            	 numberOfBlocks = (byte)((double)Math.ceil((buffer[1] + THREE_BYTE_FORMAT_ADDTL) / BLOCK_LENGTH));
             else
            	 numberOfBlocks = (byte)((double)Math.ceil((buffer[1] + NON_THREE_BYTE_FORMAT_ADDTL) / BLOCK_LENGTH));
            	 
             blockNumber += BLOCK_COUNT;

             for (int indx = 0; indx < numberOfBlocks - 1; indx++, blockNumber += BLOCK_COUNT)
             {
                 tmpArray = new byte[BLOCK_DATA_LENGTH];

                 addTitleToLog("Read block " + Integer.toString(blockNumber));

                 tmpArray = _ntag203.readBinary(blockNumber, (byte)0x10);
                 buffer = Helper.appendArrays(buffer, tmpArray);
             }
             
             //Check if NDEF Message TAG is present
             if (buffer[0] != (byte)(NfcForum.TAGs.NDEF_MESSAGE_TLV.getTags()))
                 throw new Exception("NDEF Message TAG is not found");

             //Find the Terminator TLV
             for (endIndex = 0; endIndex < buffer.length; endIndex++)
             {
                 if (buffer[endIndex] == TERMINATOR_TLV)
                     break;

                 if ((endIndex + 1) >= buffer.length)
                     throw new Exception("Invalid Card.\r\nTerminator Tag is not found");
             }

             arrayBuffer = new byte[endIndex];
             System.arraycopy(buffer, 0, arrayBuffer, 0, arrayBuffer.length);

             //Remove NDEF Message Tag
             //Check if it uses one byte or 3 byte format for the length
             if (arrayBuffer[1] == 0xFF)
             {
                 //Uses 3 bytes format
                 tmpArray = new byte[arrayBuffer.length - THREE_BYTE_MESSAGE_TAG];
                 System.arraycopy(arrayBuffer, THREE_BYTE_MESSAGE_TAG, tmpArray, 0, tmpArray.length);
             }
             else
             {
                 tmpArray = new byte[arrayBuffer.length - TWO_BYTE_MESSAGE_TAG];
                 System.arraycopy(arrayBuffer, TWO_BYTE_MESSAGE_TAG, tmpArray, 0, tmpArray.length);
             }
                                            

             _TextAreaNdefMessage.setText(Helper.byteAsString(tmpArray, false));

             ndefMessage = NdefRecord.getNestedNdefMessage(0, tmpArray);

             if (ndefMessage.getNumberOfRecords() < 0)
             {
                 showErrorMessage("No record to display");
                 return;
             }

             ndefRecords = ndefMessage.getRecords();

             payLoadTypeNameStr = ndefRecords.get(0).getRecordType().getTypeName();

             if (payLoadTypeNameStr.equals("Sp")) 
             {
                 temporaryNdefMessage = ndefRecords.get(0).getNestedNdefMessage();

                 if (temporaryNdefMessage.getNumberOfRecords() < 1)
                     throw new Exception("Invalid Smart Poster NDEF Message");

                 //Get URI Record (Mandatory)
                 temporaryNdefRecord = temporaryNdefMessage.getRecord(new Ndef.NdefRecordType(TYPE_NAME_FORMAT.NFC_FORM_WELL_KNOWN_TYPE, "U"));
                 if (temporaryNdefRecord == null)
                 {
                     showErrorMessage("Mandatory URI Record is not found");
                     return;
                 }

                 _ComboBoxSmartPosterUri.setSelectedIndex(temporaryNdefRecord.getPayLoad()[0]);
                 payload = new byte[temporaryNdefRecord.getPayLoad().length - 1];
                 System.arraycopy(temporaryNdefRecord.getPayLoad(), 1, payload, 0, payload.length);
                 _TextFieldSmartPosterUri.setText(Helper.byteArrayToString(payload, payload.length));                

                 //Get Action Record
                 _ComboBoxSmartPosterAction.setSelectedIndex(0);
                 temporaryNdefRecord = temporaryNdefMessage.getRecord(new Ndef.NdefRecordType(TYPE_NAME_FORMAT.NFC_FORM_WELL_KNOWN_TYPE, "act"));
                 if (temporaryNdefRecord != null)
                 {
                     switch ((ReferenceData.ACTION_RECORD.values()[temporaryNdefRecord.getPayLoad()[0]]))
                     {
                         case DO_THE_ACTION:
                         {
                             _ComboBoxSmartPosterAction.setSelectedIndex(1);
                             break;
                         }
                         case SAVE_FOR_LATER:
                         {
                        	 _ComboBoxSmartPosterAction.setSelectedIndex(2);
                             break;
                         }
                         case OPEN_FOR_EDITING:
                         {
                        	 _ComboBoxSmartPosterAction.setSelectedIndex(3);
                             break;
                         }
                         default:
                         {
                        	 _ComboBoxSmartPosterAction.setSelectedIndex(0);
                             break;
                         }
                     }
                 }
                 else
                 {
                	 _ComboBoxSmartPosterAction.setSelectedIndex(0);
                 }

                 //Get Text Record
                 temporaryNdefRecord = temporaryNdefMessage.getRecord(new Ndef.NdefRecordType(TYPE_NAME_FORMAT.NFC_FORM_WELL_KNOWN_TYPE, "T"));
                 if (temporaryNdefRecord != null)
                 {
                     temporaryLen = (temporaryNdefRecord.getPayLoad()[0] & (byte)0x1F);
                     payload = new byte[temporaryLen]; 
                     System.arraycopy(temporaryNdefRecord.getPayLoad(), 1, payload, 0, temporaryLen);
                     
                     if(temporaryLen == 0)
                    	 _ComboBoxSmartPosterTitle.setSelectedIndex(0);
                     else
                    	 _ComboBoxSmartPosterTitle.setSelectedItem(Helper.byteArrayToString(payload, temporaryLen));

                     temporaryPayLoadLen = temporaryNdefRecord.getPayLoad().length - (temporaryLen - 1);
                     payload = new byte[temporaryPayLoadLen];
                     System.arraycopy(temporaryNdefRecord.getPayLoad(), temporaryLen + 1, payload, 0, temporaryNdefRecord.getPayLoad().length - (temporaryLen + 1));
                     _TextFieldSmartPosterTitle.setText(Helper.byteArrayToString(payload, temporaryPayLoadLen));

                 }

                 showInformationMessage("Smart Poster Message Found");
                 _TabbedPaneNfcType.setSelectedIndex(0);

             }
             else if (payLoadTypeNameStr.equals("T"))
             {
                 //Get Text Record
                 temporaryNdefRecord = ndefRecords.get(0);

                 if ((temporaryNdefRecord.getPayLoad()[0] & 0x80) != 0x00)
                 {
                     _RadioButtonUtf16.setSelected(true);
                     _RadioButtonUtf8.setSelected(false);
                 }
                 else
                 {
                     _RadioButtonUtf8.setSelected(true);
                     _RadioButtonUtf16.setSelected(false);
                 }

                 temporaryLen = temporaryNdefRecord.getPayLoad()[0] & 0x1F;
                 payload = new byte[temporaryLen];
                 System.arraycopy(temporaryNdefRecord.getPayLoad(), 1, payload, 0, temporaryLen);
                 _ComboBoxTextLanguage.setSelectedItem(Helper.byteArrayToString(payload, temporaryLen));

                 temporaryPayLoadLen = temporaryNdefRecord.getPayLoad().length - (temporaryLen - 1);
                 payload = new byte[temporaryPayLoadLen];
                 System.arraycopy(temporaryNdefRecord.getPayLoad(), temporaryLen + 1, payload, 0, temporaryNdefRecord.getPayLoad().length - (temporaryLen + 1));
                 _TextAreaTextMessage.setText(Helper.byteArrayToString(payload, temporaryPayLoadLen));

                 showInformationMessage("Text Message Found");
                 _TabbedPaneNfcType.setSelectedIndex(2);                 

             }
             else if (payLoadTypeNameStr.equals("U"))
             {
                 temporaryNdefRecord = ndefRecords.get(0);
                 _ComboBoxUriPrefix.setSelectedIndex(temporaryNdefRecord.getPayLoad()[0]);
                 payload = new byte[temporaryNdefRecord.getPayLoad().length - 1];
                 System.arraycopy(temporaryNdefRecord.getPayLoad(), 1, payload, 0, payload.length);
                 _TextFieldUrlUri.setText(Helper.byteArrayToString(payload, payload.length));

                 showInformationMessage("URI Message Found");
                 _TabbedPaneNfcType.setSelectedIndex(1);  
             }
             else if (payLoadTypeNameStr.equals("text/x-vCard"))
             {
                 temporaryNdefRecord = ndefRecords.get(0);
                 vCard = Helper.byteArrayToString(temporaryNdefRecord.getPayLoad(), temporaryNdefRecord.getPayLoad().length).split(delimeter);
                 vCardInfo = vCard[2].split(";");
                 _TextFieldSurname.setText(vCardInfo[0].substring(2, vCardInfo[0].length()));
                 _TextFieldGivenName.setText(vCardInfo[1]);
                 _TextFieldMiddleName.setText(vCardInfo[2]);

                 showInformationMessage("vCard Message Found");
                 _TabbedPaneNfcType.setSelectedIndex(3); 
             }
             else
             {
                 showErrorMessage("Unsupported payload type");
             }
         }         
         catch (Exception ex)
         {
	         if(ex.getCause() != null)
	         {
	 	    	 addMsgToLog(ex.getCause().getMessage().toString(), 2);        	
	 	         showErrorMessage(ex.getCause().getMessage().toString());
	         }
	         else
	         {
	         	 addMsgToLog(ex.getMessage().toString(), 2);        	
	 	         showErrorMessage(ex.getMessage().toString());
	         }
         }
	}
	
	void resetFields()
    {
        _ComboBoxReaderList.removeAllItems();

        _ComboBoxSmartPosterAction.setSelectedIndex(0);
        _ComboBoxSmartPosterUri.setSelectedIndex(0);
        _ComboBoxSmartPosterTitle.setSelectedIndex(0);
        _ComboBoxTextLanguage.setSelectedIndex(0);
        _ComboBoxUriPrefix.setSelectedIndex(0);

        _RadioButtonUtf8.setSelected(true);
        _RadioButtonUtf16.setSelected(false);
        
        enableControls(false);

        _ButtonConnect.setEnabled(false);
        
        _TabbedPaneNfcType.setSelectedIndex(0);    

        _TextFieldSmartPosterTitle.setText("Advanced Card Systems Ltd.");
        _TextFieldSmartPosterUri.setText("http://www.acs.com.hk");
        _TextFieldUrlUri.setText("http://www.acs.com.hk");
        _TextAreaTextMessage.setText("Hello World!");
        _TextFieldGivenName.setText("Given Name");
        _TextFieldMiddleName.setText("Middle Name");
        _TextFieldSurname.setText("Surname");

        _TextAreaNdefMessage.setText("");
        _TextAreaApduLogs.setText("");
        addMsgToLog("Program Ready", 2);
    }
	
	public void addMsgToLog(String prefixStr, byte[] buff, String postfixStr, int buffLen)
	{
		String tmpStr = "";

        if (buff.length < buffLen)
            return;

        tmpStr = null;

        //Convert each byte from buff to its string representation.
        for (int i = 0; i < buffLen; i++)
            tmpStr += String.format("{0:X2}", buff[i]) + " ";

        addMsgToLog(prefixStr + tmpStr + postfixStr, 1);
	}
	
	void addTitleToLog(String title)
	{
		_TextAreaApduLogs.setSelectedTextColor(Color.black);
		_TextAreaApduLogs.append("\r\n" + title + "\r\n");
	}
	
	void addMsgToLog(String msg, int msgType)
	{
		switch(msgType)
		{
			case 1: _TextAreaApduLogs.append(msg + "\r\n"); break;
			default: _TextAreaApduLogs.append(msg + "\r\n\n"); break;
		}
	}
	
	void showInformationMessage(String message)
	{
		JOptionPane.showMessageDialog(this, message, "Information", JOptionPane.INFORMATION_MESSAGE);
	}
	
	void showWarningMessage(String message)
	{
		JOptionPane.showMessageDialog(this, message, "Warning", JOptionPane.WARNING_MESSAGE);
	}
	
	void showErrorMessage(String message)
	{
		JOptionPane.showMessageDialog(this, message, "Error", JOptionPane.ERROR_MESSAGE);
	}
	
	void enableControls(boolean isEnabled)
	{
		// Smart Poster Tab
        _ComboBoxSmartPosterTitle.setEnabled(isEnabled);
        _ComboBoxSmartPosterUri.setEnabled(isEnabled);
        _ComboBoxSmartPosterAction.setEnabled(isEnabled);
        _TextFieldSmartPosterTitle.setEnabled(isEnabled);
        _TextFieldSmartPosterUri.setEnabled(isEnabled);
        _ButtonEncodeSmartPoster.setEnabled(isEnabled);
        
        // URL Tab
        _ComboBoxUriPrefix.setEnabled(isEnabled);
        _TextFieldUrlUri.setEnabled(isEnabled);
        _ButtonEncodeUrl.setEnabled(isEnabled);
        
        // Text Tab
        _ComboBoxTextLanguage.setEnabled(isEnabled);
        _RadioButtonUtf16.setEnabled(isEnabled);
        _RadioButtonUtf8.setEnabled(isEnabled);
        _TextAreaTextMessage.setEnabled(isEnabled);
        _ButtonEncodeText.setEnabled(isEnabled);
        
        // vCard Tab
        _TextFieldGivenName.setEnabled(isEnabled);
        _TextFieldMiddleName.setEnabled(isEnabled);
        _TextFieldSurname.setEnabled(isEnabled);
        _ButtonEncodeVcard.setEnabled(isEnabled);
        
        _ButtonWriteToCard.setEnabled(isEnabled);
        _ButtonReadFromCard.setEnabled(isEnabled);
	}

	public void keyTyped(KeyEvent ke) 
	{  		
  		char charInput = ke.getKeyChar();
  		//Check valid characters
  		if(_TextFieldGivenName.isFocusOwner() || _TextFieldMiddleName.isFocusOwner() || _TextFieldSurname.isFocusOwner())
  		{	  			
  			if (!(Character.isLetter(charInput)
                || (charInput == KeyEvent.VK_BACK_SPACE)
                || (charInput == KeyEvent.VK_DELETE)
                || (charInput == KeyEvent.VK_PERIOD) 
                || (charInput == KeyEvent.VK_MINUS)
                || (charInput == KeyEvent.VK_SPACE))) {
                ke.consume();
            }
  		}  	
  		
  		if(_TextFieldSmartPosterUri.isFocusOwner())
  		{
  			if(_ComboBoxSmartPosterUri.getSelectedItem().equals("tel:"))
  			{
  				if (!(Character.isDigit(charInput)
  		                || (charInput == KeyEvent.VK_BACK_SPACE)
  		                || (charInput == KeyEvent.VK_DELETE)  		                
  		                || (charInput == KeyEvent.VK_MINUS) 
  		                || (charInput == '+')
  		                || (charInput == '(')
  		                || (charInput == ')'))) {
  		                ke.consume();
  		            }
  			}
  		}
  		
  		if(_TextFieldUrlUri.isFocusOwner())
  		{
  			if(_ComboBoxUriPrefix.getSelectedItem().equals("tel:"))
  			{
  				if (!(Character.isDigit(charInput)
  		                || (charInput == KeyEvent.VK_BACK_SPACE)
  		                || (charInput == KeyEvent.VK_DELETE)  		                
  		                || (charInput == KeyEvent.VK_MINUS) 
  		                || (charInput == '+')
  		                || (charInput == '(')
  		                || (charInput == ')'))) {
  		                ke.consume();
  		            }
  			}
  		}
	}
	
	@Override
	public void keyPressed(KeyEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void keyReleased(KeyEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void actionPerformed(ActionEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	public void onSendCommand(ReaderEvents.TransmitApduEventArg event) 
	{
		addMsgToLog("<< " + event.getAsString(true), 1);		
	}

	public void onReceiveCommand(ReaderEvents.TransmitApduEventArg event) 
	{
		addMsgToLog(">> " + event.getAsString(true), 1);		
	}
}
