package NDEF;

import java.util.ArrayList;
import java.util.List;

import ReferenceData.ACTION_RECORD;
import ReferenceData.URI_IDENTIFIER_CODE;
import ReferenceData.TYPE_NAME_FORMAT;
import NDEF.Helper;

public class Ndef {
	public static class NdefMessage
	{
		List<NdefRecord> _ndefRecords = new ArrayList<NdefRecord>();

        public void appendRecord(NdefRecord record)
        {
        	_ndefRecords.add(record);
        }
        
        public void insertRecord(int index, NdefRecord record)
        {
        	_ndefRecords.add(index, record);
        }

        public List<NdefRecord> getRecords()
        {
            return _ndefRecords;
        }

        public NdefRecord getRecord(NdefRecordType recordType)
        {
            for (NdefRecord records : _ndefRecords)
            {
                if (records.getRecordType().getTypeName().equals(recordType.getTypeName())  &&
                		records.getRecordType().getTypeNameFormat() == recordType.getTypeNameFormat())
                {
                    return records;
                }
            }

            return null;
        }

        public int getNumberOfRecords()
        {
            return _ndefRecords.size();
        }

        public byte[] toByteArray() throws Exception
        {
            byte[] buffer = new byte[0];
            byte[] tmpArray;
            int idx = 0;

            for (int i = 0; i < _ndefRecords.size(); i++)
            {
            	idx = buffer.length;

                if (i == 0)
                	_ndefRecords.get(i).setMessageBegin(true);

                if ((i + 1) == _ndefRecords.size())
                	_ndefRecords.get(i).setMessageEnd(true);

                tmpArray = _ndefRecords.get(i).encodeToNDEF();

                // Resize destination array to accommodate new record
                buffer = (byte[]) Helper.resizeArray(buffer, buffer.length + tmpArray.length);

                // Copy new ndef record to byte array
                System.arraycopy(tmpArray, 0, buffer, idx, tmpArray.length);
            }

            return buffer;
        } 
	}
	
	public static class NdefRecordType
	{
		private TYPE_NAME_FORMAT _typeNameFormat;
		private String _typeName;
		
		public TYPE_NAME_FORMAT getTypeNameFormat() { return _typeNameFormat; }
        public void setTypeNameFormat(TYPE_NAME_FORMAT value) { this._typeNameFormat = value; }
        
		public String getTypeName() { return _typeName; }
        public void setTypeName(String value) throws Exception
        {
        	if(value.trim().length() > 255)
        		throw new Exception("Pay load type is too long");
        	this._typeName = value.trim();
        }
        
		public NdefRecordType(TYPE_NAME_FORMAT format, String name)
		{
			_typeNameFormat = format;
			_typeName = name;
		}
	}
	
	public static class NdefRecord
	{	
		private boolean			_isMessageBegin;
		private boolean			_isMessageEnd;
		private boolean			_isTerminatingRecordChunk;
		public  boolean			_isShortRecord;
		private NdefRecordType	_recordType;
		private byte[]			_bytePayLoad;		
		private byte[]			_messageID = new byte[0];
		public  String			_payLoadStr;
		
		public boolean getMessageBegin() { return _isMessageBegin; }
        public void setMessageBegin(boolean value) { this._isMessageBegin = value; }
        
        public boolean getMessageEnd() { return _isMessageEnd; }
        public void setMessageEnd(boolean value) { this._isMessageEnd = value; }
        
        public boolean getTerminatingRecordChunk() { return _isTerminatingRecordChunk; }
        public void setTerminatingRecordChunk(boolean value) { this._isTerminatingRecordChunk = value; }
		
        public boolean getShortRecord()
        {
        	if(_bytePayLoad.length < 256)
        		return true;
        	else
        		return false;
        }
        
        public NdefRecordType getRecordType() { return _recordType; }
        public void setRecordType(NdefRecordType value) { this._recordType = value; }
        
        public byte[] getPayLoad() { return _bytePayLoad; }
        public void setPayLoad(byte[] value) { this._bytePayLoad = value; }
        
        public String getPayloadStr()
        {
        	return new String(_bytePayLoad);
        }
        public void setPayloadStr(String sValue) throws Exception
        {
        	_bytePayLoad = sValue.getBytes("ASCII");
        }
        
        public byte[] getMessageID() { return _messageID; }
        public void setMessageID(byte[] value) { this._messageID = value; }
		
		public NdefRecord(NdefRecordType recordType)
        {
			_isMessageBegin = false;
			_isMessageEnd = false;
            _isTerminatingRecordChunk = true;
            _recordType = recordType;
        }
		
		public byte[] encodeToNDEF() throws Exception
        {
            byte ndefHeader = 0;
            byte[] ndef = new byte[0];

            // Set NDEF Header
            if (getMessageBegin())
            	ndefHeader |= 0x80;

            if (getMessageEnd())
            	ndefHeader |= 0x40;

            if (!getTerminatingRecordChunk())
            	ndefHeader |= 0x20;

            if (getPayLoad().length < 256)
            	ndefHeader |= 0x10;

            if (getMessageID().length > 0)
            	ndefHeader |= 0x08;

            ndefHeader |= (byte)getRecordType().getTypeNameFormat().getTypeNameFormat();

            ndef = Helper.appendArrays(ndef, ndefHeader);

            // Set Payload Type Length
            ndef = Helper.appendArrays(ndef, (byte)getRecordType().getTypeName().length());

            // Set Payload Length
            if (getPayLoad().length < 256)
            	ndef = Helper.appendArrays(ndef, (byte)(getPayLoad().length));
            else
            	ndef = Helper.appendArrays(ndef, Helper.intToByte(getPayLoad().length));

            // Set Message ID Length
            if (getMessageID() != null && getMessageID().length > 0)
            	ndef = Helper.appendArrays(ndef, (byte)(getMessageID().length));

            // Set Payload Type
            ndef = Helper.appendArrays(ndef, (getRecordType().getTypeName().trim().getBytes("ASCII")));

            // Set Message ID
            if (getMessageID() != null && getMessageID().length > 0)
            	ndef = Helper.appendArrays(ndef, getMessageID());            

            // Set Payload
            ndef =  Helper.appendArrays(ndef, getPayLoad());
            
            return ndef;
        }
		
		public void appendPayload(byte[] paramPayLoad)
        {
            int index = 0;

            if (getPayLoad() == null)
            	setPayLoad(new byte[0]);

            index = _bytePayLoad.length;

            setPayLoad((byte[])Helper.resizeArray(_bytePayLoad, _bytePayLoad.length + paramPayLoad.length));

            System.arraycopy(paramPayLoad, 0, getPayLoad(), index, _bytePayLoad.length);
        }
		
		public void appendPayload(String paramPayLoad) throws Exception
        {
            int index = 0;
            byte[] buffer;

            if (paramPayLoad == "")
                return;

            if (_bytePayLoad == null)
            	_bytePayLoad = new byte[0];

            index = _bytePayLoad.length;

            buffer = paramPayLoad.getBytes("ASCII");

            _bytePayLoad = (byte[])Helper.resizeArray(_bytePayLoad, _bytePayLoad.length + buffer.length);

            System.arraycopy(buffer, 0, _bytePayLoad, index, buffer.length);
        }
		
		public void appendPayload(byte paramPayLoad)
        {
            appendPayload(new byte[] { paramPayLoad });
        }

        public NdefMessage getNestedNdefMessage() throws Exception
        {
            if (getPayLoad() == null)
                throw new Exception("Payload is not yet been set");

            return getNestedNdefMessage(0, getPayLoad());
        }
        
        public static NdefMessage getNestedNdefMessage(int index, byte[] paramData) throws Exception
        {
            NdefMessage ndefMessage = new NdefMessage();
            NdefRecord ndefRecord;
            byte typeNameFormat = 0x00;
            byte typeNameLength = 0x00;
            byte idLength = 0x00;
            byte[] dataTemp2;
            byte[] dataTemp3;
            byte[] dataTemp4;
            byte[] dataTemp;

            int payloadLength = 0;
            int currentIndex = 0;

            String typeName = "";

            boolean isMessageBeginSet = false;
            boolean isMessageEndSet = false;
            boolean isIdLengthPresent = false;
            boolean isChunkFlagPresent = false;
            boolean isShortRecordSet = false;

            if (paramData.length <= index)
                throw new Exception("Invalid index");

            for (; index < paramData.length;)
            {
            	currentIndex = index;

                // Get Type Name Format
                // e.g. NFC Well Know Type = 0x01
                typeNameFormat = (byte)(paramData[currentIndex] & 0x07);

                if (typeNameFormat != 0x01 && typeNameFormat != 0x02)
                    throw new Exception("Type Name Format " + TYPE_NAME_FORMAT.getKeyNameTypeNameFormat(typeNameFormat).toString() + " is not supported.");
                
                // Check if Message Begin (Bit 7) is set 
                if ((paramData[currentIndex] & 0x80) != 0x00)
                    isMessageBeginSet = true;
                else
                    isMessageBeginSet = false;

                // Check if Message End (Bit 6) is set
                if ((paramData [currentIndex] & 0x40) != 0x00)
                    isMessageEndSet = true;
                else
                    isMessageEndSet = false;

                // Check if Chunk Flag (Bit 5) is set
                if ((paramData[currentIndex] & 0x20) != 0x00)
                    isChunkFlagPresent = true;
                else
                    isChunkFlagPresent = false;

                // Check if Short Record (bit 4) is set
                if ((paramData[currentIndex] & 0x10) != 0x00)
                    isShortRecordSet = true;
                else
                    isShortRecordSet = false;

                // Check if ID length is set
                if ((paramData[currentIndex] & 0x08) != 0x00)
                    isIdLengthPresent = true;
                else
                    isIdLengthPresent = false;

                currentIndex += 1;

                // Get the type length
                // Refer for Short Record section of NFC Data Exchange Format
                // Technical Specification for more information
                typeNameLength = paramData[currentIndex];
                currentIndex += 1;

                if (isShortRecordSet)
                {
                    // For Short Record payload, length is 1 byte
                    // Get Payload Length    
                	payloadLength = paramData[currentIndex];
                    currentIndex += 1;
                }
                else
                {
                    // For Non Short Record payload, length is 4 bytes
                	dataTemp = null;
                	System.arraycopy(paramData, currentIndex, dataTemp, 0, 4);
                	payloadLength = Helper.byteToInt(dataTemp);
                    currentIndex += 4;
                }

                if (isIdLengthPresent)
                {
                    //+1 to get ID Length
                	idLength = paramData[currentIndex];
                    currentIndex += 1;
                }

                // +1 to get Payload Type Name
                // Payload Type Name offset = currentIndex to currentIndex  + Type Name Length
                dataTemp2 = new byte[typeNameLength];
                System.arraycopy(paramData, currentIndex, dataTemp2, 0, typeNameLength);
                typeName = new String(dataTemp2); 
                currentIndex += typeNameLength;
                
                // Initialize new ndef record object
                ndefRecord = new NdefRecord(new NdefRecordType(TYPE_NAME_FORMAT.getKeyNameTypeNameFormat(typeNameFormat), typeName));
                ndefRecord.setMessageBegin(isMessageBeginSet);
                ndefRecord.setMessageEnd(isMessageEndSet);
                ndefRecord.setTerminatingRecordChunk(!isChunkFlagPresent);

                //If ID Length is present get record ID
                if (isIdLengthPresent)
                {
                	dataTemp3 = new byte[idLength];
                	System.arraycopy(paramData, currentIndex, dataTemp3, 0, idLength);
                    ndefRecord.setMessageID(dataTemp3);
                    currentIndex += idLength;
                }

                dataTemp4 = new byte[payloadLength];
            	System.arraycopy(paramData, currentIndex, dataTemp4, 0, payloadLength);
                ndefRecord.setPayLoad(dataTemp4);
                currentIndex += payloadLength;
                index = currentIndex;

                ndefMessage.appendRecord(ndefRecord);
            }

            return ndefMessage;
        }
	}
	
	public static class EncodeNdef 
	{
		public static byte[] encodeSmartPoster(String titleLanguage, String title, byte uriPrefix, String uri, ACTION_RECORD action) throws Exception
        {
            byte[] buffer = new byte[0];
            byte statusByte = 0x00;
            boolean useUtf8 = true;           

            NdefRecord tmpRecord;
            NdefMessage ndefMessage = new NdefMessage();

            // Encode Title
            if (title.trim() != "")
            {
                // If UTF 8 set bit 7 to 0
                // If UTF 16 set bit 7 to 1
                if(!useUtf8)
                    statusByte = (byte) 0x80;
                   
                // Length of the Language (ISO/IANA language code)
                // ex. "en-US"
                statusByte |= (byte)titleLanguage.length();

                tmpRecord = new NdefRecord(new NdefRecordType(TYPE_NAME_FORMAT.NFC_FORM_WELL_KNOWN_TYPE, "T"));
                tmpRecord.setMessageBegin(true);
                tmpRecord.appendPayload(statusByte);
                tmpRecord.appendPayload(titleLanguage);
                tmpRecord.appendPayload(title);

                ndefMessage.appendRecord(tmpRecord);
                                
            }

            // Encode Action            
            if (action != null)
            {
                tmpRecord = new NdefRecord(new NdefRecordType(TYPE_NAME_FORMAT.NFC_FORM_WELL_KNOWN_TYPE, "act"));
                tmpRecord.appendPayload((byte)action.getActionRecord());
                ndefMessage.appendRecord(tmpRecord);
                                
            }

            // Encode URI
            tmpRecord = new NdefRecord(new NdefRecordType(TYPE_NAME_FORMAT.NFC_FORM_WELL_KNOWN_TYPE, "U"));

            // URI Prefix
            // See URI Record Type Definition 
            // http://www.nfc-forum.org                                   
            tmpRecord.appendPayload(uriPrefix);
            tmpRecord.appendPayload(uri);

            ndefMessage.appendRecord(tmpRecord);                       

            buffer = ndefMessage.toByteArray();

            // Smart Poster Header
            tmpRecord = new NdefRecord(new NdefRecordType(TYPE_NAME_FORMAT.NFC_FORM_WELL_KNOWN_TYPE, "Sp"));
            tmpRecord.setMessageBegin(true);
            tmpRecord.setMessageEnd(true);
            tmpRecord.appendPayload(buffer);

            buffer = tmpRecord.encodeToNDEF();

            // Create NDEF Message for SmartPoster
            ndefMessage = new NdefMessage();
            ndefMessage.appendRecord(tmpRecord);

            // Encode NDEF Message
            buffer = ndefMessage.toByteArray();

            return buffer;
        }

        public static byte[] encodeNfcForumExternalType(String type, String paramPayload) throws Exception
        {
            byte[] buffer = new byte[0];

            NdefRecord tmpRecord;
            NdefMessage ndefMessage = new NdefMessage();

            tmpRecord = new NdefRecord(new NdefRecordType(TYPE_NAME_FORMAT.NFC_FORM_EXTERNAL_TYPE, type));
            tmpRecord.setMessageBegin(true);
            tmpRecord.setMessageEnd(true);
            tmpRecord.appendPayload(paramPayload);

            buffer = tmpRecord.encodeToNDEF();

            // Create NDEF Message for SmartPoster
            ndefMessage = new NdefMessage();
            ndefMessage.appendRecord(tmpRecord);

            // Encode NDEF Message
            buffer = ndefMessage.toByteArray();

            return buffer;
        }

        public static String getURIIdentifierCode(URI_IDENTIFIER_CODE code)
        {
            // For more information please refer to
            // URI Record Type Definition Technical Specification
            switch (code)
            {
                case HTTP: return "http://";
                case HTTPS: return "https://";
                case HTTP_WWW: return "http://www.";
                case HTTPS_WWW: return "https://www.";
                case TEL: return "tel:";
                case MAIL_TO: return "mailto:";
                case FTP: return "ftp://";
                case NONE: return "";
                default: return "Unknown";
            }
        }
		
        public static String getActionCode(ACTION_RECORD code)
        {
            switch (code)
            {
        		case DO_THE_ACTION: return "Do the action";
        		case SAVE_FOR_LATER: return "Save for later";
        		case OPEN_FOR_EDITING: return "Open for editing";
        		default: return "";
            }
        }
	}
}
