package NDEF;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

public class VCard
{
	public enum ADDRESS_TYPE
    {
		DOMESTIC((byte)0x01),
        INTERNATIONAL((byte)0x02),
        POSTAL((byte)0x04),
        PARCEL((byte)0x08),
        HOME((byte)0x10),
        WORK((byte)0x20),
        PREFERRED((byte)0x40);
        
        private byte byteId;
        ADDRESS_TYPE(byte byteId) {this.byteId = byteId;}
        public int getAddressType() {return byteId;}
        
        public static ADDRESS_TYPE getKeyNameAddressType(byte byteAddressType)
        {
        	for (ADDRESS_TYPE addressType : ADDRESS_TYPE.values()) 
                if (addressType.byteId == byteAddressType) return addressType;
            
        	return HOME;
        }
    }
	
	public enum TELEPHONE_NUMBER_TYPE
    {
        /// <summary>
        /// home telephone number
        /// </summary>
        HOME((byte)0x01),

        /// <summary>
        /// telephone number has voice messaging support
        /// </summary>
        MSG((byte)0x02),

        /// <summary>
        /// Work telephone number
        /// </summary>
        WORK((byte)0x03),

        /// <summary>
        /// preferred-use telephone number
        /// </summary>
        PREFERRED((byte)0x04),

        /// <summary>
        /// voice telephone number
        /// </summary>
        VOICE((byte)0x05),

        /// <summary>
        /// facsimile telephone number
        /// </summary>
        FAX((byte)0x06),

        /// <summary>
        /// cellular telephone number
        /// </summary>
        CELL((byte)0x07),

        /// <summary>
        /// video conferencing telephone number
        /// </summary>
        VIDEO((byte)0x08),

        /// <summary>
        /// paging device telephone number
        /// </summary>
        PAGER((byte)0x09),

        /// <summary>
        /// bulletin board system telephone number
        /// </summary>
        BBS((byte)0x0A),

        /// <summary>
        /// MODEM connected telephone number
        /// </summary>
        MODEM((byte)0x0B),

        /// <summary>
        /// car-phone telephone number
        /// </summary>
        CAR((byte)0x0C),

        /// <summary>
        /// ISDN service telephone number
        /// </summary>
        ISDN((byte)0x0D),

        /// <summary>
        /// personal communication services 
        /// </summary>
        PCS((byte)0x0E);
        
        private byte id;
        TELEPHONE_NUMBER_TYPE(byte byteId) {this.id = byteId;}
        public int getTelephoneNumberType() {return this.id;}
        
        public static TELEPHONE_NUMBER_TYPE getKeyNameTelephoneNumberType(byte byteTelephoneNumberType)
        {
        	for (TELEPHONE_NUMBER_TYPE telephoneNumberType : TELEPHONE_NUMBER_TYPE.values())
        	{
                if (telephoneNumberType.id == byteTelephoneNumberType) 
                	return telephoneNumberType;
        	}
            
        	return HOME;
        }
    }
	
	public enum EMAIL_TYPE
    {
        INTERNET((byte)0x01),
        X400((byte)0x02),
        PREFERRED((byte)0x04);
        
        private byte id;
        EMAIL_TYPE(byte byteId) {this.id = byteId;}
        public int getEmailType() {return id;}
    }
	
	public static class IdentificationType
    {
		private String _formattedName = "";
		private String _givenName = "";
		private String _additionalName = "";
		private String _familyName = "";
		private String _nickName = "";
		private String _honorificPrefix = "";
		private String _honorificSuffix = "";
		private Date _birthdate = null;
		
		public String getFormattedName() { return _formattedName; }
        public void setFormattedName(String value) { this._formattedName = value; }
        
        public String getGivenName() { return _givenName; }
        public void setGivenName(String value) { this._givenName = value; }
        
        public String getAdditionalName() { return _additionalName; }
        public void setAdditionalName(String value) { this._additionalName = value; }
        
        public String getFamilyName() { return _familyName; }
        public void setFamilyName(String value) { this._familyName = value; }
        
        public String getNickName() { return _nickName; }
        public void setNickName(String value) { this._nickName = value; }
        
        public String getHonorificPrefix() { return _honorificPrefix; }
        public void setHonorificPrefix(String value) { this._honorificPrefix = value; }
        
        public String getHonorificSuffix() { return _honorificSuffix; }
        public void setHonorificSuffix(String value) { this._honorificSuffix = value; }
        
        public Date getBirthDate() { return _birthdate; }
        public void setBirthDate(Date value) { this._birthdate = value; }
        
        public String getFormattedString()
        {
    		String formattedString = "";
    		String tempString = "";

            // FORMATTED NAME
            if (getFormattedName().trim() != "")
            {
            	tempString = VCard.escapeString(getFormattedName());

            	formattedString += "FN:" + tempString;
            }

            // NAME : "N:
        	// Purpose: To specify the components of the name of the object the vCard represents.
            // Family Name, Given Name, Additional Names, Honorific Prefixes, and Honorific Suffixes.
            // The text components are separated by the SEMI-COLON character (ASCII decimal 59). Individual text
            // components can include multiple text values (e.g., multiple Additional Names) separated by the COMMA character (ASCII decimal 44).            
            formattedString += "N:";

            // Family name
            formattedString += getFamilyName();

            // Given name
            formattedString += ";" + getGivenName();

            // Additional name
            formattedString += ";" + getAdditionalName();

            return formattedString;
        }
        
        public void loadData(byte[] uData)
        {
            loadData(new String(uData));
        }
        
        public void loadData(String dataAsString)
        {
            String[] vCardProperties = dataAsString.split("\r\n");
            String tempString = "";

            for (String property : vCardProperties)
            {
            	if(property == "")
            		continue;
            	
            	tempString = property;

            	tempString.trim();
            	tempString.replaceAll("[\n\r\0]", "");

                //Check if formatted Name
                if(property.substring(0,2).toLowerCase() == "fn")
                {
                    setFormattedName(property.substring(3));
                    continue;
                }

                //Check if Name
                if (property.substring(0, 2).toLowerCase() == "n")
                {
                	setFormattedName(property.substring(2));
                    continue;
                }
            }
        }
    }
	
	public static class OrganizationalType
    {
        private String _title = "";
        private String _organizationName = "";
        private String _role;
        private String _agent;
        private List<String> _organizatinalUnit = new ArrayList<String>();
	
        /// <summary>
        /// To specify the job title, functional position or function of the object the vCard represents
        /// Ex. TITLE:Director\, Research and Development
        /// </summary>
        public String getTitle() { return _title; }
        public void setTitle(String value) { this._title = value; }
        
        /// <summary>
        /// The type is based on the X.520 Organization Name
        /// and Organization Unit attributes. The type value is a structured type
        /// consisting of the organization name, followed by one or more levels
        /// of organizational unit names.
        /// 
        /// Ex. ORG:ABC\, Inc.;North American Division;Marketing
        /// </summary>        
        public String getOrganizationName() { return _organizationName; }
        public void setOrganizationName(String value) { this._organizationName = value; }
        
        /// <summary>
        /// To specify information concerning the role, occupation, or business category of the object the vCard represents.
        /// Ex. ROLE:Programmer
        /// </summary>
        public String getRole() { return _role; }
        public void setRole(String value) { this._role = value; }
        
        /// <summary>
        ///  To specify information about another person who will
        ///  act on behalf of the individual or resource associated with the vCard.
        ///  
        ///  Type value: The default is a single vcard value. It can also be reset
        ///  to either a single text or uri value. The text value can be used to
        ///  specify textual information. The uri value can be used to specify
        ///  information outside of this MIME entity.
        /// </summary>
        public String getAgent() { return _agent; }
        public void setAgent(String value) { this._agent = value; }
        
        // <summary>
        /// 
        /// </summary>
        public List<String> getOrganizatinalUnit() { return _organizatinalUnit; }
        public void setOrganizatinalUnit(List<String> value) { this._organizatinalUnit = value; }
        
        /// <summary>
        /// Add Division
        /// </summary>
        /// <param name="unitName"></param>
        public void addOrganizationalUnit(String unitName)
        {
        	_organizatinalUnit.add(unitName);
        }
        
        public String getFormattedString()
        {
            String formattedString = "";

            // TITLE
            formattedString = "TITLE:" + VCard.escapeString(getTitle());

            // ROLE
            formattedString += "\r\nROLE:" + VCard.escapeString(getRole());

            // ORGANIZATION
            formattedString += "\r\nORG:" + VCard.escapeString(getOrganizationName());

            // Divisions/Organizational Unit
            for (int i = 0; i < getOrganizatinalUnit().size(); i++)
            	formattedString += ";" + VCard.escapeString(getOrganizatinalUnit().get(i));

            if (getAgent().trim() != "")
            {
                //AGENT - Currently Single Text/URI is only supported. Single vCard will be supported later
            	formattedString += "\r\nAGENT;VALUE=text:" + getAgent();
            }

            return formattedString;
        }
    }
        
    public static class AddressingType
    {
    	private String _poBoxNumber = "";
        private String _extendedAddress = "";
        private String _streetAddress = "";
        private String _locality = "";
        private String _region = "";
        private String _postalCode = "";
        private String _country = "";
        private String _label = "";
        
        private List<ADDRESS_TYPE> _labelType = new ArrayList<ADDRESS_TYPE>();
        private List<ADDRESS_TYPE> _addressType = new ArrayList<ADDRESS_TYPE>();
        
        public String getPoBoxNumber() { return _poBoxNumber; }
        public void setPoBoxNumber(String value) { this._poBoxNumber = value; }
        
        public String getExtendedAddress() { return _extendedAddress; }
        public void setExtendedAddress(String value) { this._extendedAddress = value; }
        
        public String getStreetAddress() { return _streetAddress; }
        public void setStreetAddress(String value) { this._streetAddress = value; }
        
        public String getLocality() { return _locality; }
        public void setLocality(String value) { this._locality = value; }
        
        public String getRegion() { return _region; }
        public void setRegion(String value) { this._region = value; }
        
        public String getPostalCode() { return _postalCode; }
        public void setPostalCode(String value) { this._postalCode = value; }
        
        public String getCountry() { return _country; }
        public void setCountry(String value) { this._country = value; }
        
        /// <summary>
        /// To specify the formatted text corresponding to delivery address of the object the vCard represents.
        /// 
        /// </summary>
        public String getLabel() { return _label; }
        public void setLabel(String value) { this._label = value; }
        
        /// <summary>
        /// The TYPE parameter values can include "dom" to indicate a domestic delivery address; "intl" to indicate an
        /// international delivery address; "postal" to indicate a postal delivery address; "parcel" to indicate a parcel delivery address;
        /// "home" to indicate a delivery address for a residence; "work" to indicate delivery address for a place of work; and "pref" to indicate
        /// the preferred delivery address when more than one address is specified.
        /// 
        /// Value: AddressType 
        /// You can ORed AddressType to specify multiple value
        /// </summary>
        public List<ADDRESS_TYPE> getLabelType() { return _labelType; }
        public void setLabelType(List<ADDRESS_TYPE> value) { this._labelType = value; }
        
        /// <summary>
        /// The TYPE parameter values can include "dom" to indicate a domestic delivery address; "intl" to indicate an
        /// international delivery address; "postal" to indicate a postal delivery address; "parcel" to indicate a parcel delivery address;
        /// "home" to indicate a delivery address for a residence; "work" to indicate delivery address for a place of work; and "pref" to indicate
        /// the preferred delivery address when more than one address is specified.
        /// 
        /// Value: AddressType 
        /// You can ORed AddressType to specify multiple value
        /// </summary>
        public List<ADDRESS_TYPE> getAddressType() { return _addressType; }
        public void setAddressType(List<ADDRESS_TYPE> value) { this._addressType = value; }
        
        public void addLabelType(ADDRESS_TYPE addType)
        {
            _labelType.add(addType);
        }

        public void addAddressType(ADDRESS_TYPE addType)
        {
            _addressType.add(addType);
        }
        
        public String getFormattedString()
        {
            String separator = ";";
            String formattedString = "";

            // ADDRESS
            formattedString = "ADR";

            // PO Box Number
            formattedString += getPoBoxNumber();

            // Extended Address
            formattedString += separator + VCard.escapeString(getExtendedAddress());

            // Street Address
            formattedString += separator + VCard.escapeString(getStreetAddress());

            // Locality
            formattedString += separator + VCard.escapeString(getLocality());

            // Region
            formattedString += separator + VCard.escapeString(getRegion());

            // Postal Code
            formattedString += separator + getPostalCode();

            // Country
            formattedString += separator + VCard.escapeString(getCountry());

            // Label
            if (getLabel() != "")
            {
            	formattedString += "\r\nLABEL;";

                // Address type
                if (getLabelType().size() > 0)
                {
                	formattedString += "TYPE=";

                    for (int i = 0; i < getLabelType().size(); i++)
                    {
                    	formattedString += getAddressTypeAsString(getLabelType().get(i));

                        if ((i + 1) < getLabelType().size())
                        	formattedString += ",";
                    }

                    formattedString += ":";
                }

                // Label 
                formattedString += VCard.escapeString(getLabel());
            }

            return formattedString;
        }
        
        private String getAddressTypeAsString(ADDRESS_TYPE addType)
        {
            switch (addType)
            {
                case DOMESTIC: return "dom";
                case HOME: return "home";
                case INTERNATIONAL: return "intl";
                case PARCEL: return "parcel";
                case POSTAL: return "postal";
                case PREFERRED: return "pref";
                case WORK: return "work";
                default: return "home";
            }
        }
    }
    
    public static class TelecommunicationType
    {
        private String _telephoneNumber = "";
        private String _emailAddress = "";
        private String _mailerName = "";

        private List<TELEPHONE_NUMBER_TYPE> _telephoneNumberType = new ArrayList<TELEPHONE_NUMBER_TYPE>();
        private List<EMAIL_TYPE> _emailType = new ArrayList<EMAIL_TYPE>();
        
        public String getTelephoneNumber() { return _telephoneNumber; }
        public void setTelephoneNumber(String value) { this._telephoneNumber = value; }
        
        public String getEmailAddress() { return _emailAddress; }
        public void setEmailAddress(String value) { this._emailAddress = value; }
        
        public String getMailerName() { return _mailerName; }
        public void setMailerName(String value) { this._mailerName = value; }
        
        public List<TELEPHONE_NUMBER_TYPE> getTelephoneNumberType() { return _telephoneNumberType; }
        public void setTelephoneNumberType(List<TELEPHONE_NUMBER_TYPE> value) { this._telephoneNumberType = value; }
        
        public List<EMAIL_TYPE> getEmailType() { return _emailType; }
        public void setEmailType(List<EMAIL_TYPE> value) { this._emailType = value; }
        
        public void addTelephoneNumberType(TELEPHONE_NUMBER_TYPE telType)
        {
            if (_telephoneNumberType.contains(telType))
                return;

            _telephoneNumberType.add(telType);
        }
        
        public void addEmailType(EMAIL_TYPE emailType)
        {
            if (_emailType.contains(emailType))
                return;

            _emailType.add(emailType);
        }       
        
        public TelecommunicationType()
        {
        	_telephoneNumberType.add(TELEPHONE_NUMBER_TYPE.VOICE);
            _emailType.add(EMAIL_TYPE.INTERNET);
        }
        
        public String getFormattedString()
        {
            String formattedString = "";

            if (getTelephoneNumber().trim() != "")
            {
            	formattedString += "TEL;";

                // Telephone Number type

            	formattedString += "TYPE=";

                for (int i = 0; i < getTelephoneNumberType().size(); i++)
                {
                	formattedString += getTelphoneNumberTypeAsString(getTelephoneNumberType().get(i));

                    if ((i + 1) < getTelephoneNumberType().size())
                    	formattedString += ",";
                }

                formattedString += ":";

                // Telephone Number
                formattedString += getTelephoneNumber();
            }

            if (getEmailAddress().trim() != "")
            {

            	formattedString += "\nEMAIL;";

                // Email Type
            	formattedString += "TYPE=";

                for (int i = 0; i < getEmailType().size(); i++)
                {
                	formattedString += getEmailTypeAsString(getEmailType().get(i));

                    if ((i + 1) < getEmailType().size())
                    	formattedString += ",";
                }

                formattedString += ":";

                // Email Address
                formattedString += getEmailAddress();
            }
            
            if (getMailerName() != "")
            {
            	formattedString += "\nMAILER:" + getMailerName();
            }

            return formattedString;
        }

        private String getTelphoneNumberTypeAsString(TELEPHONE_NUMBER_TYPE telphoneNumberType)
        {
            switch (telphoneNumberType)
            {
                case BBS: return "bss";
                case CAR: return "car";
                case CELL: return "cell";
                case FAX: return "fax";
                case HOME: return "home";
                case ISDN: return "isdn";
                case MODEM: return "modem";
                case MSG: return "msg";
                case PAGER: return "pager";
                case PCS: return "pcs";
                case PREFERRED: return "pref";
                case VIDEO: return "video";
                case VOICE: return "voice";
                case WORK: return "work";
                default: return "voice";
            }
        }

        private String getEmailTypeAsString(EMAIL_TYPE emailType)
        {
            switch (emailType)
            {
                case INTERNET: return "internet";
                case PREFERRED: return "pref";
                case X400: return "x400";
                default: return "internet";
            }
        }
    }
    
    public static class ExplanatoryType
    {
        private String _url;

        public String getUrl() { return _url; }
        public void setUrl(String value) { this._url = value; }

        public String getFormattedString()
        {
            String formattedString = "";

            // URL
            formattedString = "URL:" + getUrl();

            return formattedString;
        }
    }
    
    public static String escapeString(String text)
    {
        return escapeString(text, new char[] { ',', ';' });
    }

    public static String escapeString(String text, char[] characters)
    {
    	String parsedString = "";
        String tempString = "";

        if (text == "")
            return text;

        if (characters == null || characters.length < 1)
            return text;

        parsedString = text;

        for (int c = 0; c < characters.length; c++)
        {
         	tempString = parsedString;

         	for (int index = 0; index < tempString.length();)
            {
         		if (tempString.charAt(index) == characters[c])
         		{
         			tempString = new StringBuilder(tempString).insert(index, "\\").toString();
         			index += 2;
         		}
                else
                {
                	index++;
                }
            }

         	parsedString = tempString;
        }

    	return parsedString;
	}
}
