﻿'===========================================================================================
' * 
' *  Copyright (C)   : Advanced Card System Ltd
' * 
' *  File            : Acr1281UC1.cs
' * 
' *  Description     : Contains Methods and Properties related to ACR1281U-C1 device's functionality
' * 
' *  Author          : Arturo Salvamante
' *  
' *  Date            : October 19, 2011
' * 
' *  Revision Traile : [Author] / [Date if modification] / [Details of Modifications done] 
' * 
' * =========================================================================================


Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports Mifare_NDEF_Sample_Code.Acs.Readers.Pcsc
Imports Mifare_NDEF_Sample_Code.Acs

Namespace Mifare_Change_Key
    Public Enum CHIP_TYPE
        UNKNOWN = 0
        MIFARE_1K = 1
        MIFARE_4K = 2
        MIFARE_ULTRALIGHT = 3
    End Enum

    Public Enum KEY_STRUCTURE
        VOLATILE = &H0
        NON_VOLATILE = &H20
    End Enum

    Public Enum KEYTYPES
        ACR122_KEYTYPE_A = 96
        ACR122_KEYTYPE_B = 97
    End Enum

    Public Class ReaderFunctions
        Inherits PcscReader
        Public Function getCardSerialNumber() As Byte()
            Dim cardSerial As Byte()

            apduCommand = New Apdu()
            'Intruction Class
            'Intruction Code
            'Parameter 1
            'Parameter 2
            apduCommand.setCommand(New Byte() {&HFF, &HCA, &H0, &H0, &H0})
            'Parameter 3
            apduCommand.lengthExpected = 20
            sendCommand()

            If apduCommand.statusWord(0) <> &H90 Then
                Return Nothing
            End If

            cardSerial = New Byte(apduCommand.response.Length - 1) {}
            Array.Copy(apduCommand.response, cardSerial, cardSerial.Length)

            Return cardSerial

        End Function

        Public Function getAnswerToSelect() As Byte()
            apduCommand = New Apdu()
            apduCommand.setCommand(New Byte() {&HFF, &HCA, &H1, &H0, &H0})

            apduCommand.lengthExpected = 50

            sendCommand()

            If Not apduCommand.statusWordEqualTo(New Byte() {&H90, &H0}) Then
                Throw New CardException("Unable to get Answer to Select (ATS)", apduCommand.statusWord)
            End If

            Return apduCommand.response
        End Function

        Public Function loadAuthKey(ByVal key As Byte(), ByVal keyNumber As Byte) As Boolean

            If keyNumber > &H1 Then
                Throw New Exception("Key number is invalid")
            End If

            If key.Length <> 6 Then
                Throw New Exception("Invalid key length")
            End If


            apduCommand = New Apdu()
            apduCommand.setCommand(New Byte() {&HFF, &H82, &H0, keyNumber, &H6})
            apduCommand.data = key

            sendCommand()

            If Not apduCommand.statusWordEqualTo(New Byte() {&H90, &H0}) Then
                Return False
            End If

            Return True
        End Function

        Public Function authenticate(ByVal blockNum As Byte, ByVal keyType As KEYTYPES, ByVal KeyNum As Byte) As Boolean

            If KeyNum < &H0 AndAlso KeyNum > &H20 Then
                Throw New Exception("Key number is invalid")
            End If

            apduCommand = New Apdu()
            apduCommand.setCommand(New Byte() {&HFF, &H86, &H0, &H0, &H5})
            apduCommand.data = New Byte() {&H1, &H0, CByte(blockNum), CByte(keyType), KeyNum}

            sendCommand()

            If Not apduCommand.statusWordEqualTo(New Byte() {&H90, &H0}) Then
                Return False
            End If

            Return True
        End Function

        Public Function getChipType() As CHIP_TYPE
            Dim rdrLen As Integer = 0, retCode As Integer, protocol As Integer = pcscConnection.activeProtocol
            Dim pdwSate As Integer = 0, atrLen As Integer = 33
            Dim atr As Byte() = New Byte(99) {}
            Dim cardType As CHIP_TYPE = CHIP_TYPE.UNKNOWN

            Try
                retCode = PcscProvider.SCardStatus(pcscConnection.cardHandle, pcscConnection.readerName, rdrLen, pdwSate, protocol, atr, _
                 atrLen)

                If retCode <> PcscProvider.SCARD_S_SUCCESS Then
                    Throw New PcscException(retCode)
                End If

                pcscConnection.activeProtocol = protocol

                If atr.Length < 33 Then
                    Return CHIP_TYPE.UNKNOWN
                End If

                Array.Resize(atr, atrLen)

                If atr(13) = &H0 AndAlso atr(14) = &H1 Then
                    cardType = CHIP_TYPE.MIFARE_1K
                ElseIf atr(13) = &H0 AndAlso atr(14) = &H2 Then
                    cardType = CHIP_TYPE.MIFARE_4K
                ElseIf atr(13) = &H0 AndAlso atr(14) = &H3 Then
                    cardType = CHIP_TYPE.MIFARE_ULTRALIGHT
                Else
                    cardType = CHIP_TYPE.UNKNOWN
                End If

            Catch generalException As Exception
            End Try
            Return cardType
        End Function

    End Class
End Namespace

