﻿'===========================================================================================
' * 
' *  Author          : Arturo Salvamante
' * 
' *  File            : NDEF.cs
' * 
' *  Copyright (C)   : Advanced Card System Ltd
' * 
' *  Description     : Contains Methods and Properties for NDEF
' * 
' *  Date            : June 03, 2011
' * 
' *  Revision Traile : [Author] / [Date if modification] / [Details of Modifications done]
' * 
' * 
' * =========================================================================================

Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports System.Text

Namespace Acs.SmartCard.Reader.Pcsc.Nfc

    Public Enum TYPE_NAME_FORMAT
        EMPTY = &H0
        NFC_WELL_KNOWN_TYPE = &H1
        MEDIA_TYPE = &H2
        ABSOLUTE_URI = &H3
        NFC_FORM_EXTERNAL_TYPE = &H4
        UNKNOWN = &H5
    End Enum

    Public Enum ACTION_RECORD
        DO_THE_ACTION = &H0
        SAVE_FOR_LATER = &H1
        OPEN_FOR_EDITING = &H2
    End Enum

    Public Enum URI_IDENTIFIER
        NONE = &H0
        HTTP_WWW = &H1
        HTTPS_WWW = &H2
        HTTP = &H3
        HTTPS = &H4
        TEL = &H5
        MAIL_TO = &H6
        FTP_ANONYMOUS = &H7
        FTP_FTP = &H8
        FTPS = &H9
        SFTP = &HA
        SMB = &HB
        NFS = &HC
        FTP = &HD
        DAV = &HE
        NEWS = &HF
        TELNET = &H10
        IMAP = &H11
        RTSP = &H12
        URN = &H13
        POP = &H14
        SIP = &H15
        SIPS = &H16
        TFTP = &H17
        BTSPP = &H18
        BTL2CAP = &H19
        BTGOEP = &H1A
        TCPOBEX = &H1B
        IRDAOBEX = &H1C
        FILE = &H1D
        URNEPCID = &H1E
        URNEPCTAG = &H1F
        URNEPCRAW = &H21
        URNEPC = &H22
        URNNFC = &H23
    End Enum

    Public Class NdefMessage
        Private ndefRecords As New List(Of NdefRecord)()

        Public Sub appendRecord(ByVal record As NdefRecord)
            ndefRecords.Add(record)
        End Sub

        Public Sub insertRecord(ByVal index As Integer, ByVal record As NdefRecord)
            ndefRecords.Insert(index, record)
        End Sub

        Public Function getRecords() As List(Of NdefRecord)
            Return ndefRecords
        End Function

        Public Function getRecord(ByVal recordType As NdefRecordType) As NdefRecord
            For Each r As NdefRecord In ndefRecords
                If r.recordType.typeName = recordType.typeName AndAlso r.recordType.typeNameFormat = recordType.typeNameFormat Then
                    Return r
                End If
            Next

            Return Nothing
        End Function

        Public Function getNumberOfRecords() As Integer
            Return ndefRecords.Count
        End Function

        Public Function toByteArray() As Byte()
            Dim buffer As Byte() = New Byte(-1) {}
            Dim tmpArray As Byte()
            Dim indx As Integer = 0

            For i As Integer = 0 To ndefRecords.Count - 1
                indx = buffer.Length

                If i = 0 Then
                    ndefRecords(i).messageBegin = True
                End If

                If (i + 1) = ndefRecords.Count Then
                    ndefRecords(i).messageEnd = True
                End If


                tmpArray = ndefRecords(i).encodeToNDEF()

                'Resize destination array to acoomodate new record
                Array.Resize(buffer, buffer.Length + tmpArray.Length)

                'Copy new  ndef record to byte array
                Array.Copy(tmpArray, 0, buffer, indx, tmpArray.Length)
            Next

            Return buffer
        End Function
    End Class

    Public Class NdefRecordType
        Const MAX_PAYLOAD_SIZE As Integer = 255

        Private _typeNameFormat As TYPE_NAME_FORMAT
        Private _typeName As String

        Public Sub New(ByVal format As TYPE_NAME_FORMAT, ByVal name As String)
            _typeNameFormat = format
            _typeName = name
        End Sub

        Public Property typeNameFormat() As TYPE_NAME_FORMAT
            Get
                Return _typeNameFormat
            End Get
            Set(ByVal value As TYPE_NAME_FORMAT)
                _typeNameFormat = value
            End Set
        End Property

        Public Property typeName() As String
            Get
                Return _typeName
            End Get
            Set(ByVal value As String)
                If value.Trim().Length > MAX_PAYLOAD_SIZE Then
                    Throw New Exception("Pay load type is too long")
                End If

                _typeName = value.Trim()
            End Set
        End Property
    End Class

    Public Class NdefRecord
        Const MAX_PAYLOAD As Integer = 256

        Private _messageBegin As Boolean
        Private _messageEnd As Boolean
        Private _isTerminatingRecordChunk As Boolean
        Private _recordType As NdefRecordType
        Private _payLoad As Byte()
        Private _messageID As Byte() = New Byte(-1) {}

        Public Sub New(ByVal recordType As NdefRecordType)
            _messageBegin = False
            _messageEnd = False
            _isTerminatingRecordChunk = True
            _recordType = recordType
        End Sub

        Public Property messageBegin() As Boolean
            Get
                Return _messageBegin
            End Get
            Set(ByVal value As Boolean)
                _messageBegin = value
            End Set
        End Property

        Public Property messageEnd() As Boolean
            Get
                Return _messageEnd
            End Get
            Set(ByVal value As Boolean)
                _messageEnd = value
            End Set
        End Property

        Public Property isTerminatingRecordChunk() As Boolean
            Get
                Return _isTerminatingRecordChunk
            End Get
            Set(ByVal value As Boolean)
                _isTerminatingRecordChunk = value
            End Set
        End Property

        Public ReadOnly Property isShortRecord() As Boolean
            Get
                If payLoad.Length < MAX_PAYLOAD Then
                    Return True
                Else
                    Return False
                End If
            End Get
        End Property

        Public Property recordType() As NdefRecordType
            Get
                Return _recordType
            End Get
            Set(ByVal value As NdefRecordType)
                _recordType = value
            End Set
        End Property

        Public Property payLoad() As Byte()
            Get
                Return _payLoad
            End Get
            Set(ByVal value As Byte())
                _payLoad = value
            End Set
        End Property

        Public Property payLoadStr() As String
            Get
                Return ASCIIEncoding.ASCII.GetString(_payLoad)
            End Get
            Set(ByVal value As String)
                _payLoad = ASCIIEncoding.ASCII.GetBytes(value)
            End Set
        End Property

        Public Property messageID() As Byte()
            Get
                Return _messageID
            End Get
            Set(ByVal value As Byte())
                _messageID = value
            End Set
        End Property

        Public Function encodeToNDEF() As Byte()
            Dim enableByte1 As Byte = &H80
            Dim enableByte2 As Byte = &H40
            Dim enableByte3 As Byte = &H20
            Dim enableByte4 As Byte = &H10
            Dim enableByte5 As Byte = &H8
            Dim ndefHeader As Byte = 0
            Dim ndef As Byte() = New Byte(-1) {}

            'Set NDEF Header
            If messageBegin Then
                ndefHeader = ndefHeader Or enableByte1
            End If

            If messageEnd Then
                ndefHeader = ndefHeader Or enableByte2
            End If

            If Not isTerminatingRecordChunk Then
                ndefHeader = ndefHeader Or enableByte3
            End If

            If payLoad.Length < MAX_PAYLOAD Then
                ndefHeader = ndefHeader Or enableByte4
            End If

            If messageID.Length > 0 Then
                ndefHeader = ndefHeader Or enableByte5
            End If

            ndefHeader = ndefHeader Or CByte(recordType.typeNameFormat)

            ndef = Helper.appendArrays(ndef, ndefHeader)

            'Set Payload Type Length
            ndef = Helper.appendArrays(ndef, CByte(recordType.typeName.Length))

            'Set Payload Length
            If payLoad.Length < MAX_PAYLOAD Then
                ndef = Helper.appendArrays(ndef, CByte(payLoad.Length))
            Else
                ndef = Helper.appendArrays(ndef, Helper.intToByte(payLoad.Length))
            End If

            'Set Message ID Length
            If messageID IsNot Nothing AndAlso messageID.Length > 0 Then
                ndef = Helper.appendArrays(ndef, CByte(messageID.Length))
            End If

            'Set Payload Type
            ndef = Helper.appendArrays(ndef, ASCIIEncoding.ASCII.GetBytes(recordType.typeName.Trim()))

            'Set Message ID
            If messageID IsNot Nothing AndAlso messageID.Length > 0 Then
                ndef = Helper.appendArrays(ndef, messageID)
            End If

            'Set Payload
            ndef = Helper.appendArrays(ndef, payLoad)

            Return ndef
        End Function

        Public Sub appendPayload(ByVal payLoad As Byte())
            Dim indx As Integer = 0

            If _payLoad Is Nothing Then
                _payLoad = New Byte(-1) {}
            End If

            indx = _payLoad.Length

            Array.Resize(_payLoad, _payLoad.Length + payLoad.Length)

            Array.Copy(payLoad, 0, _payLoad, indx, payLoad.Length)
        End Sub

        Public Sub appendPayload(ByVal payLoad As String)
            Dim indx As Integer = 0

            If payLoad = "" Then
                Return
            End If

            If _payLoad Is Nothing Then
                _payLoad = New Byte(-1) {}
            End If

            indx = _payLoad.Length

            Dim buffer As Byte() = ASCIIEncoding.ASCII.GetBytes(payLoad)

            Array.Resize(_payLoad, _payLoad.Length + buffer.Length)

            Array.Copy(buffer, 0, _payLoad, indx, buffer.Length)
        End Sub

        Public Sub appendPayload(ByVal payLoad As Byte)
            appendPayload(New Byte() {payLoad})
        End Sub

        Public Function getNestedNdefMessage() As NdefMessage
            Dim payLoadIndex As Integer = 0

            If payLoad Is Nothing Then
                Throw New Exception("Payload is not yet been set")
            End If

            Return getNestedNdefMessage(payLoadIndex, payLoad)
        End Function

        Public Shared Function getNestedNdefMessage(ByVal index As Integer, ByVal data As Byte()) As NdefMessage
            Dim ndefMessage As New NdefMessage()
            Dim ndefRecord As NdefRecord
            Dim typeNameFormat As Byte = &H0
            Dim typeNameLength As Byte = &H0
            Dim idLength As Byte = &H0

            Dim payloadLength As Integer = &H0
            Dim currentIndex As Integer = 0

            Dim typeName As String = ""

            Dim isMessageBeginSet As Boolean = False
            Dim isMessageEndSet As Boolean = False
            Dim isIdLengthPresent As Boolean = False
            Dim isChunkFlagPresent As Boolean = False
            Dim isShortRecordSet As Boolean = False


            If data.Length <= index Then
                Throw New Exception("Invalid index")
            End If

            While index < data.Length
                currentIndex = index

                'Get Type Name Format
                'e.g. NFC Well Know Type = 0x01
                typeNameFormat = CByte(data(currentIndex) And &H7)

                If typeNameFormat <> &H1 AndAlso typeNameFormat <> &H2 Then
                    Throw New Exception("Type Name Format " + CType(typeNameFormat, TYPE_NAME_FORMAT).ToString() + " is not supported.")
                End If

                'Check if Message Begin (Bit 7) is set 
                If (data(currentIndex) And &H80) <> &H0 Then
                    isMessageBeginSet = True
                Else
                    isMessageBeginSet = False
                End If

                'Check if Message End (Bit 6) is set
                If (data(currentIndex) And &H40) <> &H0 Then
                    isMessageEndSet = True
                Else
                    isMessageEndSet = False
                End If

                'Check if Chunk Flag (Bit 5) is set
                If (data(currentIndex) And &H20) <> &H0 Then
                    isChunkFlagPresent = True
                Else
                    isChunkFlagPresent = False
                End If

                'Check if Short Record (bit 4) is set
                If (data(currentIndex) And &H10) <> &H0 Then
                    isShortRecordSet = True
                Else
                    isShortRecordSet = False
                End If

                'Check if ID length is set
                If (data(currentIndex) And &H8) <> &H0 Then
                    isIdLengthPresent = True
                Else
                    isIdLengthPresent = False
                End If

                currentIndex += 1

                'get the type length
                'Refer for Short Record section of NFC Data Exchange Format
                'Technical Specification for more information
                typeNameLength = data(currentIndex)
                currentIndex += 1

                If isShortRecordSet Then
                    'For Short Record payload, length is 1 byte
                    'Get Payload Length    
                    payloadLength = data(currentIndex)
                    currentIndex += 1
                Else
                    'For Non Short Record payload, length is 4 bytes
                    payloadLength = Helper.byteToInt(data.Skip(currentIndex).Take(4).ToArray())
                    currentIndex += 4
                End If

                If isIdLengthPresent Then
                    '+1 to get ID Length
                    idLength = data(currentIndex)
                    currentIndex += 1
                End If

                '+1 to get Payload Type Name
                'Payload Type Name offset = currentIndex to currentIndex  + Type Name Length
                typeName = ASCIIEncoding.ASCII.GetString(data.Skip(currentIndex).Take(typeNameLength).ToArray())
                currentIndex += typeNameLength

                'Initialize new ndef record object
                ndefRecord = New NdefRecord(New NdefRecordType(CType(typeNameFormat, TYPE_NAME_FORMAT), typeName))
                ndefRecord.messageBegin = isMessageBeginSet
                ndefRecord.messageEnd = isMessageEndSet
                ndefRecord.isTerminatingRecordChunk = Not isChunkFlagPresent

                'If ID Length is present get record ID
                If isIdLengthPresent Then
                    ndefRecord.messageID = data.Skip(currentIndex).Take(idLength).ToArray()
                    currentIndex += idLength
                End If

                ndefRecord.payLoad = data.Skip(currentIndex).Take(payloadLength).ToArray()
                currentIndex += payloadLength
                index = currentIndex

                ndefMessage.appendRecord(ndefRecord)
            End While
            Return ndefMessage
        End Function
    End Class

    Public Class Ndef
        ''' <summary>
        ''' Encode the following paramter for NDEF SmartPoster
        ''' </summary>
        ''' <param name="titleLanguage"></param>
        ''' <param name="title"></param>
        ''' <param name="uriPrefix"></param>
        ''' <param name="uri"></param>
        ''' <param name="action"></param>
        ''' <returns></returns>
        Public Shared Function encodeSmartPoster(ByVal titleLanguage As String, ByVal title As String, ByVal uriPrefix As URI_IDENTIFIER, ByVal uri As String, ByVal action As System.Nullable(Of ACTION_RECORD)) As Byte()
            Dim buffer As Byte() = New Byte(-1) {}
            Dim useUtf8 As Boolean = True
            Dim statusByte As Byte = 0

            Dim tmpRecord As NdefRecord
            Dim ndefMessage As New NdefMessage()

            'Encode Title
            If title.Trim() <> "" Then
                'If UTF 8 set bit 7 to 0
                'If UTF 16 set bit 7 to 1
                If Not useUtf8 Then
                    statusByte = &H80
                End If

                'Length of the Language (ISO/IANA language code)
                'ex. "en-US"
                statusByte = statusByte Or CByte(titleLanguage.Length)

                tmpRecord = New NdefRecord(New NdefRecordType(TYPE_NAME_FORMAT.NFC_WELL_KNOWN_TYPE, "T"))
                tmpRecord.messageBegin = True
                tmpRecord.appendPayload(statusByte)
                tmpRecord.appendPayload(titleLanguage)
                tmpRecord.appendPayload(title)

                ndefMessage.appendRecord(tmpRecord)
            End If


            'Encode Action            
            If action IsNot Nothing AndAlso action.HasValue Then
                tmpRecord = New NdefRecord(New NdefRecordType(TYPE_NAME_FORMAT.NFC_WELL_KNOWN_TYPE, "act"))
                tmpRecord.appendPayload(CByte(action))
                ndefMessage.appendRecord(tmpRecord)
            End If


            'Encode URI
            tmpRecord = New NdefRecord(New NdefRecordType(TYPE_NAME_FORMAT.NFC_WELL_KNOWN_TYPE, "U"))

            'URI Prefix
            'See URI Record Type Definition 
            'http://www.nfc-forum.org
            tmpRecord.appendPayload(CByte(uriPrefix))
            tmpRecord.appendPayload(uri)

            ndefMessage.appendRecord(tmpRecord)

            buffer = ndefMessage.toByteArray()

            'Smart Poster Header
            tmpRecord = New NdefRecord(New NdefRecordType(TYPE_NAME_FORMAT.NFC_WELL_KNOWN_TYPE, "Sp"))
            tmpRecord.messageBegin = True
            tmpRecord.messageEnd = True
            tmpRecord.appendPayload(buffer)

            buffer = tmpRecord.encodeToNDEF()

            'Create NDEF Message for SmartPoster
            ndefMessage = New NdefMessage()
            ndefMessage.appendRecord(tmpRecord)

            'Encode NDEF Message
            buffer = ndefMessage.toByteArray()

            Return buffer

        End Function

        Public Shared Function getURIIdentifierCode(ByVal code As URI_IDENTIFIER) As String
            'For more information please refer to
            ' URI Record Type Definition Technical Specification
            Select Case code
                Case URI_IDENTIFIER.HTTP
                    Return "http://"
                Case URI_IDENTIFIER.HTTPS
                    Return "https://"
                Case URI_IDENTIFIER.HTTP_WWW
                    Return "http://www."
                Case URI_IDENTIFIER.HTTPS_WWW
                    Return "https://www."
                Case URI_IDENTIFIER.TEL
                    Return "tel:"
                Case URI_IDENTIFIER.MAIL_TO
                    Return "mailto:"
                Case URI_IDENTIFIER.FTP
                    Return "ftp://"
                Case URI_IDENTIFIER.NONE
                    Return ""
                Case Else
                    Return "Unknown"
            End Select
        End Function
    End Class
End Namespace