﻿Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.IO
Imports System.Drawing
Imports System.Drawing.Imaging

Namespace Acs.SmartCard.Reader.Pcsc.Nfc

    Public Enum ADDRESS_TYPE
        DOMESTIC = &H1
        INTERNATIONAL = &H2
        POSTAL = &H4
        PARCEL = &H8
        HOME = &H10
        WORK = &H20
        PREFERRED = &H40
    End Enum

    Public Enum TELEPHONE_NUMBER_TYPE
        ''' <summary>
        ''' home telephone number
        ''' </summary>
        HOME = &H1

        ''' <summary>
        ''' telephone number has voice messaging support
        ''' </summary>
        MSG = &H2

        ''' <summary>
        ''' Work telephone number
        ''' </summary>
        WORK = &H3

        ''' <summary>
        ''' preferred-use telephone number
        ''' </summary>
        PREFERRED = &H4

        ''' <summary>
        ''' voice telephone number
        ''' </summary>
        VOICE = &H5

        ''' <summary>
        ''' facsimile telephone number
        ''' </summary>
        FAX = &H6

        ''' <summary>
        ''' cellular telephone number
        ''' </summary>
        CELL = &H7

        ''' <summary>
        ''' video conferencing telephone number
        ''' </summary>
        VIDEO = &H8

        ''' <summary>
        ''' paging device telephone number
        ''' </summary>
        PAGER = &H9

        ''' <summary>
        ''' bulletin board system telephone number
        ''' </summary>
        BBS = &HA

        ''' <summary>
        ''' MODEM connected telephone number
        ''' </summary>
        MODEM = &HB

        ''' <summary>
        ''' car-phone telephone number
        ''' </summary>
        CAR = &HC

        ''' <summary>
        ''' ISDN service telephone number
        ''' </summary>
        ISDN = &HD

        ''' <summary>
        ''' personal communication services 
        ''' </summary>
        PCS = &HE
    End Enum

    Public Enum EMAIL_TYPE
        INTERNET = &H1
        X400 = &H2
        PREFERRED = &H4
    End Enum

    Public Class IdentificationType
        Const SEPERATOR As String = ";"

        Private _formattedName As String = ""
        Private _givenName As String = ""
        Private _additionalName As String = ""
        Private _familyName As String = ""
        Private _nickName As String = ""
        Private _honorificPrefix As String = ""
        Private _honorificSuffix As String = ""
        Private _birthdate As System.Nullable(Of DateTime) = Nothing

        Public Property formattedName() As String
            Get
                Return _formattedName
            End Get
            Set(ByVal value As String)
                _formattedName = value
            End Set
        End Property

        Public Property givenName() As String
            Get
                Return _givenName
            End Get
            Set(ByVal value As String)
                _givenName = value
            End Set
        End Property

        Public Property additionalName() As String
            Get
                Return _additionalName
            End Get
            Set(ByVal value As String)
                _additionalName = value
            End Set
        End Property

        Public Property familyName() As String
            Get
                Return _familyName
            End Get
            Set(ByVal value As String)
                _familyName = value
            End Set
        End Property

        Public Property nickName() As String
            Get
                Return _nickName
            End Get
            Set(ByVal value As String)
                _nickName = value
            End Set
        End Property

        Public Property honorificPrefix() As String
            Get
                Return _honorificPrefix
            End Get
            Set(ByVal value As String)
                _honorificPrefix = value
            End Set
        End Property

        Public Property honorificSuffix() As String
            Get
                Return _honorificSuffix
            End Get
            Set(ByVal value As String)
                _honorificSuffix = value
            End Set
        End Property

        Public Property birthDate() As DateTime
            Get
                Return _birthdate.Value
            End Get
            Set(ByVal value As DateTime)
                _birthdate = value
            End Set
        End Property

        Public Function getFormattedString() As String
            Dim fString As String = ""
            Dim tempString As String = ""

            'FORMATTED NAME
            If formattedName.Trim() <> "" Then
                tempString = VCard.escapeString(formattedName)

                fString += Convert.ToString("FN:") & tempString
            End If

            'NAME
            'Family Name, Given Name, Additional Names, Honorific Prefixes, and Honorific Suffixes.
            'The text components are separated by the SEMI-COLON character (ASCII decimal 59). Individual text
            'components can include multiple text values (e.g., multiple Additional Names) separated by the COMMA character (ASCII decimal 44).            
            'fString += "\r\nN:";

            fString += "N:"

            'Family name
            fString += familyName

            'Given name
            fString += Convert.ToString(";") & givenName

            'Additional name
            fString += Convert.ToString(";") & additionalName

            Return fString
        End Function

        Public Sub loadData(ByVal data As Byte())
            loadData(ASCIIEncoding.ASCII.GetString(data))
        End Sub

        Public Sub loadData(ByVal dataAsString As String)
            Dim vProperties As String() = dataAsString.Split(New String() {vbCr & vbLf}, StringSplitOptions.RemoveEmptyEntries)
            Dim tempString As String = ""

            For Each [property] As String In vProperties
                tempString = [property]

                tempString.TrimEnd()
                tempString.TrimStart()
                tempString.Trim(New Char() {ControlChars.NullChar, ControlChars.Lf, ControlChars.Cr})

                'Check if formatted Name
                If [property].Substring(0, 2).ToLower() = "fn" Then
                    formattedName = [property].Substring(3)
                    Continue For
                End If

                'Check if Name
                If [property].Substring(0, 2).ToLower() = "n" Then
                    formattedName = [property].Substring(2)
                    Continue For
                End If
            Next
        End Sub
    End Class

    Public Class OrganizationalType
        Private _title As String = ""
        Private _organizationName As String = ""
        Private _role As String
        Private _agent As String
        Private _organizatinalUnit As New List(Of String)()

        ''' <summary>
        ''' To specify the job title, functional position or function of the object the vCard represents
        ''' Ex. TITLE:Director\, Research and Development
        ''' </summary>
        Public Property title() As String
            Get
                Return _title
            End Get
            Set(ByVal value As String)
                _title = value
            End Set
        End Property

        ''' <summary>
        ''' The type is based on the X.520 Organization Name
        ''' and Organization Unit attributes. The type value is a structured type
        ''' consisting of the organization name, followed by one or more levels
        ''' of organizational unit names.
        ''' 
        ''' Ex. ORG:ABC\, Inc.;North American Division;Marketing
        ''' </summary>        
        Public Property organizationName() As String
            Get
                Return _organizationName
            End Get
            Set(ByVal value As String)
                _organizationName = value
            End Set
        End Property

        ''' <summary>
        ''' To specify information concerning the role, occupation, or business category of the object the vCard represents.
        ''' Ex. ROLE:Programmer
        ''' </summary>
        Public Property role() As String
            Get
                Return _role
            End Get
            Set(ByVal value As String)
                _role = value
            End Set
        End Property

        ''' <summary>
        '''  To specify information about another person who will
        '''  act on behalf of the individual or resource associated with the vCard.
        '''  
        '''  Type value: The default is a single vcard value. It can also be reset
        '''  to either a single text or uri value. The text value can be used to
        '''  specify textual information. The uri value can be used to specify
        '''  information outside of this MIME entity.
        ''' </summary>
        Public Property agent() As String
            Get
                Return _agent
            End Get
            Set(ByVal value As String)
                _agent = value
            End Set
        End Property

        ''' <summary>
        ''' 
        ''' </summary>
        Public Property organizationalUnit() As List(Of String)
            Get
                Return _organizatinalUnit
            End Get
            Set(ByVal value As List(Of String))
                _organizatinalUnit = value
            End Set
        End Property

        ''' <summary>
        ''' Add Division
        ''' </summary>
        ''' <param name="unitName"></param>
        Public Sub addOrganizationalUnit(ByVal unitName As String)
            _organizatinalUnit.Add(unitName)
        End Sub

        Public Function getFormattedString() As String
            Dim fString As String = ""

            'TITLE
            fString = Convert.ToString("TITLE:") & VCard.escapeString(title)

            'ROLE
            fString += Convert.ToString(vbCr & vbLf & "ROLE:") & VCard.escapeString(role)

            'ORGANIZATION
            fString += Convert.ToString(vbCr & vbLf & "ORG:") & VCard.escapeString(organizationName)

            'Divisions/Organizational Unit
            For i As Integer = 0 To organizationalUnit.Count - 1
                fString += Convert.ToString(";") & VCard.escapeString(organizationalUnit(i))
            Next

            If agent.Trim() <> "" Then
                'AGENT - Currently Single Text/URI is only supported. Single vCard will be supported later
                fString += Convert.ToString(vbCr & vbLf & "AGENT;VALUE=text:") & agent
            End If

            Return fString
        End Function

    End Class

    Public Class AddressingType
        Private _poBoxNumber As String = ""
        Private _extendedAddress As String = ""
        Private _streetAddress As String = ""
        Private _locality As String = ""
        Private _region As String = ""
        Private _postalCode As String = ""
        Private _country As String = ""
        Private _label As String = ""

        Private _labelType As New List(Of ADDRESS_TYPE)()
        Private _addressType As New List(Of ADDRESS_TYPE)()

        Public Property poBoxNumber() As String
            Get
                Return _poBoxNumber
            End Get
            Set(ByVal value As String)
                _poBoxNumber = value
            End Set
        End Property

        Public Property extendedAddress() As String
            Get
                Return _extendedAddress
            End Get
            Set(ByVal value As String)
                _extendedAddress = value
            End Set
        End Property

        Public Property streetAddress() As String
            Get
                Return _streetAddress
            End Get
            Set(ByVal value As String)
                _streetAddress = value
            End Set
        End Property

        Public Property locality() As String
            Get
                Return _locality
            End Get
            Set(ByVal value As String)
                _locality = value
            End Set
        End Property

        Public Property region() As String
            Get
                Return _region
            End Get
            Set(ByVal value As String)
                _region = value
            End Set
        End Property

        Public Property postalCode() As String
            Get
                Return _postalCode
            End Get
            Set(ByVal value As String)
                _postalCode = value
            End Set
        End Property

        Public Property country() As String
            Get
                Return _country
            End Get
            Set(ByVal value As String)
                _country = value
            End Set
        End Property

        ''' <summary>
        ''' To specify the formatted text corresponding to delivery address of the object the vCard represents.
        ''' 
        ''' </summary>
        Public Property label() As String
            Get
                Return _label
            End Get
            Set(ByVal value As String)
                _label = value
            End Set
        End Property

        ''' <summary>
        ''' The TYPE parameter values can include "dom" to indicate a domestic delivery address; "intl" to indicate an
        ''' international delivery address; "postal" to indicate a postal delivery address; "parcel" to indicate a parcel delivery address;
        ''' "home" to indicate a delivery address for a residence; "work" to indicate delivery address for a place of work; and "pref" to indicate
        ''' the preferred delivery address when more than one address is specified.
        ''' 
        ''' Value: AddressType 
        ''' You can ORed AddressType to specify mutiple value
        ''' </summary>
        Public Property labelType() As List(Of ADDRESS_TYPE)
            Get
                Return _labelType
            End Get
            Set(ByVal value As List(Of ADDRESS_TYPE))
                _labelType = value
            End Set
        End Property

        ''' <summary>
        ''' The TYPE parameter values can include "dom" to indicate a domestic delivery address; "intl" to indicate an
        ''' international delivery address; "postal" to indicate a postal delivery address; "parcel" to indicate a parcel delivery address;
        ''' "home" to indicate a delivery address for a residence; "work" to indicate delivery address for a place of work; and "pref" to indicate
        ''' the preferred delivery address when more than one address is specified.
        ''' 
        ''' Value: AddressType 
        ''' You can ORed AddressType to specify mutiple value
        ''' </summary>
        Public Property addressType() As List(Of ADDRESS_TYPE)
            Get
                Return _addressType
            End Get
            Set(ByVal value As List(Of ADDRESS_TYPE))
                _addressType = value
            End Set
        End Property

        Public Sub addLabelType(ByVal addType As ADDRESS_TYPE)
            _labelType.Add(addType)
        End Sub

        Public Sub addAddressType(ByVal addType As ADDRESS_TYPE)
            _addressType.Add(addType)
        End Sub

        Public Function getFormattedString() As String
            Const SEPERATOR As String = ";"
            Dim fString As String = ""


            'ADDRESS
            fString = "ADR"

            'PO Box Number
            fString += poBoxNumber

            'Extended Address
            fString += SEPERATOR & VCard.escapeString(extendedAddress)

            'Street Address
            fString += SEPERATOR & VCard.escapeString(streetAddress)

            'locality
            fString += SEPERATOR & VCard.escapeString(locality)

            'Region
            fString += SEPERATOR & VCard.escapeString(region)

            'Postal Code
            fString += SEPERATOR & postalCode

            'Country
            fString += SEPERATOR & VCard.escapeString(country)

            'LABEL
            If label <> "" Then
                fString += vbCr & vbLf & "LABEL;"

                'Address type
                If labelType.Count > 0 Then
                    fString += "TYPE="

                    For i As Integer = 0 To labelType.Count - 1
                        fString += getAddressTypeAsString(labelType(i))

                        If (i + 1) < labelType.Count Then
                            fString += ","
                        End If
                    Next

                    fString += ":"
                End If

                'Label 
                fString += VCard.escapeString(label)
            End If

            Return fString

        End Function

        Private Function getAddressTypeAsString(ByVal addType As ADDRESS_TYPE) As String
            Select Case addType
                Case ADDRESS_TYPE.DOMESTIC
                    Return "dom"
                Case ADDRESS_TYPE.HOME
                    Return "home"
                Case ADDRESS_TYPE.INTERNATIONAL
                    Return "intl"
                Case ADDRESS_TYPE.PARCEL
                    Return "parcel"
                Case ADDRESS_TYPE.POSTAL
                    Return "postal"
                Case ADDRESS_TYPE.PREFERRED
                    Return "pref"
                Case ADDRESS_TYPE.WORK
                    Return "work"
                Case Else
                    Return "home"
            End Select
        End Function
    End Class

    Public Class TelecommunicationType
        Private _telephoneNumber As String = ""
        Private _emailAddress As String = ""
        Private _mailerName As String = ""

        Public Sub New()
            _telephoneNumberType.Add(TELEPHONE_NUMBER_TYPE.VOICE)
            _emailType.Add(EMAIL_TYPE.INTERNET)
        End Sub

        Private _telephoneNumberType As New List(Of TELEPHONE_NUMBER_TYPE)()

        Private _emailType As New List(Of EMAIL_TYPE)()

        Public Property telephoneNumber() As String
            Get
                Return _telephoneNumber
            End Get
            Set(ByVal value As String)
                _telephoneNumber = value
            End Set
        End Property

        Public Property emailAddress() As String
            Get
                Return _emailAddress
            End Get
            Set(ByVal value As String)
                _emailAddress = value
            End Set
        End Property

        Public Property mailerName() As String
            Get
                Return _mailerName
            End Get
            Set(ByVal value As String)
                _mailerName = value
            End Set
        End Property

        Public Property emailType() As List(Of EMAIL_TYPE)
            Get
                Return _emailType
            End Get
            Set(ByVal value As List(Of EMAIL_TYPE))
                _emailType = value
            End Set
        End Property

        Public Property telephoneNumberType() As List(Of TELEPHONE_NUMBER_TYPE)
            Get
                Return _telephoneNumberType
            End Get
            Set(ByVal value As List(Of TELEPHONE_NUMBER_TYPE))
                _telephoneNumberType = value
            End Set
        End Property

        Public Sub addEmailType(ByVal emailType As EMAIL_TYPE)
            If _emailType.Contains(emailType) Then
                Return
            End If

            _emailType.Add(emailType)
        End Sub

        Public Sub addTelephoneNumberType(ByVal telType As TELEPHONE_NUMBER_TYPE)
            If _telephoneNumberType.Contains(telType) Then
                Return
            End If

            _telephoneNumberType.Add(telType)
        End Sub

        Public Function getFormattedString() As String
            Dim fString As String = ""

            '#Region "TELEPHONE"

            If telephoneNumber.Trim() <> "" Then
                fString += "TEL;"

                'Telephone Number type

                fString += "TYPE="

                For i As Integer = 0 To telephoneNumberType.Count - 1
                    fString += getTelphoneNumberTypeAsString(telephoneNumberType(i))

                    If (i + 1) < telephoneNumberType.Count Then
                        fString += ","
                    End If
                Next

                fString += ":"

                'Telephone Number
                fString += telephoneNumber
            End If

            '#End Region

            '#Region "EMAIL"

            If emailAddress.Trim() <> "" Then

                fString += vbLf & "EMAIL;"

                'Email Type
                fString += "TYPE="

                For i As Integer = 0 To emailType.Count - 1
                    fString += getEmailTypeAsString(emailType(i))

                    If (i + 1) < emailType.Count Then
                        fString += ","
                    End If
                Next

                fString += ":"

                'Email Address
                fString += emailAddress
            End If

            '#End Region

            '#Region "MAILER"

            If mailerName <> "" Then
                fString += Convert.ToString(vbLf & "MAILER:") & mailerName
            End If

            '#End Region

            Return fString
        End Function

        Private Function getTelphoneNumberTypeAsString(ByVal telphoneNumberType As TELEPHONE_NUMBER_TYPE) As String
            Select Case telphoneNumberType
                Case TELEPHONE_NUMBER_TYPE.BBS
                    Return "bss"
                Case TELEPHONE_NUMBER_TYPE.CAR
                    Return "car"
                Case TELEPHONE_NUMBER_TYPE.CELL
                    Return "cell"
                Case TELEPHONE_NUMBER_TYPE.FAX
                    Return "fax"
                Case TELEPHONE_NUMBER_TYPE.HOME
                    Return "home"
                Case TELEPHONE_NUMBER_TYPE.ISDN
                    Return "isdn"
                Case TELEPHONE_NUMBER_TYPE.MODEM
                    Return "modem"
                Case TELEPHONE_NUMBER_TYPE.MSG
                    Return "msg"
                Case TELEPHONE_NUMBER_TYPE.PAGER
                    Return "pager"
                Case TELEPHONE_NUMBER_TYPE.PCS
                    Return "pcs"
                Case TELEPHONE_NUMBER_TYPE.PREFERRED
                    Return "pref"
                Case TELEPHONE_NUMBER_TYPE.VIDEO
                    Return "video"
                Case TELEPHONE_NUMBER_TYPE.VOICE
                    Return "voice"
                Case TELEPHONE_NUMBER_TYPE.WORK
                    Return "work"
                Case Else
                    Return "voice"
            End Select
        End Function

        Private Function getEmailTypeAsString(ByVal emailType As EMAIL_TYPE) As String
            Select Case emailType
                Case EMAIL_TYPE.INTERNET
                    Return "internet"
                Case EMAIL_TYPE.PREFERRED
                    Return "pref"
                Case EMAIL_TYPE.X400
                    Return "x400"
                Case Else
                    Return "internet"
            End Select
        End Function
    End Class

    Public Class ExplanatoryType
        Private _url As String

        Public Property url() As String
            Get
                Return _url
            End Get
            Set(ByVal value As String)
                _url = value
            End Set
        End Property

        Public Function getFormattedString() As String
            Dim fString As String = ""

            'URL
            fString = Convert.ToString("URL:") & url

            Return fString
        End Function
    End Class

    Class VCard
        'private string firstname

        Public Shared Function escapeString(ByVal text As String) As String
            Return escapeString(text, New Char() {","c, ";"c})
        End Function

        Public Shared Function escapeString(ByVal text As String, ByVal characters As Char()) As String
            Dim parsedString As String = ""
            Dim tempString As String = ""

            If text = "" Then
                Return text
            End If

            If characters Is Nothing OrElse characters.Length < 1 Then
                Return text
            End If

            parsedString = text

            For c As Integer = 0 To characters.Length - 1
                tempString = parsedString

                Dim indx As Integer = 0
                While indx < tempString.Length
                    If tempString(indx) = characters(c) Then
                        tempString = tempString.Insert(indx, "\")
                        indx += 2
                    Else
                        indx += 1
                    End If
                End While

                parsedString = tempString
            Next

            Return parsedString
        End Function
    End Class
End Namespace

