﻿Imports Mifare_NDEF_Sample_Code.Mifare_Change_Key
Imports Mifare_NDEF_Sample_Code.Acs.Readers.Pcsc
Imports Mifare_NDEF_Sample_Code.Acs.SmartCard.Reader.PCSC.Nfc
Imports System.Text

Public Class FormMain

    Private _readerFunctions As ReaderFunctions
    Private _mifareClassic As MifareClassic
    Private _cardSelector As ACS.Readers.Pcsc.CardSelector

    Dim _mifareTransportKey As Byte() = New Byte() {&HFF, &HFF, &HFF, &HFF, &HFF, &HFF}
    Dim _mifareTransportAccessBits As Byte() = New Byte() {&HFF, &H7, &H80, &H69}

    Dim _ndefPublicKeyAMadSector As Byte() = New Byte() {&HA0, &HA1, &HA2, &HA3, &HA4, &HA5}
    Dim _ndefPublicKeyANfcForumSector As Byte() = New Byte() {&HD3, &HF7, &HD3, &HF7, &HD3, &HF7}

    Dim _ndefSectorTrailerMadSector As Byte() = New Byte() {&H78, &H77, &H88, &HC1}
    Dim _ndefSectorTrailerNfcForumSector As Byte() = New Byte() {&H7F, &H7, &H88, &H40}

    Dim _cardName As String = ""

    Private Sub addTitleToLog(ByVal isBold As Boolean, ByVal msg As String)
        If isBold Then
            RichTextBoxApduLogs.SelectionFont = New Font(RichTextBoxApduLogs.Font.Name, RichTextBoxApduLogs.Font.Size, FontStyle.Bold)
        Else
            RichTextBoxApduLogs.SelectionFont = New Font(RichTextBoxApduLogs.Font.Name, RichTextBoxApduLogs.Font.Size, FontStyle.Regular)
        End If

        RichTextBoxApduLogs.[Select](RichTextBoxApduLogs.Text.Length, 0)
        RichTextBoxApduLogs.SelectionColor = Color.Black
        RichTextBoxApduLogs.SelectedText = vbCr & vbLf & msg & vbCr & vbLf
        RichTextBoxApduLogs.ScrollToCaret()
    End Sub

    Private Sub addMsgToLog(ByVal prefix As String, ByVal tmpBytes As Byte(), ByVal len As Integer)
        Dim msg As String = Helper.byteAsString(tmpBytes, 0, len, True)

        addMsgToLog(prefix & msg)
    End Sub

    Private Sub addMsgToLog(ByVal msg As String)
        RichTextBoxApduLogs.[Select](RichTextBoxApduLogs.Text.Length, 0)
        RichTextBoxApduLogs.SelectedText = msg & vbCr & vbLf
        RichTextBoxApduLogs.ScrollToCaret()
    End Sub
    Private Sub showWarningMessage(ByVal message As String)
        MessageBox.Show(message, "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning)
    End Sub

    Private Sub showErrorMessage(ByVal message As String)
        MessageBox.Show(message, "Error", MessageBoxButtons.OK, MessageBoxIcon.[Error])
    End Sub

    Private Sub enableControls(ByVal isEnable As Boolean)
        TabControlMain.Enabled = isEnable
        ButtonWriteToCard.Enabled = isEnable
        ButtonReadFromCard.Enabled = isEnable
        ButtonResetCard.Enabled = isEnable
        MaskedTextBoxMadKeyB.Enabled = isEnable
        MaskedTextBoxNfcForumSectorKeyB.Enabled = isEnable
    End Sub

    Private Sub readerFucntions_OnSendCommand(ByVal sender As Object, ByVal e As TransmitApduEventArg)
        addMsgToLog("<< ", e.data, e.data.Length)
    End Sub

    Private Sub readerFucntions_OnReceivedCommand(ByVal sender As Object, ByVal e As TransmitApduEventArg)
        addMsgToLog(">> ", e.data, e.data.Length)
    End Sub


    Private Sub ButtonInitialize_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonInitialize.Click
        Dim readerList As String()

        Try
            'resetFields()

            _readerFunctions = New ReaderFunctions()

            'Register to event OnReceivedCommand
            AddHandler _readerFunctions.OnReceivedCommand, AddressOf readerFucntions_OnReceivedCommand

            'Register to event OnSendCommand
            AddHandler _readerFunctions.OnSendCommand, AddressOf readerFucntions_OnSendCommand

            'Get all smart card reader connected to computer
            readerList = _readerFunctions.getReaderList()

            ComboBoxReaderList.Items.Clear()
            ComboBoxReaderList.Items.AddRange(readerList)

            If (ComboBoxReaderList.Items.Count > 0) Then
                ComboBoxReaderList.SelectedIndex = 0
            End If

            addMsgToLog(" ")
            addMsgToLog(vbCr & vbLf & "Initialize success")

            ButtonConnect.Enabled = True
        Catch pcscException As Acs.PcscException
            addTitleToLog(False, "[" + pcscException.errorCode.ToString() + "] " + pcscException.Message)
            showErrorMessage("[" + pcscException.errorCode.ToString() + "] " + pcscException.Message)
        Catch generalException As Exception
            addMsgToLog(generalException.Message)
            showErrorMessage(generalException.Message)
        End Try
    End Sub

    Private Sub ButtonConnect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonConnect.Click
        Dim atr As Byte() = Nothing

        Try
            If ComboBoxReaderList.SelectedItem Is Nothing Then
                showErrorMessage("Please select a reader.")
                Return
            End If

            _readerFunctions.connect(ComboBoxReaderList.SelectedItem.ToString())
            addMsgToLog(vbLf & "Successfully connected to " + ComboBoxReaderList.Text)

            'Initialize Mifare classic class
            _mifareClassic = New MifareClassic(_readerFunctions.pcscConnection)


            _readerFunctions.getStatusChange(atr)
            _cardSelector = New ACS.Readers.Pcsc.CardSelector
            _cardSelector.pcscReader = _readerFunctions

            _cardName = _cardSelector.readCardType(atr, CByte(atr.Length))

            If _cardName <> "Mifare Standard 1K" AndAlso _cardName <> "Mifare Standard 4K" Then
                showErrorMessage("Card not supported." & vbCr & vbLf & "Please present Mifare Classic card")

                enableControls(False)

                Return
            End If

            addMsgToLog("Chip Type: " + _cardName)

            enableControls(True)

        Catch pcscException As Acs.PcscException
            addTitleToLog(False, "[" + pcscException.errorCode.ToString() + "] " + pcscException.Message)
            showErrorMessage("[" + pcscException.errorCode.ToString() + "] " + pcscException.Message)
        Catch generalException As Exception
            addMsgToLog(generalException.Message)
            showErrorMessage(generalException.Message)
        End Try
    End Sub

    Private Sub ButtonSmartPosterEncodeMessage_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonSmartPosterEncodeMessage.Click
        If TextBoxSmartPosterURI.Text.Trim() = "" Then
            showErrorMessage("Please key-in Smart Poster URI.")
            TextBoxSmartPosterURI.Focus()
            Return
        End If

        If ComboBoxSmartPosterPrefixURI.SelectedIndex = 5 AndAlso Not isValidTel(TextBoxSmartPosterURI.Text.Trim()) Then
            showErrorMessage("Please key-in correct tel number.")
            TextBoxSmartPosterURI.Focus()
            Return
        End If

        encodeToNdefSmartPoster()

    End Sub

    Private Sub ButtonUriEncodeUri_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonUriEncodeUri.Click
        If TextBoxUriUri.Text.Trim() = "" Then
            showErrorMessage("Please key-in URI.")
            TextBoxUriUri.Focus()
            Return
        End If

        If ComboBoxURIPrefix.SelectedIndex = 5 AndAlso Not isValidTel(TextBoxUriUri.Text.Trim()) Then
            showErrorMessage("Please key-in correct tel number.")
            TextBoxUriUri.Focus()
            Return
        End If

        encodeToNdefUri()
    End Sub

    Private Sub ButtonTextEncodeText_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonTextEncodeText.Click
        If TextBoxTextMessage.Text.Trim() = "" Then
            showErrorMessage("Please key-in Message.")
            TextBoxTextMessage.Focus()
            Return
        End If

        encodeToNdefMessage()
    End Sub

    Private Sub ButtonEncodevCard_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonEncodevCard.Click
        If TextBoxGivenName.Text.Trim() = "" Then
            showErrorMessage("Please key-in Given Name.")
            TextBoxGivenName.Focus()
            Return
        End If

        If TextBoxLastName.Text.Trim() = "" Then
            showErrorMessage("Please key-in Surname.")
            TextBoxLastName.Focus()
            Return
        End If

        If Not isValidName(TextBoxGivenName.Text.Trim()) Then
            showErrorMessage("Please key-in correct Given Name.")
            TextBoxGivenName.Focus()
            Return
        End If

        If Not isValidName(TextBoxMiddleName.Text.Trim()) Then
            showErrorMessage("Please key-in correct Middle Name.")
            TextBoxMiddleName.Focus()
            Return
        End If

        If Not isValidName(TextBoxLastName.Text.Trim()) Then
            showErrorMessage("Please key-in correct Surname.")
            TextBoxLastName.Focus()
            Return
        End If

        encodeVCard()
    End Sub

    Private Sub ButtonWriteToCard_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonWriteToCard.Click
        writeToCard()
    End Sub

    Private Sub ButtonReadFromCard_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonReadFromCard.Click
        readFromCard()
    End Sub

    Private Sub ButtonResetCard_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonResetCard.Click
        resetCard()
    End Sub

    Private Sub ButtonClearLogs_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonClearLogs.Click
        RichTextBoxApduLogs.Text = ""
    End Sub

    Private Sub ButtonReset_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonReset.Click
        resetFields()
    End Sub

    Private Sub ButtonExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonExit.Click
        Me.Close()
    End Sub

    Private Sub ComboBoxSmartPosterPrefixURI_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ComboBoxSmartPosterPrefixURI.SelectedIndexChanged
        TextBoxSmartPosterURI.Text = ""
    End Sub

    Private Sub TextBoxSmartPosterURI_Leave(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TextBoxSmartPosterURI.Leave
        If TextBoxSmartPosterURI.Text.StartsWith(ComboBoxSmartPosterPrefixURI.Text) Then
            TextBoxSmartPosterURI.Text = TextBoxSmartPosterURI.Text.Remove(0, ComboBoxSmartPosterPrefixURI.Text.Length)
        End If
    End Sub

    Private Sub ComboBoxURIPrefix_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ComboBoxURIPrefix.SelectedIndexChanged
        TextBoxUriUri.Text = ""
    End Sub

    Private Sub TextBoxUriUri_Leave(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TextBoxUriUri.Leave
        If TextBoxUriUri.Text.StartsWith(ComboBoxURIPrefix.Text) Then
            TextBoxUriUri.Text = TextBoxUriUri.Text.Remove(0, ComboBoxURIPrefix.Text.Length)
        End If
    End Sub

    Private Sub TextBoxGivenName_KeyPress(ByVal sender As System.Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles TextBoxMiddleName.KeyPress, TextBoxLastName.KeyPress, TextBoxGivenName.KeyPress
        e.Handled = Not (Char.IsLetter(e.KeyChar) OrElse e.KeyChar = CChar(ChrW(Keys.Back)) _
                         OrElse e.KeyChar = CChar(ChrW(Keys.Delete)) OrElse _
                         e.KeyChar = CChar(ChrW(Keys.OemPeriod)) OrElse _
                         e.KeyChar = CChar(ChrW(Keys.[Decimal])) OrElse e.KeyChar = CChar(ChrW(Keys.Space)) _
                         OrElse e.KeyChar = "-"c)

    End Sub

    Private Sub encodeToNdefSmartPoster()
        'To make it nullable append '?'
        Dim act As System.Nullable(Of ACTION_RECORD) = Nothing

        Select Case ComboBoxSmartPosterAction.SelectedIndex
            Case 1
                act = ACTION_RECORD.DO_THE_ACTION
                Exit Select
            Case 2
                act = ACTION_RECORD.SAVE_FOR_LATER
                Exit Select
            Case 3
                act = ACTION_RECORD.OPEN_FOR_EDITING
                Exit Select
            Case Else
                act = Nothing
                Exit Select
        End Select

        Dim buffer As Byte() = Ndef.encodeSmartPoster(ComboBoxSmartPosterTitleLanguage.Text, TextBoxSmartPosterTitle.Text, CType(ComboBoxSmartPosterPrefixURI.SelectedIndex, URI_IDENTIFIER), TextBoxSmartPosterURI.Text, act)

        If buffer Is Nothing OrElse buffer.Length < 1 Then
            RichTextBoxEncodedNdefMessage.Text = ""
            Return
        End If


        RichTextBoxEncodedNdefMessage.Text = Helper.byteAsString(buffer, False)

    End Sub

    Private Sub encodeToNdefMessage()
        Dim ndefMessage As NdefMessage
        Dim tmpRecord As NdefRecord
        Dim statusByte As Byte = 0
        Dim buffer As Byte()

        If RadioButtonUtf16.Checked Then
            statusByte = &H80
        End If

        'Length of the Language (ISO/IANA language code)
        'ex. "en-US"
        statusByte = statusByte Or CByte(ComboBoxTextLanguage.Text.Length)

        tmpRecord = New NdefRecord(New NdefRecordType(TYPE_NAME_FORMAT.NFC_WELL_KNOWN_TYPE, "T"))
        tmpRecord.messageBegin = True
        tmpRecord.messageEnd = True
        tmpRecord.appendPayload(TextBoxTextMessage.Text)

        tmpRecord.payLoad = New Byte() {statusByte}
        tmpRecord.payLoad = Helper.appendArrays(tmpRecord.payLoad, ASCIIEncoding.ASCII.GetBytes(ComboBoxTextLanguage.Text))
        tmpRecord.payLoad = Helper.appendArrays(tmpRecord.payLoad, ASCIIEncoding.ASCII.GetBytes(TextBoxTextMessage.Text))

        ndefMessage = New NdefMessage()
        ndefMessage.appendRecord(tmpRecord)

        buffer = ndefMessage.toByteArray()

        RichTextBoxEncodedNdefMessage.Text = Helper.byteAsString(buffer, False)

    End Sub

    Private Sub encodeToNdefUri()
        Dim ndefMessage As NdefMessage
        Dim tmpRecord As NdefRecord
        Dim buffer As Byte()

        'Encode URI
        tmpRecord = New NdefRecord(New NdefRecordType(TYPE_NAME_FORMAT.NFC_WELL_KNOWN_TYPE, "U"))

        tmpRecord.messageBegin = True
        tmpRecord.messageEnd = True

        'URI Prefix
        'See URI Record Type Definition 
        'http://www.nfc-forum.org
        tmpRecord.appendPayload(CByte(DirectCast(ComboBoxURIPrefix.SelectedIndex, URI_IDENTIFIER)))
        tmpRecord.appendPayload(TextBoxUriUri.Text)

        ndefMessage = New NdefMessage()
        ndefMessage.appendRecord(tmpRecord)

        buffer = ndefMessage.toByteArray()

        RichTextBoxEncodedNdefMessage.Text = Helper.byteAsString(buffer, False)
    End Sub

    Private Sub encodeVCard()
        Dim formattedString As String = "BEGIN:VCARD" & vbCr & vbLf & "VERSION:3.0"
        Dim uBuffer As Byte()
        Dim ndef As NdefRecord
        Dim ndefMessage As NdefMessage

        Dim identificationType As New IdentificationType()
        identificationType.givenName = TextBoxGivenName.Text
        identificationType.familyName = TextBoxLastName.Text
        identificationType.additionalName = TextBoxMiddleName.Text

        formattedString += vbCr & vbLf + identificationType.getFormattedString()

        formattedString += vbCr & vbLf & "END:VCARD" & vbCr & vbLf & vbCr & vbLf

        addTitleToLog(False, formattedString)

        uBuffer = ASCIIEncoding.ASCII.GetBytes(formattedString)

        ndef = New NdefRecord(New NdefRecordType(TYPE_NAME_FORMAT.MEDIA_TYPE, "text/x-vCard"))
        ndef.appendPayload(uBuffer)

        ndefMessage = New NdefMessage()
        ndefMessage.appendRecord(ndef)

        RichTextBoxEncodedNdefMessage.Text = ""
        RichTextBoxEncodedNdefMessage.Text = Helper.byteAsString(ndefMessage.toByteArray(), False)
    End Sub

    Private Sub resetCard()
        Dim madKeyB As Byte(), nfcKeyB As Byte()
        Dim transportValue As Byte() = New Byte() {&HFF, &HFF, &HFF, &HFF, &HFF, &HFF, _
         &HFF, &H7, &H80, &H69, &HFF, &HFF, _
         &HFF, &HFF, &HFF, &HFF}
        Dim numberOfSectors As Byte = 15
        Dim sectorTrailer As Byte = 0
        Const KEY_LENGTH As Integer = 6
        Const BLOCK_DATA_LENGTH As Integer = 16
        Const MIFARE_1K_SECTOR_COUNT As Integer = 15
        Const MIFARE_4K_SECTOR_COUNT As Integer = 39
        Const MIFARE_4K_SECTOR_WITH_4_BLOCKS As Integer = 31
        Const MIFARE_4K_EXTENDED_USER_DATA_COUNT As Integer = 15
        Const MIFARE_USER_DATA_PER_SECTOR As Integer = 3
        Const MIFARE_4K_START_WITH_15_BLOCKS As Integer = 128
        Const MIFARE_4K_START_SECTOR_WITH_15_BLOCKS As Integer = 32
        Const BLOCK_COUNT As Integer = 4
        Const MAD_SECTORS_COUNT As Integer = 2
        'const int DELAY = 2000;

        Try
            If MaskedTextBoxMadKeyB.Text.Trim() = "" Then
                showErrorMessage("Please key-in hex value for MAD Key B.")
                MaskedTextBoxMadKeyB.Focus()
                Return
            End If

            If MaskedTextBoxMadKeyB.Text.Trim().Replace(" ", "").Length <> 12 Then
                showErrorMessage("Invalid input length. Length must be 6 bytes.")
                MaskedTextBoxMadKeyB.Focus()
                Return
            End If

            'madKeyB = Helper.getBytes(MaskedTextBoxMadKeyB.Text);
            madKeyB = getBytes(MaskedTextBoxMadKeyB.Text.Trim(), " "c)

            If madKeyB Is Nothing Then
                showErrorMessage("Please key-in hex value for MAD Key B.")
                MaskedTextBoxMadKeyB.Focus()
                Return
            End If

            If madKeyB.Length <> KEY_LENGTH Then
                showErrorMessage("Invalid input length. Length must be 6 bytes.")
                MaskedTextBoxMadKeyB.Focus()
                Return
            End If

            If MaskedTextBoxNfcForumSectorKeyB.Text.Trim() = "" Then
                showErrorMessage("Please key-in hex value for NFC Forum Key B")
                MaskedTextBoxNfcForumSectorKeyB.Focus()
                Return
            End If

            If MaskedTextBoxNfcForumSectorKeyB.Text.Trim().Replace(" ", "").Length <> 12 Then
                showErrorMessage("Invalid input length. Length must be 6 bytes.")
                MaskedTextBoxNfcForumSectorKeyB.Focus()
                Return
            End If

            nfcKeyB = getBytes(MaskedTextBoxNfcForumSectorKeyB.Text, " "c)

            If nfcKeyB Is Nothing Then
                showErrorMessage("Invalid input length. Length must be 6 bytes.")
                MaskedTextBoxNfcForumSectorKeyB.Focus()
                Return
            End If

            If nfcKeyB.Length <> KEY_LENGTH Then
                showErrorMessage("Invalid input length. Length must be 6 bytes.")
                MaskedTextBoxNfcForumSectorKeyB.Focus()
                Return
            End If

            addTitleToLog(False, "Load Authentication Keys")
            addTitleToLog(False, "Load NFC Forum Secret Key B to slot 1")

            'Load NFC Forum Sector Secret Key B to device's memory slot number 1
            If Not _readerFunctions.loadAuthKey(madKeyB, &H0) Then
                showErrorMessage("Load Key Failed")
                Return
            End If

            addTitleToLog(False, "Load Default Key to slot 2")
            'Load Mifare Transport Configuration Key (Default Key) to device's memory slot number 2
            If Not _readerFunctions.loadAuthKey(_mifareTransportKey, &H1) Then
                showErrorMessage("Load Key Failed")
                Return
            End If

            addTitleToLog(False, "Reset MAD Sector 1")

            For i As Integer = 0 To MAD_SECTORS_COUNT - 1
                'Authenticate using MAD Sector Secret Key B
                addTitleToLog(False, "Authenticate using MAD Sector Key B")
                If i = 0 Then
                    If Not _readerFunctions.authenticate(&H3, KEYTYPES.ACR122_KEYTYPE_B, &H0) Then
                        Continue For
                    End If
                Else
                    'If failed, assume that the sector is using the transport configuration keys
                    'FF FF FF FF FF FF
                    If Not _readerFunctions.authenticate(&H3, KEYTYPES.ACR122_KEYTYPE_A, &H1) Then
                        Continue For
                    End If
                End If

                If _mifareClassic.updateBinary(&H3, transportValue, &H10) = 0 Then
                    Exit For
                End If
            Next

            If _cardName = "Mifare Standard 4K" Then
                addTitleToLog(False, "Reset MAD Sector 2")

                For i As Integer = 0 To MAD_SECTORS_COUNT - 1
                    'Authenticate using MAD Sector Secret Key B
                    If i = 0 Then
                        If Not _readerFunctions.authenticate(&H63, KEYTYPES.ACR122_KEYTYPE_B, &H0) Then
                            Continue For
                        End If
                    Else
                        'If failed, assume that the sector is using the transport configuration keys
                        'FF FF FF FF FF FF
                        If Not _readerFunctions.authenticate(&H63, KEYTYPES.ACR122_KEYTYPE_A, &H1) Then
                            Continue For
                        End If
                    End If

                    If _mifareClassic.updateBinary(&H63, transportValue, &H10) = 0 Then
                        Exit For
                    End If
                Next
            End If

            If _cardName = "Mifare Standard 4K" Then
                numberOfSectors = MIFARE_4K_SECTOR_COUNT
                ' 39 sectors
            ElseIf _cardName = "Mifare Standard 1K" Then
                numberOfSectors = MIFARE_1K_SECTOR_COUNT
            End If
            ' 15 sectors
            For i As Integer = 1 To numberOfSectors
                If i <= MIFARE_4K_SECTOR_WITH_4_BLOCKS Then
                    sectorTrailer = CByte((i * BLOCK_COUNT) + MIFARE_USER_DATA_PER_SECTOR)
                Else
                    sectorTrailer = CByte(MIFARE_4K_START_WITH_15_BLOCKS + ((i - MIFARE_4K_START_SECTOR_WITH_15_BLOCKS) * BLOCK_DATA_LENGTH) + MIFARE_4K_EXTENDED_USER_DATA_COUNT)
                End If

                addTitleToLog(False, "Reset Sector: " + i.ToString())

                If Not _readerFunctions.authenticate(sectorTrailer, KEYTYPES.ACR122_KEYTYPE_B, &H0) Then
                    Continue For
                End If

                _mifareClassic.updateBinary(sectorTrailer, transportValue, &H10)

                'delay for ACR122U reader
                'System.Threading.Thread.Sleep(DELAY);
            Next
        Catch cardException As CardException
            addTitleToLog(False, "[" + Helper.byteAsString(cardException.statusWord, True) + "] " + cardException.Message)

            showErrorMessage(cardException.Message)
        Catch pcscException As Acs.PcscException
            addTitleToLog(False, "[" + pcscException.errorCode.ToString() + "] " + pcscException.Message)

            showErrorMessage("[" + pcscException.errorCode.ToString() + "] " + pcscException.Message)
        Catch generalException As Exception
            addMsgToLog(generalException.Message)
            showErrorMessage(generalException.Message)
        End Try
    End Sub

    Private Sub readFromCard()
        Dim numberOfSectors As Byte
        Dim currentSector As Byte = 1
        Dim physicalBlock As Byte = 0
        Dim buffer As Byte() = New Byte(-1) {}
        Dim tmpArray As Byte()
        Dim endIndex As Integer = 0
        Dim temporaryLen As Integer
        Dim temporaryPayLoadLen As Integer
        Dim payLoadTypeNameStr As String = ""
        Dim vCard As String()
        Dim vCardInfo As String()
        Dim delimeter As String() = New String() {vbCr & vbLf}
        Const DELAY As Integer = 2000
        Dim ndefMessage As NdefMessage
        Dim temporaryNdefMessage As NdefMessage
        Dim ndefRecords As List(Of NdefRecord)
        Dim temporaryNdefRecord As NdefRecord
        Const MAD_SECTOR2_ADDRESS As Integer = 16
        Const MIFARE_4K_SECTOR_WITH_4_BLOCKS As Integer = 31
        Const MIFARE_4K_EXTENDED_USER_DATA_COUNT As Integer = 15
        Const MIFARE_4K_START_WITH_15_BLOCKS As Integer = 128
        Const MIFARE_4K_START_SECTOR_WITH_15_BLOCKS As Integer = 32
        Const THREE_BYTE_MESSAGE_TAG As Integer = 4
        Const TWO_BYTE_MESSAGE_TAG As Integer = 2
        Const BLOCK_COUNT As Integer = 4
        Const BLOCK_LENGTH As Integer = 16

        Try
            'Load NFC Forum Sector Secret Key B to device's memory slot number 1
            addTitleToLog(False, "Load MAD Sector Public Key")
            If Not _readerFunctions.loadAuthKey(_ndefPublicKeyAMadSector, &H0) Then
                showErrorMessage("Load Key Failed")
                Return
            End If

            'Load Mifare Transport Configuration Key (Default Key) to device's memory slot number 2
            addTitleToLog(False, "Load NFC Forum Sector Public Key")
            If Not _readerFunctions.loadAuthKey(_ndefPublicKeyANfcForumSector, &H1) Then
                showErrorMessage("Load Key Failed")
                Return
            End If

            RichTextBoxEncodedNdefMessage.Text = ""

            'Read MAD Sector 1
            'Authenticate using the MAD public key
            'That is previously loaded on the device
            addTitleToLog(False, "Authenticate MAD1 using MAD public key")
            If Not _readerFunctions.authenticate(&H1, KEYTYPES.ACR122_KEYTYPE_A, &H0) Then
                Throw New Exception("Invalid Card")
            End If

            addTitleToLog(False, "Read MAD1 block 1")
            tmpArray = _mifareClassic.readBinary(&H1, &H10)
            buffer = Helper.appendArrays(buffer, tmpArray)

            addTitleToLog(False, "Read MAD1 block 2")
            tmpArray = _mifareClassic.readBinary(&H2, &H10)
            buffer = Helper.appendArrays(buffer, tmpArray)


            If _cardName = "Mifare Standard 4K" Then
                'Read MAD Sector 2

                'Authenticate using the MAD public key
                'That is previously loaded on the device
                addTitleToLog(False, "Authenticate MAD2 using MAD public key")
                _readerFunctions.authenticate(&H40, KEYTYPES.ACR122_KEYTYPE_A, &H0)

                addTitleToLog(False, "Read MAD2 block 64")
                tmpArray = _mifareClassic.readBinary(&H40, &H10)
                buffer = Helper.appendArrays(buffer, tmpArray)

                addTitleToLog(False, "Read MAD2 block 65")
                tmpArray = _mifareClassic.readBinary(&H41, &H10)
                buffer = Helper.appendArrays(buffer, tmpArray)

                addTitleToLog(False, "Read MAD2 block 66")
                tmpArray = _mifareClassic.readBinary(&H42, &H10)
                buffer = Helper.appendArrays(buffer, tmpArray)
            End If

            numberOfSectors = 0

            For i As Integer = 0 To buffer.Length - 1 Step 2
                If Helper.byteArrayIsEqual(New Byte() {buffer(i), buffer(i + 1)}, New Byte() {&H3, &HE1}) Then
                    numberOfSectors += 1
                End If
            Next

            buffer = New Byte(-1) {}
            currentSector = 1

            'delay for ACR122U reader
            If ComboBoxReaderList.Text.Contains("ACR122") Then
                System.Threading.Thread.Sleep(DELAY)
            End If

            For i As Integer = 0 To numberOfSectors - 1
                'Skip MAD Sector 2
                If currentSector = MAD_SECTOR2_ADDRESS Then
                    currentSector += 1
                End If

                If currentSector <= MIFARE_4K_SECTOR_WITH_4_BLOCKS Then
                    physicalBlock = CByte(currentSector * BLOCK_COUNT)
                    _readerFunctions.authenticate(physicalBlock, KEYTYPES.ACR122_KEYTYPE_A, &H1)

                    For j As Integer = 0 To (BLOCK_COUNT - 1) - 1
                        tmpArray = _mifareClassic.readBinary(CByte(physicalBlock + j), &H10)
                        buffer = Helper.appendArrays(buffer, tmpArray)
                    Next
                Else
                    physicalBlock = CByte(MIFARE_4K_START_WITH_15_BLOCKS + ((currentSector - MIFARE_4K_START_SECTOR_WITH_15_BLOCKS) * BLOCK_LENGTH))
                    _readerFunctions.authenticate(physicalBlock, KEYTYPES.ACR122_KEYTYPE_A, &H1)

                    For j As Integer = 0 To (MIFARE_4K_EXTENDED_USER_DATA_COUNT) - 1
                        tmpArray = _mifareClassic.readBinary(CByte(physicalBlock + j), &H10)
                        buffer = Helper.appendArrays(buffer, tmpArray)
                    Next
                End If

                currentSector += 1
            Next

            'Check if NDEF Message TAG is present
            If buffer(0) <> CByte(TAGS.NDEF_MESSAGE_TLV) Then
                Throw New Exception("NDEF Message TAG is not found")
            End If

            'Find the Terminator TLV
            For endIndex = 0 To buffer.Length - 1
                If buffer(endIndex) = &HFE Then
                    Exit For
                End If

                If (endIndex + 1) >= buffer.Length Then
                    Throw New Exception("Invalid Card." & vbCr & vbLf & "Terminator Tag is not found")
                End If
            Next

            Array.Resize(buffer, endIndex)

            'Remove NDEF Message Tag
            'Check if it uses one byte or 3 byte format for the length
            If buffer(1) = &HFF Then
                'Uses 3 bytes format
                tmpArray = New Byte(buffer.Length - THREE_BYTE_MESSAGE_TAG - 1) {}
                Array.Copy(buffer, THREE_BYTE_MESSAGE_TAG, tmpArray, 0, tmpArray.Length)
            Else
                tmpArray = New Byte(buffer.Length - TWO_BYTE_MESSAGE_TAG - 1) {}
                Array.Copy(buffer, TWO_BYTE_MESSAGE_TAG, tmpArray, 0, tmpArray.Length)
            End If


            RichTextBoxEncodedNdefMessage.Text = Helper.byteAsString(tmpArray, False)

            ndefMessage = NdefRecord.getNestedNdefMessage(0, tmpArray)

            If ndefMessage.getNumberOfRecords() < 0 Then
                showErrorMessage("No record to display")
                Return
            End If

            ndefRecords = ndefMessage.getRecords()

            payLoadTypeNameStr = ndefRecords(0).recordType.typeName

            If payLoadTypeNameStr = "Sp" Then
                temporaryNdefMessage = ndefRecords(0).getNestedNdefMessage()

                If temporaryNdefMessage.getNumberOfRecords() < 1 Then
                    Throw New Exception("Invalid Smart Poster NDEF Message")
                End If

                'Get URI Record (Mandatory)
                temporaryNdefRecord = temporaryNdefMessage.getRecord(New NdefRecordType(TYPE_NAME_FORMAT.NFC_WELL_KNOWN_TYPE, "U"))
                If temporaryNdefRecord Is Nothing Then
                    showErrorMessage("Mandatory URI Record is not found")
                    Return
                End If

                ComboBoxSmartPosterPrefixURI.SelectedIndex = temporaryNdefRecord.payLoad(0)
                TextBoxSmartPosterURI.Text = ASCIIEncoding.ASCII.GetString(temporaryNdefRecord.payLoad.Skip(1).Take(temporaryNdefRecord.payLoad.Length - 1).ToArray())

                'Get Action Record
                ComboBoxSmartPosterAction.SelectedIndex = 0
                temporaryNdefRecord = temporaryNdefMessage.getRecord(New NdefRecordType(TYPE_NAME_FORMAT.NFC_WELL_KNOWN_TYPE, "act"))
                If temporaryNdefRecord IsNot Nothing Then
                    Select Case CType(temporaryNdefRecord.payLoad(0), ACTION_RECORD)
                        Case ACTION_RECORD.DO_THE_ACTION
                            If True Then
                                ComboBoxSmartPosterAction.SelectedIndex = 1
                                Exit Select
                            End If
                        Case ACTION_RECORD.SAVE_FOR_LATER
                            If True Then
                                ComboBoxSmartPosterAction.SelectedIndex = 2
                                Exit Select
                            End If
                        Case ACTION_RECORD.OPEN_FOR_EDITING
                            If True Then
                                ComboBoxSmartPosterAction.SelectedIndex = 3
                                Exit Select
                            End If
                        Case Else
                            If True Then
                                ComboBoxSmartPosterAction.SelectedIndex = 0
                                Exit Select
                            End If
                    End Select
                Else
                    ComboBoxSmartPosterAction.SelectedIndex = 0
                End If

                'Get Text Record
                temporaryNdefRecord = temporaryNdefMessage.getRecord(New NdefRecordType(TYPE_NAME_FORMAT.NFC_WELL_KNOWN_TYPE, "T"))
                TextBoxSmartPosterTitle.Text = ""
                If temporaryNdefRecord IsNot Nothing Then
                    temporaryLen = temporaryNdefRecord.payLoad(0) And &H1F
                    ComboBoxSmartPosterTitleLanguage.Text = ASCIIEncoding.ASCII.GetString(temporaryNdefRecord.payLoad.Skip(1).Take(temporaryLen).ToArray())

                    temporaryPayLoadLen = temporaryNdefRecord.payLoad.Length - (temporaryLen - 1)

                    TextBoxSmartPosterTitle.Text = ASCIIEncoding.ASCII.GetString(temporaryNdefRecord.payLoad.Skip(temporaryLen + 1).Take(temporaryPayLoadLen).ToArray())
                End If

                MessageBox.Show("Smart Poster Message Found", "Information")

                TabControlMain.SelectedIndex = 0
            ElseIf payLoadTypeNameStr = "T" Then
                'Get Text Record
                temporaryNdefRecord = ndefRecords(0)

                If (temporaryNdefRecord.payLoad(0) And &H80) <> &H0 Then
                    RadioButtonUtf16.Checked = True
                Else
                    RadioButtonUtf8.Checked = True
                End If

                temporaryLen = temporaryNdefRecord.payLoad(0) And &H1F
                ComboBoxTextLanguage.Text = ASCIIEncoding.ASCII.GetString(temporaryNdefRecord.payLoad.Skip(1).Take(temporaryLen).ToArray())

                temporaryPayLoadLen = temporaryNdefRecord.payLoad.Length - (temporaryLen - 1)
                TextBoxTextMessage.Text = ASCIIEncoding.ASCII.GetString(temporaryNdefRecord.payLoad.Skip(temporaryLen + 1).Take(temporaryPayLoadLen).ToArray())

                MessageBox.Show("Text Message Found", "Information")

                TabControlMain.SelectedIndex = 2
            ElseIf payLoadTypeNameStr = "U" Then
                temporaryNdefRecord = ndefRecords(0)
                ComboBoxURIPrefix.SelectedIndex = temporaryNdefRecord.payLoad(0)
                TextBoxUriUri.Text = ASCIIEncoding.ASCII.GetString(temporaryNdefRecord.payLoad.Skip(1).Take(temporaryNdefRecord.payLoad.Length - 1).ToArray())

                MessageBox.Show("URI Message Found", "Information")
                TabControlMain.SelectedIndex = 1
            ElseIf payLoadTypeNameStr = "text/x-vCard" Then
                temporaryNdefRecord = ndefRecords(0)
                vCard = ASCIIEncoding.ASCII.GetString(temporaryNdefRecord.payLoad).Split(delimeter, StringSplitOptions.None)
                vCardInfo = vCard(2).Split(New Char() {";"c})
                TextBoxLastName.Text = vCardInfo(0).Substring(2, vCardInfo(0).Length - 2)
                TextBoxGivenName.Text = vCardInfo(1)
                TextBoxMiddleName.Text = vCardInfo(2)

                MessageBox.Show("vCard Message Found", "Information")
                TabControlMain.SelectedIndex = 3
            Else
                MessageBox.Show("Unsupported payload type", "Information")
            End If
        Catch cardException As CardException
            addTitleToLog(False, "[" + Helper.byteAsString(cardException.statusWord, True) + "] " + cardException.Message)

            showErrorMessage(cardException.Message)
        Catch pcscException As Acs.PcscException
            addTitleToLog(False, "[" + pcscException.errorCode.ToString() + "] " + pcscException.Message)

            showErrorMessage("[" + pcscException.errorCode.ToString() + "] " + pcscException.Message)
        Catch generalException As Exception
            addMsgToLog(generalException.Message)
            showErrorMessage(generalException.Message)
        End Try
    End Sub

    Private Sub writeToCard()
        Dim madSector As Byte() = New Byte(-1) {}
        Dim dataToWrite As Byte() = New Byte(-1) {}
        Dim madKeyB As Byte(), nfcKeyB As Byte()
        Dim arrLength As Byte()
        Dim buffer As Byte()
        Dim numberOfNfcForumSector As Byte = 0
        Dim numberOfBlocksNeeded As Byte = 0
        Dim lastSectorAuthenticated As Byte = 0
        Dim physicalBlockToWrite As Byte = &H4
        Dim sectorTrailer As Byte = &H3
        Dim tmpArray As Byte()
        Const KEY_LENGTH As Integer = 6
        Const ACCESS_BITS_LENGTH As Integer = 4
        Const DATA_LENGTH As Integer = 255
        Const MIFARE_WITH_4_BLOCKS_SECTOR_COUNT As Integer = 32
        Const MAD_SECTOR2_LENGTH As Integer = 48
        Const BLOCK_DATA_LENGTH As Integer = 16
        Const MIFARE_WITH_4_BLOCKS_TOTAL_COUNT As Integer = 127
        Const MIFARE_4K_TOTAL_SECTOR As Integer = 38
        Const MIFARE_1K_TOTAL_SECTOR As Integer = 35
        Const MAD2_SECTOR_TRAILER_BLOCK As Integer = 67
        Const MIFARE_4K_EXTENDED_USER_DATA_COUNT As Integer = 15
        Const MIFARE_USER_DATA_COUNT_PER_SECTOR As Integer = 3
        Const AIDS_COUNT As Integer = 15
        Const MAD2_SECTOR_START As Integer = 64
        Const MAD2_SECTOR_LAST As Integer = 68
        Const BLOCK_COUNT As Integer = 4
        Const COPY_LENGTH As Integer = 2
        Const DELAY As Integer = 2000
        Const BLOCK_LENGTH As Double = 16.0
        Const MIFARE_4K_EXTENDED_USER_DATA_COUNT_DOUBLE As Double = 15.0
        Const MIFARE_USER_BLOCK As Double = 3.0

        Dim dataLength As Integer
        Dim lenToCopy As Integer

        Try
            If RichTextBoxEncodedNdefMessage.Text.Trim() = "" Then
                showErrorMessage("There is no NDEF message to write. " & vbCr & vbLf & "Please encode NDEF message first.")
                Return
            End If

            dataToWrite = Helper.getBytes(RichTextBoxEncodedNdefMessage.Text)

            If MaskedTextBoxMadKeyB.Text.Trim() = "" Then
                showErrorMessage("Please key-in hex value for MAD Key B")
                MaskedTextBoxMadKeyB.Focus()
                Return
            End If

            If MaskedTextBoxMadKeyB.Text.Trim().Replace(" ", "").Length <> 12 Then
                showErrorMessage("Invalid input length. Length must be 6 bytes.")
                MaskedTextBoxMadKeyB.Focus()
                Return
            End If

            madKeyB = getBytes(MaskedTextBoxMadKeyB.Text, " "c)

            If madKeyB Is Nothing Then
                showErrorMessage("Invalid input length. Length must be 6 bytes.")
                MaskedTextBoxMadKeyB.Focus()
                Return
            End If

            If madKeyB.Length <> KEY_LENGTH Then
                showErrorMessage("Invalid input length. Length must be 6 bytes.")
                MaskedTextBoxMadKeyB.Focus()
                Return
            End If

            If MaskedTextBoxNfcForumSectorKeyB.Text.Trim() = "" Then
                showErrorMessage("Please key-in hex value for NFC Forum Key B.")
                MaskedTextBoxNfcForumSectorKeyB.Focus()
                Return
            End If

            If MaskedTextBoxNfcForumSectorKeyB.Text.Trim().Replace(" ", "").Length <> 12 Then
                showErrorMessage("Invalid input length. Length must be 6 bytes.")
                MaskedTextBoxNfcForumSectorKeyB.Focus()
                Return
            End If

            nfcKeyB = getBytes(MaskedTextBoxNfcForumSectorKeyB.Text, " "c)

            If nfcKeyB Is Nothing Then
                showErrorMessage("Invalid input length. Length must be 6 bytes.")
                MaskedTextBoxNfcForumSectorKeyB.Focus()
                Return
            End If

            If nfcKeyB.Length <> KEY_LENGTH Then
                showErrorMessage("Invalid input length. Length must be 6 bytes.")
                MaskedTextBoxNfcForumSectorKeyB.Focus()
                Return
            End If

            addTitleToLog(False, "Load Authentication Key")
            _readerFunctions.loadAuthKey(_mifareTransportKey, &H0)

            'delay for ACR122U reader
            If ComboBoxReaderList.Text.Contains("ACR122") Then
                System.Threading.Thread.Sleep(DELAY)
            End If

            'Form the data to be written to the card
            buffer = New Byte(-1) {}
            If dataToWrite.Length < DATA_LENGTH Then
                'If data is less than or equal to 254 (0xFE) use one byte format
                'Refere to NXP Type MF1K/4K Tag Operation document
                'http://www.nxp.com/
                buffer = Helper.appendArrays(buffer, New Byte() {CByte(TAGS.NDEF_MESSAGE_TLV), CByte(dataToWrite.Length)})
            Else
                'If data is greater than 254 (0xFE) use 3 bytes format
                dataLength = dataToWrite.Length

                'In 3 bytes format the last 2 bytes is for 00FE - FFFF                    
                dataLength -= (DATA_LENGTH - 1)

                arrLength = New Byte(2) {}

                'Indicates that two more bytes folow
                arrLength(0) = &HFF
                arrLength(1) = CByte((dataLength And &HFF00) >> 8)
                arrLength(2) = CByte(dataLength And &HFF)

                'NDEF Message TAG
                buffer = Helper.appendArrays(buffer, CByte(TAGS.NDEF_MESSAGE_TLV))

                'length

                buffer = Helper.appendArrays(buffer, arrLength)
            End If

            buffer = Helper.appendArrays(buffer, dataToWrite)

            'Append Terminator TLV
            buffer = Helper.appendArrays(buffer, CByte(TAGS.TERMINATOR_TLV))

            'Write the data to Mifare Card
            'Start from NFC Form Sector 1 Block 0

            addTitleToLog(False, "Write NDEF Message To Card")

            'Get mumber of blocks needed
            ' MAD sector blocks not included
            numberOfBlocksNeeded = CByte(CDbl(Math.Ceiling(buffer.Length / BLOCK_LENGTH)))
            physicalBlockToWrite = &H4
            'Start to write at sector 1 block 1 (Physical Block 0x04)

            Dim indx As Integer = 0
            While indx < numberOfBlocksNeeded
                tmpArray = New Byte(BLOCK_DATA_LENGTH - 1) {}
                lenToCopy = BLOCK_DATA_LENGTH

                'Check if remaining data to be copied is less than 16
                If (buffer.Length - (indx * BLOCK_DATA_LENGTH)) > BLOCK_DATA_LENGTH Then
                    lenToCopy = BLOCK_DATA_LENGTH
                Else
                    lenToCopy = buffer.Length - (indx * BLOCK_DATA_LENGTH)
                End If

                Array.Copy(buffer, (indx * BLOCK_DATA_LENGTH), tmpArray, 0, lenToCopy)


                'Check if sector trailer
                'If sector trailer skip
                If physicalBlockToWrite > MIFARE_WITH_4_BLOCKS_TOTAL_COUNT Then
                    If ((physicalBlockToWrite - MIFARE_4K_EXTENDED_USER_DATA_COUNT) Mod (MIFARE_4K_EXTENDED_USER_DATA_COUNT + 1)) = 0 Then
                        physicalBlockToWrite += 1
                    End If
                Else
                    If ((physicalBlockToWrite - MIFARE_USER_DATA_COUNT_PER_SECTOR) Mod BLOCK_COUNT) = 0 Then
                        physicalBlockToWrite += 1
                    End If

                    'Check if MAD Sector 2 (Sector 16)
                    'if MAD Sector skip
                    If physicalBlockToWrite >= MAD2_SECTOR_START AndAlso physicalBlockToWrite <= MAD2_SECTOR_LAST Then
                        addMsgToLog("Skip MAD Sector : " + (physicalBlockToWrite \ BLOCK_COUNT).ToString())
                        'Move to physical block 68 (Sector 17))
                        physicalBlockToWrite = MAD2_SECTOR_LAST + 1
                    End If
                End If

                If physicalBlockToWrite > MIFARE_WITH_4_BLOCKS_TOTAL_COUNT Then
                    If lastSectorAuthenticated <> (physicalBlockToWrite \ BLOCK_DATA_LENGTH) Then
                        addTitleToLog(False, "Authenticate Sector: " + (((physicalBlockToWrite - MIFARE_WITH_4_BLOCKS_TOTAL_COUNT) \ BLOCK_DATA_LENGTH) + MIFARE_WITH_4_BLOCKS_SECTOR_COUNT).ToString())
                        If Not _readerFunctions.authenticate(physicalBlockToWrite, KEYTYPES.ACR122_KEYTYPE_A, &H0) Then
                            Throw New Exception("Unable to authenticate in sector " + (physicalBlockToWrite \ BLOCK_DATA_LENGTH).ToString())
                        End If

                        lastSectorAuthenticated = CByte(physicalBlockToWrite \ BLOCK_DATA_LENGTH)
                    End If
                Else

                    If lastSectorAuthenticated <> (physicalBlockToWrite \ BLOCK_COUNT) Then
                        addTitleToLog(False, "Authenticate Sector: " + (physicalBlockToWrite \ BLOCK_COUNT).ToString())
                        If Not _readerFunctions.authenticate(physicalBlockToWrite, KEYTYPES.ACR122_KEYTYPE_A, &H0) Then
                            Throw New Exception("Unable to authenticate in sector " + (physicalBlockToWrite \ BLOCK_COUNT).ToString())
                        End If

                        lastSectorAuthenticated = CByte(physicalBlockToWrite \ BLOCK_COUNT)
                    End If
                End If


                addTitleToLog(False, "Write in block " + physicalBlockToWrite.ToString())


                _mifareClassic.updateBinary(physicalBlockToWrite, tmpArray, &H10)
                indx += 1
                physicalBlockToWrite += 1
            End While

            'Initialize MAD Sector
            'If number of sector blocks needed is greater than 127 (31 sectors)
            '32nd sector have 15 user blocks (0 - 31 sector have 3 user blocks)
            If numberOfBlocksNeeded > MIFARE_WITH_4_BLOCKS_TOTAL_COUNT Then
                numberOfNfcForumSector = (MIFARE_WITH_4_BLOCKS_SECTOR_COUNT - 1)
                numberOfNfcForumSector += CByte(Math.Ceiling((numberOfBlocksNeeded - MIFARE_WITH_4_BLOCKS_TOTAL_COUNT) / MIFARE_4K_EXTENDED_USER_DATA_COUNT_DOUBLE))
            Else
                numberOfNfcForumSector = CByte(Math.Ceiling(numberOfBlocksNeeded / MIFARE_USER_BLOCK))
            End If

            madSector = New Byte(MIFARE_WITH_4_BLOCKS_SECTOR_COUNT - 1) {}
            For i As Integer = 0 To numberOfNfcForumSector - 1
                Array.Copy(New Byte() {&H3, &HE1}, 0, madSector, ((i * COPY_LENGTH) + COPY_LENGTH), COPY_LENGTH)

                'Mad Sector 1 contains 15 AIDs (0-14 = 15)
                'the rest of the AIDs will be written in MAD Sector 2
                If i = (AIDS_COUNT - 1) Then
                    Exit For
                End If
            Next

            'Initialize MAD Sector 1

            addTitleToLog(False, "Initialize MAD Sector 1")
            addTitleToLog(False, "Authenticate MAD Sector 1")

            'Write AIDs to MAD Sector 1
            _readerFunctions.authenticate(&H1, KEYTYPES.ACR122_KEYTYPE_A, &H0)

            'Write the first 16 bytes (first 7 AIDs) to block 1 of MAD Sector 1 (Sector 0)
            addTitleToLog(False, "Write first 7 AIDs to block 1 of MAD1")
            _mifareClassic.updateBinary(&H1, madSector.Take(BLOCK_DATA_LENGTH).ToArray(), &H10)

            'Write next 16 bytes (next 8 AIDs) to block 2 of MAD Sector 1 (Sector 0)
            addTitleToLog(False, "Write next 8 AIDs to block 2 of MAD1")
            _mifareClassic.updateBinary(&H2, madSector.Skip(BLOCK_DATA_LENGTH).Take(BLOCK_DATA_LENGTH).ToArray(), &H10)

            'Initialize MAD Sector 2

            'If number of sectors needed (AIDs) is greater than 15 then write the remaining
            'AIDs in the MAD Sector 2 (Sector 16 of the Mifare Card)
            'MAD Sector 2 can store up to 23 AIDs
            If numberOfNfcForumSector >= (MIFARE_1K_TOTAL_SECTOR + 1) AndAlso numberOfNfcForumSector <= MIFARE_4K_TOTAL_SECTOR Then
                addTitleToLog(False, "Initialize MAD Sector 2")

                madSector = New Byte(MAD_SECTOR2_LENGTH - 1) {}

                For i As Integer = AIDS_COUNT To numberOfNfcForumSector - 1
                    Array.Copy(New Byte() {&H3, &HE1}, 0, madSector, ((i - AIDS_COUNT) * COPY_LENGTH), COPY_LENGTH)
                Next


                'Write AIDs to MAD Sector 2 (Sector 16 of the Mifare Card)
                addTitleToLog(False, "Authenticate MAD Sector 2")
                _readerFunctions.authenticate(&H40, KEYTYPES.ACR122_KEYTYPE_A, &H0)

                'Write the first 16 bytes (first 7 AIDs) to block 0 of MAD Sector 2 (Sector 16 of the Mifare Card)
                addTitleToLog(False, "Write first 7 AIDs to block 0 of MAD2")
                _mifareClassic.updateBinary(&H40, madSector.Take(BLOCK_DATA_LENGTH).ToArray(), &H10)

                'Write next 16 bytes (next 8 AIDs) to block 1 of MAD Sector 2
                addTitleToLog(False, "Write next 8 AIDs to block 1 of MAD2")
                _mifareClassic.updateBinary(&H41, madSector.Skip(BLOCK_DATA_LENGTH).Take(BLOCK_DATA_LENGTH).ToArray(), &H10)

                'Write next 16 bytes (next 8 AIDs) to block 1 of MAD Sector 2
                addTitleToLog(False, "Write next 8 AIDs to block 2 of MAD2")
                _mifareClassic.updateBinary(&H42, madSector.Skip(BLOCK_DATA_LENGTH * 2).Take(BLOCK_DATA_LENGTH).ToArray(), &H10)
            End If

            addTitleToLog(False, "Change Sector Keys")

            'Update Sector Keys and Access Bits                
            buffer = New Byte(BLOCK_DATA_LENGTH - 1) {}

            'Key A
            Array.Copy(_ndefPublicKeyANfcForumSector, 0, buffer, 0, KEY_LENGTH)

            'Access Bits
            Array.Copy(_ndefSectorTrailerNfcForumSector, 0, buffer, KEY_LENGTH, ACCESS_BITS_LENGTH)

            'Key B
            Array.Copy(nfcKeyB, 0, buffer, KEY_LENGTH + ACCESS_BITS_LENGTH, KEY_LENGTH)

            'Update NFC Forum Sector Keys
            'Start from sector 1
            sectorTrailer = &H3
            For i As Integer = 1 To numberOfNfcForumSector
                If i <= (MIFARE_WITH_4_BLOCKS_SECTOR_COUNT - 1) Then
                    sectorTrailer += BLOCK_COUNT
                Else
                    sectorTrailer += (MIFARE_4K_EXTENDED_USER_DATA_COUNT + 1)
                End If

                'Check if MAD Sector 2 (Sector 16, Trailer Block = Phycal Block 67)
                'Skip if MAD Sector 2 trailer
                If sectorTrailer = MAD2_SECTOR_TRAILER_BLOCK Then
                    'Move to next sector trailer
                    sectorTrailer += BLOCK_COUNT
                End If

                If sectorTrailer < MAD2_SECTOR_TRAILER_BLOCK Then
                    addTitleToLog(False, "Update Trailer Block of NFC Forum Sector: " + i.ToString())
                Else
                    addTitleToLog(False, "Update Trailer Block of NFC Forum Sector: " + (i + 1).ToString())
                End If

                _readerFunctions.authenticate(sectorTrailer, KEYTYPES.ACR122_KEYTYPE_A, &H0)

                _mifareClassic.updateBinary(sectorTrailer, buffer, &H10)
            Next

            'Update MAD Sector 1 Keys
            addTitleToLog(False, "Update MAD Sector 1 Trailer")

            _readerFunctions.authenticate(&H3, KEYTYPES.ACR122_KEYTYPE_A, &H0)

            buffer = New Byte(BLOCK_DATA_LENGTH - 1) {}

            'Key A
            Array.Copy(_ndefPublicKeyAMadSector, 0, buffer, 0, KEY_LENGTH)

            'Access Bits
            Array.Copy(_ndefSectorTrailerMadSector, 0, buffer, KEY_LENGTH, ACCESS_BITS_LENGTH)

            'Key B
            Array.Copy(madKeyB, 0, buffer, KEY_LENGTH + ACCESS_BITS_LENGTH, KEY_LENGTH)

            _mifareClassic.updateBinary(&H3, buffer, &H10)

            'If MAD Sector is greater than 15
            'Write the remaining AIDs in the MAD Sector 2
            If numberOfNfcForumSector > AIDS_COUNT Then
                'Update MAD Sector 2 Keys
                addTitleToLog(False, "Update MAD Sector 2 Trailer Block")

                _readerFunctions.authenticate(&H43, KEYTYPES.ACR122_KEYTYPE_A, &H0)

                buffer = New Byte(BLOCK_DATA_LENGTH - 1) {}

                'Key A
                Array.Copy(_ndefPublicKeyAMadSector, 0, buffer, 0, KEY_LENGTH)

                'Access Bits
                Array.Copy(_ndefSectorTrailerMadSector, 0, buffer, KEY_LENGTH, ACCESS_BITS_LENGTH)

                'Key B
                Array.Copy(madKeyB, 0, buffer, KEY_LENGTH + ACCESS_BITS_LENGTH, KEY_LENGTH)


                _mifareClassic.updateBinary(&H43, buffer, &H10)
            End If
        Catch cardException As CardException
            addTitleToLog(False, "[" + Helper.byteAsString(cardException.statusWord, True) + "] " + cardException.Message)

            showErrorMessage(cardException.Message)
        Catch pcscException As Acs.PcscException
            addTitleToLog(False, "[" + pcscException.errorCode.ToString() + "] " + pcscException.Message)

            showErrorMessage("[" + pcscException.errorCode.ToString() + "] " + pcscException.Message)
        Catch generalException As Exception
            addMsgToLog(generalException.Message)
            showErrorMessage(generalException.Message)
        End Try
    End Sub

    Private Sub resetFields()
        ComboBoxReaderList.Text = ""
        ComboBoxReaderList.Items.Clear()

        RichTextBoxEncodedNdefMessage.Text = ""

        ComboBoxSmartPosterAction.SelectedIndex = 0
        ComboBoxSmartPosterPrefixURI.SelectedIndex = 0
        ComboBoxSmartPosterTitleLanguage.SelectedIndex = 0
        ComboBoxTextLanguage.SelectedIndex = 0
        ComboBoxURIPrefix.SelectedIndex = 0

        TextBoxSmartPosterTitle.Text = "Advanced Card Systems Ltd."
        TextBoxSmartPosterURI.Text = "http://www.acs.com.hk"
        TextBoxUriUri.Text = "http://www.acs.com.hk"
        TextBoxTextMessage.Text = "Hello World!"
        TextBoxGivenName.Text = "Given Name"
        TextBoxMiddleName.Text = "Middle Name"
        TextBoxLastName.Text = "Last Name"
        MaskedTextBoxMadKeyB.Text = "FFFFFFFFFFFF"
        MaskedTextBoxNfcForumSectorKeyB.Text = "FFFFFFFFFFFF"

        RadioButtonUtf8.Checked = True
        RadioButtonUtf16.Checked = False

        ButtonConnect.Enabled = False

        enableControls(False)

        TabControlMain.SelectedIndex = 0

        RichTextBoxApduLogs.Clear()
        addMsgToLog("Program Ready")
    End Sub

    Private Sub FormMain_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        resetFields()
        TextBoxSmartPosterURI.Text = "http://www.acs.com.hk"
        TextBoxUriUri.Text = "http://www.acs.com.hk"
    End Sub

    Private Sub MaskedTextBoxMadKeyB_KeyDown(ByVal sender As System.Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles MaskedTextBoxMadKeyB.KeyDown, MaskedTextBoxNfcForumSectorKeyB.KeyDown
        If (Not (e.KeyCode >= Keys.A And e.KeyCode <= Keys.F) And Not (e.KeyCode >= Keys.D0 And e.KeyCode <= Keys.D9) And Not (e.KeyCode >= Keys.NumPad0 And e.KeyCode <= Keys.NumPad9)) Then
            e.SuppressKeyPress = True
        End If

    End Sub

    Private Function getBytes(ByVal stringBytes As String, ByVal delimeter As Char) As Byte()
        Dim counter As Integer = 0
        Dim tmpByte As Byte
        Dim arrayString As String() = stringBytes.Split(delimeter)
        Dim bytesResult As Byte() = New Byte(arrayString.Length - 1) {}

        For Each str As String In arrayString
            If Byte.TryParse(str, System.Globalization.NumberStyles.HexNumber, Nothing, tmpByte) Then
                bytesResult(counter) = tmpByte
                counter += 1
            Else
                Return Nothing
            End If
        Next

        Return bytesResult
    End Function

    Public Function isValidName(ByVal name As String) As Boolean
        For Each c As Char In name
            If Not [Char].IsLetter(c) AndAlso c <> " "c AndAlso c <> "."c AndAlso c <> "-"c Then
                Return False
            End If
        Next

        Return True
    End Function

    Public Function isValidTel(ByVal name As String) As Boolean
        For Each c As Char In name
            If Not [Char].IsNumber(c) AndAlso c <> "("c AndAlso c <> ")"c AndAlso c <> "-"c AndAlso c <> "+"c Then
                Return False
            End If
        Next

        Return True
    End Function

    Private Sub TextBoxSmartPosterURI_KeyPress(ByVal sender As System.Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles TextBoxSmartPosterURI.KeyPress
        If ComboBoxSmartPosterPrefixURI.SelectedIndex = 5 Then
            e.Handled = Not (Char.IsNumber(e.KeyChar) OrElse e.KeyChar = ControlChars.Back OrElse e.KeyChar = "-"c OrElse e.KeyChar = "+"c OrElse e.KeyChar = "("c OrElse e.KeyChar = ")"c)
        End If
    End Sub

    Private Sub TextBoxUriUri_KeyPress(ByVal sender As System.Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles TextBoxUriUri.KeyPress
        If ComboBoxURIPrefix.SelectedIndex = 5 Then
            e.Handled = Not (Char.IsNumber(e.KeyChar) OrElse e.KeyChar = ControlChars.Back OrElse e.KeyChar = "-"c OrElse e.KeyChar = "+"c OrElse e.KeyChar = "("c OrElse e.KeyChar = ")"c)
        End If
    End Sub
End Class
