﻿Imports Mifare_NDEF_Sample_Code.Mifare_Change_Key
Imports Mifare_NDEF_Sample_Code.Acs.Readers.Pcsc
Imports Mifare_NDEF_Sample_Code.Acs.SmartCard.Reader.PCSC.Nfc
Imports System.Text

Public Class FormMain

    Private _readerFunctions As ReaderFunctions
    Private _ntag203 As Ntag203
    Private _cardSelector As ACS.Readers.Pcsc.CardSelector

    Dim _cardName As String = ""
    Dim _maximumInputLength As Integer = 141

    Const BLOCK_3 As Byte = &H3
    Const WRITEREAD_LEN As Byte = &H10

    Private Sub addTitleToLog(ByVal isBold As Boolean, ByVal msg As String)
        If isBold Then
            RichTextBoxApduLogs.SelectionFont = New Font(RichTextBoxApduLogs.Font.Name, RichTextBoxApduLogs.Font.Size, FontStyle.Bold)
        Else
            RichTextBoxApduLogs.SelectionFont = New Font(RichTextBoxApduLogs.Font.Name, RichTextBoxApduLogs.Font.Size, FontStyle.Regular)
        End If

        RichTextBoxApduLogs.[Select](RichTextBoxApduLogs.Text.Length, 0)
        RichTextBoxApduLogs.SelectionColor = Color.Black
        RichTextBoxApduLogs.SelectedText = vbCr & vbLf & msg & vbCr & vbLf
        RichTextBoxApduLogs.ScrollToCaret()
    End Sub

    Private Sub addMsgToLog(ByVal prefix As String, ByVal tmpBytes As Byte(), ByVal len As Integer)
        Dim msg As String = Helper.byteAsString(tmpBytes, 0, len, True)

        addMsgToLog(prefix & msg)
    End Sub

    Private Sub addMsgToLog(ByVal msg As String)
        RichTextBoxApduLogs.[Select](RichTextBoxApduLogs.Text.Length, 0)
        RichTextBoxApduLogs.SelectedText = msg & vbCr & vbLf
        RichTextBoxApduLogs.ScrollToCaret()
    End Sub
    Private Sub showWarningMessage(ByVal message As String)
        MessageBox.Show(message, "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning)
    End Sub

    Private Sub showErrorMessage(ByVal message As String)
        MessageBox.Show(message, "Error", MessageBoxButtons.OK, MessageBoxIcon.[Error])
    End Sub

    Private Sub showInformationMessage(ByVal message As String)
        MessageBox.Show(message, "Information", MessageBoxButtons.OK, MessageBoxIcon.Information)
    End Sub

    Private Sub readerFucntions_OnSendCommand(ByVal sender As Object, ByVal e As TransmitApduEventArg)
        addMsgToLog("<< ", e.data, e.data.Length)
    End Sub

    Private Sub readerFucntions_OnReceivedCommand(ByVal sender As Object, ByVal e As TransmitApduEventArg)
        addMsgToLog(">> ", e.data, e.data.Length)
    End Sub


    Private Sub ButtonInitialize_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonInitialize.Click
        Dim readerList As String()

        Try
            _readerFunctions = New ReaderFunctions()

            'Register to event OnReceivedCommand
            AddHandler _readerFunctions.OnReceivedCommand, AddressOf readerFucntions_OnReceivedCommand

            'Register to event OnSendCommand
            AddHandler _readerFunctions.OnSendCommand, AddressOf readerFucntions_OnSendCommand

            'Get all smart card reader connected to computer
            readerList = _readerFunctions.getReaderList()

            ComboBoxReaderList.Items.Clear()
            ComboBoxReaderList.Items.AddRange(readerList)

            If (ComboBoxReaderList.Items.Count > 0) Then
                ComboBoxReaderList.SelectedIndex = 0
            End If

            addMsgToLog(" ")
            addMsgToLog(vbCr & vbLf & "Initialize success")

            ButtonConnect.Enabled = True
        Catch pcscException As Acs.PcscException
            addTitleToLog(False, "[" + pcscException.errorCode.ToString() + "] " + pcscException.Message)
            showErrorMessage("[" + pcscException.errorCode.ToString() + "] " + pcscException.Message)
        Catch generalException As Exception
            addMsgToLog(generalException.Message)
            showErrorMessage(generalException.Message)
        End Try
    End Sub

    Private Sub ButtonConnect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonConnect.Click
        Dim atr As Byte() = Nothing

        Try
            If ComboBoxReaderList.SelectedItem Is Nothing Then
                showErrorMessage("Please select a reader.")
                Return
            End If

            _readerFunctions.connect(ComboBoxReaderList.SelectedItem.ToString())
            addMsgToLog(vbLf & "Successfully connected to " + ComboBoxReaderList.Text)

            'Initialize NTAG class
            _ntag203 = New Ntag203(_readerFunctions.pcscConnection)

            _readerFunctions.getStatusChange(atr)
            _cardSelector = New ACS.Readers.Pcsc.CardSelector
            _cardSelector.pcscReader = _readerFunctions

            _cardName = _cardSelector.readCardType(atr, CByte(atr.Length))

            If _cardName <> "NTAG203" Then
                showErrorMessage("Card not supported." & vbCr & vbLf & "Please present NTAG203 card.")
                enableControls(False)
                Return
            End If

            addMsgToLog("Chip Type: " + _cardName)
            addMsgToLog("")

            enableControls(True)
        Catch pcscException As ACS.PcscException
            addTitleToLog(False, "[" + pcscException.errorCode.ToString() + "] " + pcscException.Message)
            showErrorMessage("[" + pcscException.errorCode.ToString() + "] " + pcscException.Message)
        Catch generalException As Exception
            addMsgToLog(generalException.Message)
            showErrorMessage(generalException.Message)
        End Try
    End Sub

    Private Sub ButtonSmartPosterEncodeMessage_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonSmartPosterEncodeMessage.Click
        If TextBoxSmartPosterURI.Text.Trim() = "" Then
            showErrorMessage("Please key-in Smart Poster URI.")
            TextBoxSmartPosterURI.Focus()
            Return
        End If

        If ComboBoxSmartPosterPrefixURI.SelectedIndex = 5 AndAlso Not isValidTel(TextBoxSmartPosterURI.Text.Trim()) Then
            showErrorMessage("Please key-in correct tel number.")
            TextBoxSmartPosterURI.Focus()
            Return
        End If

        encodeToNdefSmartPoster()

    End Sub

    Private Sub ButtonUriEncodeUri_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonUriEncodeUri.Click
        If TextBoxUriUri.Text.Trim() = "" Then
            showErrorMessage("Please key-in URI.")
            TextBoxUriUri.Focus()
            Return
        End If

        If ComboBoxUriPrefix.SelectedIndex = 5 AndAlso Not isValidTel(TextBoxUriUri.Text.Trim()) Then
            showErrorMessage("Please key-in correct tel number.")
            TextBoxUriUri.Focus()
            Return
        End If

        encodeToNdefUri()
    End Sub

    Private Sub ButtonTextEncodeText_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonTextEncodeText.Click
        If TextBoxTextMessage.Text.Trim() = "" Then
            showErrorMessage("Please key-in Message.")
            TextBoxTextMessage.Focus()
            Return
        End If

        encodeToNdefMessage()
    End Sub

    Private Sub ButtonEncodevCard_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonEncodevCard.Click
        If TextBoxGivenName.Text.Trim() = "" Then
            showErrorMessage("Please key-in Given Name.")
            TextBoxGivenName.Focus()
            Return
        End If

        If TextBoxLastName.Text.Trim() = "" Then
            showErrorMessage("Please key-in Surname.")
            TextBoxLastName.Focus()
            Return
        End If

        If Not isValidName(TextBoxGivenName.Text.Trim()) Then
            showErrorMessage("Please key-in correct Given Name.")
            TextBoxGivenName.Focus()
            Return
        End If

        If Not isValidName(TextBoxMiddleName.Text.Trim()) Then
            showErrorMessage("Please key-in correct Middle Name.")
            TextBoxMiddleName.Focus()
            Return
        End If

        If Not isValidName(TextBoxLastName.Text.Trim()) Then
            showErrorMessage("Please key-in correct Surname.")
            TextBoxLastName.Focus()
            Return
        End If


        encodeVCard()
    End Sub

    Private Sub ButtonWriteToCard_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonWriteToCard.Click
        writeToCard()
    End Sub

    Private Sub ButtonReadFromCard_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonReadFromCard.Click
        readFromCard()
    End Sub

    Private Sub ButtonClearLogs_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonClearLogs.Click
        RichTextBoxApduLogs.Text = ""
    End Sub

    Private Sub ButtonReset_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonReset.Click
        resetFields()
    End Sub

    Private Sub ButtonExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonExit.Click
        Me.Close()
    End Sub

    Private Sub ComboBoxSmartPosterPrefixURI_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ComboBoxSmartPosterPrefixURI.SelectedIndexChanged
        TextBoxSmartPosterURI.Text = ""
    End Sub

    Private Sub TextBoxSmartPosterURI_Leave(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TextBoxSmartPosterURI.Leave
        If TextBoxSmartPosterURI.Text.StartsWith(ComboBoxSmartPosterPrefixURI.Text) Then
            TextBoxSmartPosterURI.Text = TextBoxSmartPosterURI.Text.Remove(0, ComboBoxSmartPosterPrefixURI.Text.Length)
        End If
    End Sub

    Private Sub ComboBoxURIPrefix_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ComboBoxUriPrefix.SelectedIndexChanged
        TextBoxUriUri.Text = ""
    End Sub

    Private Sub TextBoxUriUri_Leave(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TextBoxUriUri.Leave
        If TextBoxUriUri.Text.StartsWith(ComboBoxUriPrefix.Text) Then
            TextBoxUriUri.Text = TextBoxUriUri.Text.Remove(0, ComboBoxUriPrefix.Text.Length)
        End If
    End Sub

    Private Sub TextBoxGivenName_KeyPress(ByVal sender As System.Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles TextBoxMiddleName.KeyPress, TextBoxLastName.KeyPress, TextBoxGivenName.KeyPress
        e.Handled = Not (Char.IsLetter(e.KeyChar) OrElse e.KeyChar = CChar(ChrW(Keys.Back)) _
                         OrElse e.KeyChar = CChar(ChrW(Keys.Delete)) OrElse _
                         e.KeyChar = CChar(ChrW(Keys.OemPeriod)) OrElse _
                         e.KeyChar = CChar(ChrW(Keys.[Decimal])) OrElse e.KeyChar = CChar(ChrW(Keys.Space)) _
                         OrElse e.KeyChar = "-"c)

    End Sub

    Private Sub encodeToNdefSmartPoster()
        Dim act As System.Nullable(Of ACTION_RECORD) = Nothing

        Select Case ComboBoxSmartPosterAction.SelectedIndex
            Case 1
                act = ACTION_RECORD.DO_THE_ACTION
                Exit Select
            Case 2
                act = ACTION_RECORD.SAVE_FOR_LATER
                Exit Select
            Case 3
                act = ACTION_RECORD.OPEN_FOR_EDITING
                Exit Select
            Case Else
                act = Nothing
                Exit Select
        End Select

        Dim buffer As Byte() = Ndef.encodeSmartPoster(ComboBoxSmartPosterTitleLanguage.Text, TextBoxSmartPosterTitle.Text, CType(ComboBoxSmartPosterPrefixURI.SelectedIndex, URI_IDENTIFIER), TextBoxSmartPosterURI.Text, act)

        If buffer Is Nothing OrElse buffer.Length < 1 Then
            RichTextBoxEncodedNdefMessage.Text = ""
            Return
        End If

        If buffer.Length > _maximumInputLength Then
            showErrorMessage([String].Format("Input characters exceeds maximum input length. Please remove {0} character/s from Title or URI.", buffer.Length - _maximumInputLength))
            Return
        End If

        RichTextBoxEncodedNdefMessage.Text = Helper.byteAsString(buffer, False)

    End Sub

    Private Sub encodeToNdefMessage()
        Dim ndefMessage As NdefMessage
        Dim tmpRecord As NdefRecord
        Dim statusByte As Byte = 0
        Dim buffer As Byte()

        If RadioButtonUtf16.Checked Then
            statusByte = &H80
        End If

        'Length of the Language (ISO/IANA language code)
        'ex. "en-US"
        statusByte = statusByte Or CByte(ComboBoxTextLanguage.Text.Length)

        tmpRecord = New NdefRecord(New NdefRecordType(TYPE_NAME_FORMAT.NFC_WELL_KNOWN_TYPE, "T"))
        tmpRecord.messageBegin = True
        tmpRecord.messageEnd = True
        tmpRecord.appendPayload(TextBoxTextMessage.Text)

        tmpRecord.payLoad = New Byte() {statusByte}
        tmpRecord.payLoad = Helper.appendArrays(tmpRecord.payLoad, ASCIIEncoding.ASCII.GetBytes(ComboBoxTextLanguage.Text))
        tmpRecord.payLoad = Helper.appendArrays(tmpRecord.payLoad, ASCIIEncoding.ASCII.GetBytes(TextBoxTextMessage.Text))

        ndefMessage = New NdefMessage()
        ndefMessage.appendRecord(tmpRecord)

        buffer = ndefMessage.toByteArray()

        If buffer.Length > _maximumInputLength Then
            showErrorMessage([String].Format("Input characters exceeds maximum input length. Please remove {0} characters from Message.", buffer.Length - _maximumInputLength))
            Return
        End If

        RichTextBoxEncodedNdefMessage.Text = Helper.byteAsString(buffer, False)

    End Sub

    Private Sub encodeToNdefUri()
        Dim ndefMessage As NdefMessage
        Dim tmpRecord As NdefRecord
        Dim buffer As Byte()

        'Encode URI
        tmpRecord = New NdefRecord(New NdefRecordType(TYPE_NAME_FORMAT.NFC_WELL_KNOWN_TYPE, "U"))

        tmpRecord.messageBegin = True
        tmpRecord.messageEnd = True

        'URI Prefix
        'See URI Record Type Definition 
        'http://www.nfc-forum.org
        tmpRecord.appendPayload(CByte(DirectCast(ComboBoxUriPrefix.SelectedIndex, URI_IDENTIFIER)))
        tmpRecord.appendPayload(TextBoxUriUri.Text)

        ndefMessage = New NdefMessage()
        ndefMessage.appendRecord(tmpRecord)

        buffer = ndefMessage.toByteArray()

        If buffer.Length > _maximumInputLength Then
            showErrorMessage([String].Format("Input characters exceeds maximum input length. Please remove {0} characters from URI.", buffer.Length - _maximumInputLength))
            Return
        End If

        RichTextBoxEncodedNdefMessage.Text = Helper.byteAsString(buffer, False)
    End Sub

    Private Sub encodeVCard()
        Dim formattedString As String = "BEGIN:VCARD" & vbCr & vbLf & "VERSION:3.0"
        Dim uBuffer As Byte()
        Dim buffer As Byte()
        Dim ndef As NdefRecord
        Dim ndefMessage As NdefMessage

        Dim identificationType As New IdentificationType()
        identificationType.givenName = TextBoxGivenName.Text
        identificationType.familyName = TextBoxLastName.Text
        identificationType.additionalName = TextBoxMiddleName.Text

        formattedString += vbCr & vbLf + identificationType.getFormattedString()

        formattedString += vbCr & vbLf & "END:VCARD" & vbCr & vbLf & vbCr & vbLf

        addTitleToLog(False, formattedString)

        uBuffer = ASCIIEncoding.ASCII.GetBytes(formattedString)

        ndef = New NdefRecord(New NdefRecordType(TYPE_NAME_FORMAT.MEDIA_TYPE, "text/x-vCard"))
        ndef.appendPayload(uBuffer)

        ndefMessage = New NdefMessage()
        ndefMessage.appendRecord(ndef)

        buffer = ndefMessage.toByteArray()

        If buffer.Length > _maximumInputLength Then
            showErrorMessage([String].Format("Input characters exceeds maximum input length. Please remove {0} characters from Name.", buffer.Length - _maximumInputLength))
            Return
        End If

        RichTextBoxEncodedNdefMessage.Text = ""
        RichTextBoxEncodedNdefMessage.Text = Helper.byteAsString(ndefMessage.toByteArray(), False)
    End Sub

    Private Sub readFromCard()
        Dim blockNumber As Byte = &H4  'starting block
        Dim buffer As Byte() = New Byte(-1) {}
        Dim tmpArray As Byte()
        Dim endIndex As Integer = 0
        Dim temporaryLen As Integer
        Dim temporaryPayLoadLen As Integer
        Dim numberOfBlocks As Integer = 0
        Dim payLoadTypeNameStr As String = ""
        Dim vCard As String()
        Dim vCardInfo As String()
        Dim delimeter As String() = New String() {vbCr & vbLf}
        Dim ndefMessage As NdefMessage
        Dim temporaryNdefMessage As NdefMessage
        Dim ndefRecords As List(Of NdefRecord)
        Dim temporaryNdefRecord As NdefRecord

        Const BLOCK_DATA_LENGTH As Integer = 16
        Const BLOCK_LENGTH As Double = 16.0
        Const THREE_BYTE_FORMAT_ADDTL As Integer = 5
        Const NON_THREE_BYTE_FORMAT_ADDTL As Integer = 3

        Try
            RichTextBoxEncodedNdefMessage.Text = ""

            buffer = New Byte(-1) {}

            tmpArray = _ntag203.readBinary(blockNumber, &H10)
            buffer = Helper.appendArrays(buffer, tmpArray)

            If buffer(1) = &HFF Then
                numberOfBlocks = CByte(CDbl(Math.Ceiling((CInt(buffer(1)) + THREE_BYTE_FORMAT_ADDTL) / BLOCK_LENGTH)))
            Else
                numberOfBlocks = CByte(CDbl(Math.Ceiling((CInt(buffer(1)) + NON_THREE_BYTE_FORMAT_ADDTL) / BLOCK_LENGTH)))
            End If

            blockNumber += 4

            Dim indx As Integer = 0
            While indx < numberOfBlocks - 1
                tmpArray = New Byte(BLOCK_DATA_LENGTH - 1) {}

                addTitleToLog(False, "Read block " + blockNumber.ToString())

                tmpArray = _ntag203.readBinary(blockNumber, &H10)
                buffer = Helper.appendArrays(buffer, tmpArray)
                indx += 1
                blockNumber += 4
            End While

            'Check if NDEF Message TAG is present
            If buffer(0) <> CByte(TAGS.NDEF_MESSAGE_TLV) Then
                Throw New Exception("NDEF Message TAG is not found")
            End If

            'Find the Terminator TLV
            For endIndex = 0 To buffer.Length - 1
                If buffer(endIndex) = CByte(TAGS.TERMINATOR_TLV) Then
                    Exit For
                End If

                If (endIndex + 1) >= buffer.Length Then
                    Throw New Exception("Invalid Card." & vbCr & vbLf & "Terminator Tag is not found")
                End If
            Next

            Array.Resize(buffer, endIndex)

            'Remove NDEF Message Tag
            'Check if it uses one byte or 3 byte format for the length
            If buffer(1) = &HFF Then
                'Uses 3 bytes format
                tmpArray = New Byte(buffer.Length - (THREE_BYTE_FORMAT_ADDTL - 1) - 1) {}
                Array.Copy(buffer, THREE_BYTE_FORMAT_ADDTL - 1, tmpArray, 0, tmpArray.Length)
            Else
                tmpArray = New Byte(buffer.Length - (NON_THREE_BYTE_FORMAT_ADDTL - 1) - 1) {}
                Array.Copy(buffer, NON_THREE_BYTE_FORMAT_ADDTL - 1, tmpArray, 0, tmpArray.Length)
            End If

            RichTextBoxEncodedNdefMessage.Text = Helper.byteAsString(tmpArray, False)

            ndefMessage = NdefRecord.getNestedNdefMessage(0, tmpArray)

            If ndefMessage.getNumberOfRecords() < 0 Then
                MessageBox.Show("No record to display")
                Return
            End If

            ndefRecords = ndefMessage.getRecords()

            payLoadTypeNameStr = ndefRecords(0).recordType.typeName

            If payLoadTypeNameStr = "Sp" Then
                temporaryNdefMessage = ndefRecords(0).getNestedNdefMessage()

                If temporaryNdefMessage.getNumberOfRecords() < 1 Then
                    Throw New Exception("Invalid SmartPoster NDEF Message")
                End If

                'Get URI Record (Mandatory)
                temporaryNdefRecord = temporaryNdefMessage.getRecord(New NdefRecordType(TYPE_NAME_FORMAT.NFC_WELL_KNOWN_TYPE, "U"))
                If temporaryNdefRecord Is Nothing Then
                    MessageBox.Show("Mandatory URI Record is not found")
                    Return
                End If

                ComboBoxSmartPosterPrefixURI.SelectedIndex = temporaryNdefRecord.payLoad(0)
                TextBoxSmartPosterURI.Text = ASCIIEncoding.ASCII.GetString(temporaryNdefRecord.payLoad.Skip(1).Take(temporaryNdefRecord.payLoad.Length - 1).ToArray())

                'Get Action Record
                ComboBoxSmartPosterAction.SelectedIndex = 0
                temporaryNdefRecord = temporaryNdefMessage.getRecord(New NdefRecordType(TYPE_NAME_FORMAT.NFC_WELL_KNOWN_TYPE, "act"))
                If temporaryNdefRecord IsNot Nothing Then
                    Select Case CType(temporaryNdefRecord.payLoad(0), ACTION_RECORD)
                        Case ACTION_RECORD.DO_THE_ACTION
                            If True Then
                                ComboBoxSmartPosterAction.SelectedIndex = 1
                                Exit Select
                            End If
                        Case ACTION_RECORD.SAVE_FOR_LATER
                            If True Then
                                ComboBoxSmartPosterAction.SelectedIndex = 2
                                Exit Select
                            End If
                        Case ACTION_RECORD.OPEN_FOR_EDITING
                            If True Then
                                ComboBoxSmartPosterAction.SelectedIndex = 3
                                Exit Select
                            End If
                        Case Else
                            If True Then
                                ComboBoxSmartPosterAction.SelectedIndex = 0
                                Exit Select
                            End If
                    End Select
                Else
                    ComboBoxSmartPosterAction.SelectedIndex = 0
                End If

                'Get Text Record
                temporaryNdefRecord = temporaryNdefMessage.getRecord(New NdefRecordType(TYPE_NAME_FORMAT.NFC_WELL_KNOWN_TYPE, "T"))
                If temporaryNdefRecord IsNot Nothing Then
                    temporaryLen = temporaryNdefRecord.payLoad(0) And &H1F
                    ComboBoxSmartPosterTitleLanguage.Text = ASCIIEncoding.ASCII.GetString(temporaryNdefRecord.payLoad.Skip(1).Take(temporaryLen).ToArray())

                    temporaryPayLoadLen = temporaryNdefRecord.payLoad.Length - (temporaryLen - 1)

                    TextBoxSmartPosterTitle.Text = ASCIIEncoding.ASCII.GetString(temporaryNdefRecord.payLoad.Skip(temporaryLen + 1).Take(temporaryPayLoadLen).ToArray())
                Else
                    TextBoxSmartPosterTitle.Text = ""
                End If

                showInformationMessage("Smart Poster Message Found")

                TabControlMain.SelectedIndex = 0
            ElseIf payLoadTypeNameStr = "T" Then
                'Get Text Record
                temporaryNdefRecord = ndefRecords(0)

                If (temporaryNdefRecord.payLoad(0) And &H80) <> &H0 Then
                    RadioButtonUtf16.Checked = True
                Else
                    RadioButtonUtf8.Checked = True
                End If

                temporaryLen = temporaryNdefRecord.payLoad(0) And &H1F
                ComboBoxTextLanguage.Text = ASCIIEncoding.ASCII.GetString(temporaryNdefRecord.payLoad.Skip(1).Take(temporaryLen).ToArray())

                temporaryPayLoadLen = temporaryNdefRecord.payLoad.Length - (temporaryLen - 1)
                TextBoxTextMessage.Text = ASCIIEncoding.ASCII.GetString(temporaryNdefRecord.payLoad.Skip(temporaryLen + 1).Take(temporaryPayLoadLen).ToArray())

                showInformationMessage("Text Message Found")

                TabControlMain.SelectedIndex = 2
            ElseIf payLoadTypeNameStr = "U" Then
                temporaryNdefRecord = ndefRecords(0)
                ComboBoxUriPrefix.SelectedIndex = temporaryNdefRecord.payLoad(0)
                TextBoxUriUri.Text = ASCIIEncoding.ASCII.GetString(temporaryNdefRecord.payLoad.Skip(1).Take(temporaryNdefRecord.payLoad.Length - 1).ToArray())

                showInformationMessage("URI Message Found")
                TabControlMain.SelectedIndex = 1
            ElseIf payLoadTypeNameStr = "text/x-vCard" Then
                temporaryNdefRecord = ndefRecords(0)
                vCard = ASCIIEncoding.ASCII.GetString(temporaryNdefRecord.payLoad).Split(delimeter, StringSplitOptions.None)
                vCardInfo = vCard(2).Split(New Char() {";"c})
                TextBoxLastName.Text = vCardInfo(0).Substring(2, vCardInfo(0).Length - 2)
                TextBoxGivenName.Text = vCardInfo(1)
                TextBoxMiddleName.Text = vCardInfo(2)

                showInformationMessage("vCard Message Found")
                TabControlMain.SelectedIndex = 3
            Else
                showInformationMessage("Unsupported payload type")

            End If
        Catch cardException As CardException
            addTitleToLog(False, "[" + Helper.byteAsString(cardException.statusWord, True) + "] " + cardException.Message)

            showErrorMessage(cardException.Message)
        Catch pcscException As Acs.PcscException
            addTitleToLog(False, "[" + pcscException.errorCode.ToString() + "] " + pcscException.Message)

            showErrorMessage("[" + pcscException.errorCode.ToString() + "] " + pcscException.Message)
        Catch generalException As Exception
            addMsgToLog(generalException.Message)
            showErrorMessage(generalException.Message)
        End Try
    End Sub

    Private Sub writeToCard()
        Dim dataToWrite As Byte() = New Byte(-1) {}
        Dim arrLength As Byte()
        Dim buffer As Byte()
        Dim tmpArray As Byte() = New Byte(15) {}
        Dim numberOfWritesNeeded As Byte = 0
        Dim blockNumber As Byte = &H4
        Dim dataLength As Integer
        Dim lenToCopy As Integer = 4

        Const DATA_LENGTH As Integer = 255
        Const BLOCK_DATA_LENGTH As Integer = 4
        Const BLOCK_LENGTH As Double = 4.0

        Try
            If RichTextBoxEncodedNdefMessage.Text.Trim() = "" Then
                showErrorMessage("There is no NDEF message to write. Please encode NDEF message first.")
                Return
            End If

            dataToWrite = Helper.getBytes(RichTextBoxEncodedNdefMessage.Text)

            If dataToWrite.Length > _maximumInputLength Then
                showErrorMessage("NDEF Message too long. NDEF length should be at most 141 bytes.")
                Return
            End If

            If dataToWrite.Length < 1 Then
                If RichTextBoxEncodedNdefMessage.Text.Trim() = "" Then
                    showErrorMessage("There is no NDEF message to write. Please encode NDEF message first.")
                    Return
                End If
            End If

            'Form the data to be written to the card
            buffer = New Byte(-1) {}
            If dataToWrite.Length < DATA_LENGTH Then
                'If data is less than or equal to 254 (0xFE) use one byte format
                'Refer to Type 2 Tag Operation document
                'http://www.nxp.com/
                buffer = Helper.appendArrays(buffer, New Byte() {CByte(TAGS.NDEF_MESSAGE_TLV), CByte(dataToWrite.Length)})
            Else
                'If data is greater than 254 (0xFE) use 3 bytes format
                dataLength = dataToWrite.Length

                'In 3 bytes format the last 2 bytes is for 00FE - FFFF                    
                dataLength -= 254

                arrLength = New Byte(2) {}

                'Indicates that two more bytes folow
                arrLength(0) = &HFF
                arrLength(1) = CByte((dataLength And &HFF00) >> 8)
                arrLength(2) = CByte(dataLength And &HFF)

                'NDEF Message TAG
                buffer = Helper.appendArrays(buffer, CByte(TAGS.NDEF_MESSAGE_TLV))

                'length
                buffer = Helper.appendArrays(buffer, arrLength)
            End If

            buffer = Helper.appendArrays(buffer, dataToWrite)

            'Append Terminator TLV
            buffer = Helper.appendArrays(buffer, CByte(TAGS.TERMINATOR_TLV))

            'Write the data to NTAG
            'Start from Block 4
            addTitleToLog(False, "Write NDEF Message To Card")

            'Get number of blocks needed
            numberOfWritesNeeded = CByte(CDbl(Math.Ceiling(buffer.Length / BLOCK_LENGTH)))
            blockNumber = &H4
            'Start to write at Block 4
            Dim indx As Integer = 0
            While indx < numberOfWritesNeeded
                tmpArray = New Byte(BLOCK_DATA_LENGTH - 1) {}
                lenToCopy = BLOCK_DATA_LENGTH

                'Check if remaining data to be copied is less than 16
                If (buffer.Length - (indx * BLOCK_DATA_LENGTH)) > BLOCK_DATA_LENGTH Then
                    lenToCopy = BLOCK_DATA_LENGTH
                Else
                    lenToCopy = buffer.Length - (indx * BLOCK_DATA_LENGTH)
                End If

                Array.Copy(buffer, (indx * BLOCK_DATA_LENGTH), tmpArray, 0, lenToCopy)

                addTitleToLog(False, "Write in block " + blockNumber.ToString())

                _ntag203.updateBinary(blockNumber, tmpArray, BLOCK_DATA_LENGTH)
                indx += 1
                blockNumber += (BLOCK_DATA_LENGTH / 4)
            End While
        Catch cardException As CardException
            addTitleToLog(False, "[" + Helper.byteAsString(cardException.statusWord, True) + "] " + cardException.Message)

            showErrorMessage(cardException.Message)
        Catch pcscException As Acs.PcscException
            addTitleToLog(False, "[" + pcscException.errorCode.ToString() + "] " + pcscException.Message)
            showErrorMessage("[" + pcscException.errorCode.ToString() + "] " + pcscException.Message)
        Catch generalException As Exception
            addMsgToLog(generalException.Message)
            showErrorMessage(generalException.Message)
        End Try
    End Sub

    Private Sub resetFields()
        ComboBoxReaderList.Text = ""
        ComboBoxReaderList.Items.Clear()

        ComboBoxSmartPosterAction.SelectedIndex = 0
        ComboBoxSmartPosterPrefixURI.SelectedIndex = 0
        ComboBoxSmartPosterTitleLanguage.SelectedIndex = 0
        ComboBoxTextLanguage.SelectedIndex = 0
        ComboBoxUriPrefix.SelectedIndex = 0

        TextBoxSmartPosterTitle.Text = "Advanced Card Systems Ltd."
        TextBoxSmartPosterURI.Text = "http://www.acs.com.hk"
        TextBoxUriUri.Text = "http://www.acs.com.hk"
        TextBoxTextMessage.Text = "Hello World!"
        TextBoxGivenName.Text = "Given Name"
        TextBoxMiddleName.Text = "Middle Name"
        TextBoxLastName.Text = "Surname"

        RichTextBoxEncodedNdefMessage.Clear()

        RadioButtonUtf8.Checked = True
        RadioButtonUtf16.Checked = False
        ButtonConnect.Enabled = False

        enableControls(False)

        TabControlMain.SelectedIndex = 0

        RichTextBoxApduLogs.Clear()
        addMsgToLog("Program Ready")
    End Sub

    Private Sub enableControls(ByVal isEnable As Boolean)
        TabControlMain.Enabled = isEnable
        ButtonWriteToCard.Enabled = isEnable
        ButtonReadFromCard.Enabled = isEnable
    End Sub

    Private Sub FormMain_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        resetFields()
    End Sub

    Private Sub ComboBoxTextLanguage_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ComboBoxTextLanguage.SelectedIndexChanged
        If ComboBoxTextLanguage.SelectedIndex = 1 Then
            TextBoxTextMessage.MaxLength = 131
        Else
            TextBoxTextMessage.MaxLength = 134
        End If

        If Not String.IsNullOrEmpty(TextBoxTextMessage.Text) AndAlso TextBoxTextMessage.Text.Length > TextBoxTextMessage.MaxLength Then
            TextBoxTextMessage.Text = TextBoxTextMessage.Text.Substring(0, TextBoxTextMessage.MaxLength)
        End If
    End Sub

    Public Function isValidName(ByVal name As String) As Boolean
        For Each c As Char In name
            If Not [Char].IsLetter(c) AndAlso c <> " "c AndAlso c <> "."c AndAlso c <> "-"c Then
                Return False
            End If
        Next

        Return True
    End Function

    Public Function isValidTel(ByVal name As String) As Boolean
        For Each c As Char In name
            If Not [Char].IsNumber(c) AndAlso c <> "("c AndAlso c <> ")"c AndAlso c <> "-"c AndAlso c <> "+"c Then
                Return False
            End If
        Next

        Return True
    End Function

    Private Sub TextBoxSmartPosterURI_KeyPress(ByVal sender As System.Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles TextBoxSmartPosterURI.KeyPress
        If ComboBoxSmartPosterPrefixURI.SelectedIndex = 5 Then
            e.Handled = Not (Char.IsNumber(e.KeyChar) OrElse e.KeyChar = ControlChars.Back OrElse e.KeyChar = "-"c OrElse e.KeyChar = "+"c OrElse e.KeyChar = "("c OrElse e.KeyChar = ")"c)
        End If
    End Sub

    Private Sub TextBoxUriUri_KeyPress(ByVal sender As System.Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles TextBoxUriUri.KeyPress
        If ComboBoxUriPrefix.SelectedIndex = 5 Then
            e.Handled = Not (Char.IsNumber(e.KeyChar) OrElse e.KeyChar = ControlChars.Back OrElse e.KeyChar = "-"c OrElse e.KeyChar = "+"c OrElse e.KeyChar = "("c OrElse e.KeyChar = ")"c)
        End If
    End Sub
End Class
