﻿
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Acs;
using Acs.Pcsc;
using Acs.Readers.Pcsc;

namespace NDEF
{
    public class Ntag203
    {

        public Ntag203(string readerName)
        {
            _pcscConnection = new PcscReader(readerName);
        }

        public Ntag203(PcscReader pcsc)
        {
            _pcscConnection = pcsc;
        }

        private PcscReader _pcscConnection;
        public PcscReader pcscConnection
        {
            get { return _pcscConnection; }
            set { _pcscConnection = value; }
        }
        
        private string getErrorMessage(byte[] sw1sw2)
        {
            if (sw1sw2.Length < 2)
                return "Unknown Status Word (" + Helper.byteAsString(sw1sw2, false) + ")";

            else if (sw1sw2[0] == 0x63 && sw1sw2[1] == 0x00)
                return "Command failed";
            else
                return "Unknown Status Word (" + Helper.byteAsString(sw1sw2, false) + ")";
        }

        public byte[] readBinary(byte blockNumber, byte length)
        {
            Apdu apdu;
            
            apdu = new Apdu();
            apdu.setCommand(new byte[] { 0xFF, 0xB0, 0x00, blockNumber, length });
            apdu.data = new byte[0];
            apdu.lengthExpected =length;

            pcscConnection.sendCommand(ref apdu);
            if (apdu.statusWord[0] != 0x90)
                throw new CardException("Read failed", apdu.statusWord);

            return apdu.response.Take(length).ToArray();
        }
        
        public void updateBinary(byte blockNumber, byte[] data, byte length)
        {
            Apdu apdu;

            if (data.Length != length)
                throw new Exception("Data has invalid length");

            if (length % 4 != 0)
            {
                throw new Exception("Data length must be multiple of 4");
            }

            apdu = new Apdu();
            apdu.setCommand(new byte[] { 0xFF, 0xD6, 0x00, blockNumber, length });

            apdu.data = new byte[data.Length];
            Array.Copy(data, apdu.data, length);

            pcscConnection.sendCommand(ref apdu);

            if (apdu.statusWord[0] != 0x90)
                throw new CardException("Update failed", apdu.statusWord);
        }

    }
}
