﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Acs;
using NDEF;
using Acs.Readers.Pcsc;
using Acs.Pcsc;
using Acs.SmartCard.Reader.Pcsc.Nfc;

namespace NDEF
{
    public partial class FormMain : Form
    {
        #region Global Variables

        Ntag203 _ntag203;
        ReaderFunctions _readerFunctions;
        CardSelector _cardSelector = new CardSelector();
        string _sCardName = "";
        //int _remainingSmartPosterCharacter = 0;
        //int _remainingVCardCharacter = 0;
        int _maximumInputLength = 141;

        const byte BLOCK_3 = 0x03;
        
        #endregion

        public FormMain()
        {
            InitializeComponent();
        }

        void addTitleToLog(bool isBold, string msg)
        {
            if (isBold)
                RichTextBoxApduLogs.SelectionFont = new Font(RichTextBoxApduLogs.Font.Name, RichTextBoxApduLogs.Font.Size, FontStyle.Bold);
            else
                RichTextBoxApduLogs.SelectionFont = new Font(RichTextBoxApduLogs.Font.Name, RichTextBoxApduLogs.Font.Size, FontStyle.Regular);

            RichTextBoxApduLogs.Select(RichTextBoxApduLogs.Text.Length, 0);
            RichTextBoxApduLogs.SelectionColor = Color.Black;
            RichTextBoxApduLogs.SelectedText = "\r\n" + msg + "\r\n";
            RichTextBoxApduLogs.ScrollToCaret();
        }

        void addMsgToLog(string prefix, byte[] tmpBytes, int len)
        {
            string msg = Helper.byteArrayToString(tmpBytes, 0, len, true);

            addMsgToLog(prefix + msg);
        }

        void addMsgToLog(string msg)
        {
            RichTextBoxApduLogs.Select(RichTextBoxApduLogs.Text.Length, 0);
            RichTextBoxApduLogs.SelectedText = msg + "\r\n";
            RichTextBoxApduLogs.ScrollToCaret();
        }

        void showWarningMessage(string message)
        {
            MessageBox.Show(message, "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
        }

        void showErrorMessage(string message)
        {
            MessageBox.Show(message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        void showInformationMessage(string message)
        {
            MessageBox.Show(message, "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void ButtonConnect_Click(object sender, EventArgs e)
        {
            byte[] atr = null;

            try
            {
                if (ComboBoxReaderList.SelectedItem == null)
                {
                    showErrorMessage("Please select a reader.");
                    return;
                }

                _readerFunctions.connect(ComboBoxReaderList.SelectedItem.ToString());
                addMsgToLog("\nSuccessfully connected to " + ComboBoxReaderList.Text);

                //Initialize NTAG203 classic class
                _ntag203 = new Ntag203(_readerFunctions.pcscConnection);

                _readerFunctions.getStatusChange(ref atr);
                _cardSelector.pcscReader = _readerFunctions;

                _sCardName = _cardSelector.readCardType(atr, (byte)atr.Length);

                if (_sCardName != "NTAG203")
                {
                    showErrorMessage("Card not supported.\r\nPlease present NTAG203 card.");
                    enableControls(false);
                    return;
                }

                addMsgToLog("Chip Type: " + _sCardName);
                addMsgToLog("");

                enableControls(true);
            }
            catch (PcscException pcscException)
            {
                addTitleToLog(false, "[" + pcscException.errorCode.ToString() + "] "
                    + pcscException.Message);

                showErrorMessage("[" + pcscException.errorCode.ToString() + "] "
                    + pcscException.Message);
            }
            catch (Exception generalException)
            {
                addMsgToLog(generalException.Message);
                showErrorMessage(generalException.Message);
            }
        }

        private void ButtonInitialize_Click(object sender, EventArgs e)
        {
            string[] readerList;
            try
            {
                _readerFunctions = new ReaderFunctions();

                //Register to event OnReceivedCommand
                _readerFunctions.OnReceivedCommand += new TransmitApduDelegate(readerFunctions_OnReceivedCommand);

                //Register to event OnSendCommand
                _readerFunctions.OnSendCommand += new TransmitApduDelegate(readerFunctions_OnSendCommand);

                //Get all smart card reader connected to computer
                readerList = _readerFunctions.getReaderList();

                ComboBoxReaderList.Items.Clear();
                ComboBoxReaderList.Items.AddRange(readerList);

                if (ComboBoxReaderList.Items.Count > 0)
                    ComboBoxReaderList.SelectedIndex = 0;

                addMsgToLog("\nInitialize success");

                ButtonConnect.Enabled = true;
            }
            catch (PcscException pcscException)
            {
                addTitleToLog(false, "[" + pcscException.errorCode.ToString() + "] "
                  + pcscException.Message);

                showErrorMessage("[" + pcscException.errorCode.ToString() + "] "
                    + pcscException.Message);
            }
            catch (Exception generalException)
            {
                addMsgToLog(generalException.Message);
                showErrorMessage(generalException.Message);
            }
        }

        void resetFields()
        {
            //_remainingSmartPosterCharacter = _maximumInputLength - (TextBoxSmartPosterTitle.Text.Length + TextBoxSmartPosterURI.Text.Length);
            //_remainingVCardCharacter = _maximumInputLength - (TextBoxGivenName.Text.Length + TextBoxMiddleName.Text.Length + TextBoxLastName.Text.Length);
            
            ComboBoxReaderList.Text = "";
            ComboBoxReaderList.Items.Clear();

            ComboBoxSmartPosterAction.SelectedIndex = 0;
            ComboBoxSmartPosterPrefixURI.SelectedIndex = 0;
            ComboBoxSmartPosterTitleLanguage.SelectedIndex = 0;
            ComboBoxTextLanguage.SelectedIndex = 0;
            ComboBoxUriPrefix.SelectedIndex = 0;

            TextBoxSmartPosterTitle.Text = "Advanced Card Systems Ltd.";
            TextBoxSmartPosterURI.Text = "http://www.acs.com.hk";
            TextBoxUriUri.Text = "http://www.acs.com.hk";
            TextBoxTextMessage.Text = "Hello World!";
            TextBoxGivenName.Text = "Given Name";
            TextBoxMiddleName.Text = "Middle Name";
            TextBoxLastName.Text = "Surname";

            RadioButtonUtf8.Checked = true;
            RadioButtonUtf16.Checked = false;
            ButtonConnect.Enabled = false;

            TabControlMain.SelectedIndex = 0;
            RichTextBoxEncodedNdefMessage.Clear();

            enableControls(false);

            RichTextBoxApduLogs.Clear();
            addMsgToLog("Program Ready");
        }

        void encodeToNdefSmartPoster()
        {
            //To make it nullable append '?'
            ACTION_RECORD? act = null;

            switch (ComboBoxSmartPosterAction.SelectedIndex)
            {
                case 1: act = ACTION_RECORD.DO_THE_ACTION; break;
                case 2: act = ACTION_RECORD.SAVE_FOR_LATER; break;
                case 3: act = ACTION_RECORD.OPEN_FOR_EDITING; break;
                default: act = null; break;
            }

            byte[] buffer = NDEF.Ndef.encodeSmartPoster(ComboBoxSmartPosterTitleLanguage.Text,
                                                       TextBoxSmartPosterTitle.Text,
                                                       (URI_IDENTIFIER)ComboBoxSmartPosterPrefixURI.SelectedIndex,
                                                       TextBoxSmartPosterURI.Text,
                                                       act);
          
            if (buffer == null || buffer.Length < 1)
            {
                RichTextBoxEncodedNdefMessage.Text = "";
                return;
            }

            if (buffer.Length > _maximumInputLength)
            {
                showErrorMessage(String.Format("Input characters exceeds maximum input length. Please remove {0} character/s from Title or URI.", buffer.Length - _maximumInputLength));
                return;
            }
             
             RichTextBoxEncodedNdefMessage.Text = Helper.byteAsString(buffer, false);
        }

        void encodeToNdefMessage()
        {
            NdefMessage ndefMessage;
            NdefRecord tmpRecord;
            byte statusByte = 0;
            byte[] buffer;

            if (RadioButtonUtf16.Checked)
                statusByte = 0x80;

            //Length of the Language (ISO/IANA language code)
            //ex. "en-US"
            statusByte |= (byte)ComboBoxTextLanguage.Text.Length;

            tmpRecord = new NdefRecord(new NdefRecordType(TYPE_NAME_FORMAT.NFC_WELL_KNOWN_TYPE, "T"));
            tmpRecord.messageBegin = true;
            tmpRecord.messageEnd = true;
            tmpRecord.appendPayload(TextBoxTextMessage.Text);

            tmpRecord.payLoad = new byte[] { statusByte };
            tmpRecord.payLoad = Helper.appendArrays(tmpRecord.payLoad, ASCIIEncoding.ASCII.GetBytes(ComboBoxTextLanguage.Text));
            tmpRecord.payLoad = Helper.appendArrays(tmpRecord.payLoad, ASCIIEncoding.ASCII.GetBytes(TextBoxTextMessage.Text));

            ndefMessage = new NdefMessage();
            ndefMessage.appendRecord(tmpRecord);

            buffer = ndefMessage.toByteArray();

            if (buffer.Length > _maximumInputLength)
            {
                showErrorMessage(String.Format("Input characters exceeds maximum input length. Please remove {0} characters from Message.", buffer.Length - _maximumInputLength));
                return;
            }

            RichTextBoxEncodedNdefMessage.Text = Helper.byteAsString(buffer, false);
        }

        void encodeToNdefUri()
        {
            NdefMessage ndefMessage;
            NdefRecord tmpRecord;
            byte[] buffer;

            //Encode URI
            tmpRecord = new NdefRecord(new NdefRecordType(TYPE_NAME_FORMAT.NFC_WELL_KNOWN_TYPE, "U"));

            tmpRecord.messageBegin = true;
            tmpRecord.messageEnd = true;

            //URI Prefix
            //See URI Record Type Definition 
            //http://www.nfc-forum.org
            tmpRecord.appendPayload((byte)((URI_IDENTIFIER)ComboBoxUriPrefix.SelectedIndex));
            tmpRecord.appendPayload(TextBoxUriUri.Text);

            ndefMessage = new NdefMessage();
            ndefMessage.appendRecord(tmpRecord);

            buffer = ndefMessage.toByteArray();

            if (buffer.Length > _maximumInputLength)
            {
                showErrorMessage(String.Format("Input characters exceeds maximum input length. Please remove {0} characters from URI.", buffer.Length - _maximumInputLength));
                return;
            }

            RichTextBoxEncodedNdefMessage.Text = Helper.byteAsString(buffer, false);
        }

        void encodeVCard()
        {
            string formattedString = "BEGIN:VCARD\r\nVERSION:3.0";
            byte[] uBuffer;
            byte[] buffer;
            NdefRecord ndef;
            NdefMessage ndefMessage;

            IdentificationType identificationType = new IdentificationType();
            identificationType.givenName = TextBoxGivenName.Text;
            identificationType.familyName = TextBoxLastName.Text;
            identificationType.additionalName = TextBoxMiddleName.Text;

            formattedString += "\r\n" + identificationType.getFormattedString();

            formattedString += "\r\nEND:VCARD\r\n\r\n";

            addTitleToLog(false, formattedString);

            uBuffer = ASCIIEncoding.ASCII.GetBytes(formattedString);

            ndef = new NdefRecord(new NdefRecordType(TYPE_NAME_FORMAT.MEDIA_TYPE, "text/x-vCard"));
            ndef.appendPayload(uBuffer);

            ndefMessage = new NdefMessage();
            ndefMessage.appendRecord(ndef);

            buffer = ndefMessage.toByteArray();

            if (buffer.Length > _maximumInputLength)
            {
                showErrorMessage(String.Format("Input characters exceeds maximum input length. Please remove {0} characters from Name.", buffer.Length - _maximumInputLength));
                return;
            }

            RichTextBoxEncodedNdefMessage.Text = "";
            RichTextBoxEncodedNdefMessage.Text = Helper.byteAsString(buffer, false);
        }

        void readerFunctions_OnSendCommand(object sender, TransmitApduEventArg e)
        {
            addMsgToLog("<< ", e.data, e.data.Length);
        }

        void readerFunctions_OnReceivedCommand(object sender, TransmitApduEventArg e)
        {
            addMsgToLog(">> ", e.data, e.data.Length);
        }

        private void ButtonSmartPosterEncodeMessage_Click(object sender, EventArgs e)
        {
            if (TextBoxSmartPosterURI.Text.Trim() == "")
            {
                showErrorMessage("Please key-in Smart Poster URI.");
                TextBoxSmartPosterURI.Focus();
                return;
            }

            if (ComboBoxSmartPosterPrefixURI.SelectedIndex == 5 && !isValidTel(TextBoxSmartPosterURI.Text.Trim()))
            {
                showErrorMessage("Please key-in correct tel number.");
                TextBoxSmartPosterURI.Focus();
                return;
            }

            encodeToNdefSmartPoster();
        }

        private void ButtonUriEncodeUri_Click(object sender, EventArgs e)
        {
            if (TextBoxUriUri.Text.Trim() == "")
            {
                showErrorMessage("Please key-in URI.");
                TextBoxUriUri.Focus();
                return;
            }

            if (ComboBoxUriPrefix.SelectedIndex == 5 && !isValidTel(TextBoxUriUri.Text.Trim()))
            {
                showErrorMessage("Please key-in correct tel number.");
                TextBoxUriUri.Focus();
                return;
            }

            encodeToNdefUri();
        }

        private void ButtonTextEncodeText_Click(object sender, EventArgs e)
        {
            if (TextBoxTextMessage.Text.Trim() == "")
            {
                showErrorMessage("Please key-in Message.");
                TextBoxTextMessage.Focus();
                return;
            }

            encodeToNdefMessage();
        }

        private void ButtonEncodevCard_Click(object sender, EventArgs e)
        {
            if (TextBoxGivenName.Text.Trim() == "")
            {
                showErrorMessage("Please key-in Given Name.");
                TextBoxGivenName.Focus();
                return;
            }            

            if (TextBoxLastName.Text.Trim() == "")
            {
                showErrorMessage("Please key-in Surname.");
                TextBoxLastName.Focus();
                return;
            }

            if (!isValidName(TextBoxGivenName.Text.Trim()))
            {
                showErrorMessage("Please key-in correct Given Name.");
                TextBoxGivenName.Focus();
                return;
            }

            if (!isValidName(TextBoxMiddleName.Text.Trim()))
            {
                showErrorMessage("Please key-in correct Middle Name.");
                TextBoxMiddleName.Focus();
                return;
            }

            if (!isValidName(TextBoxLastName.Text.Trim()))
            {
                showErrorMessage("Please key-in correct Surname.");
                TextBoxLastName.Focus();
                return;
            }

            encodeVCard();
        }

        private void FormMain_Load(object sender, EventArgs e)
        {
            resetFields();
        }

        private void ButtonReset_Click(object sender, EventArgs e)
        {
            resetFields();
        }

        private void ButtonClearLogs_Click(object sender, EventArgs e)
        {
            RichTextBoxApduLogs.Text = "";
        }

        private void ButtonExit_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void ButtonWriteToCard_Click(object sender, EventArgs e)
        {
            byte[] dataToWrite = new byte[0];
            byte[] arrLength;
            byte[] buffer;
            byte[] tmpArray = new byte[16];
            byte numberOfWritesNeeded = 0;
            byte blockNumber = 0x04;
            int dataLength;            
            int lenToCopy = 4;

            const int DATA_LENGTH = 255;
            const int BLOCK_DATA_LENGTH = 4;
            const double BLOCK_LENGTH = 4.00;

            try
            {
                if (RichTextBoxEncodedNdefMessage.Text.Trim() == "")
                {
                    showErrorMessage("There is no NDEF message to write. Please encode NDEF message first.");
                    return;
                }

                dataToWrite = Helper.getBytes(RichTextBoxEncodedNdefMessage.Text);

                if (dataToWrite.Length > _maximumInputLength)
                {
                    showErrorMessage("NDEF Message too long. NDEF length should be at most 141 bytes.");
                    return;
                }

                if (dataToWrite.Length < 1)
                {
                    if (RichTextBoxEncodedNdefMessage.Text.Trim() == "")
                    {
                        showErrorMessage("There is no NDEF message to write. Please encode NDEF message first.");
                        return;
                    }
                }

                //Form the data to be written to the card
                buffer = new byte[0];
                if (dataToWrite.Length < DATA_LENGTH)
                {
                    //If data is less than or equal to 254 (0xFE) use one byte format
                    //Refer to Type 2 Tag Operation document
                    //http://www.nxp.com/
                    buffer = Helper.appendArrays(buffer, new byte[] { (byte)NFCForum.TAGs.NDEF_MESSAGE_TLV, (byte)dataToWrite.Length });
                }
                else
                {
                    //If data is greater than 254 (0xFE) use 3 bytes format
                    dataLength = dataToWrite.Length;

                    //In 3 bytes format the last 2 bytes is for 00FE - FFFF                    
                    dataLength -= 254;

                    arrLength = new byte[3];

                    //Indicates that two more bytes folow
                    arrLength[0] = 0xFF;
                    arrLength[1] = (byte)((dataLength & 0xFF00) >> 8);
                    arrLength[2] = (byte)(dataLength & 0xFF);

                    //NDEF Message TAG
                    buffer = Helper.appendArrays(buffer, (byte)NFCForum.TAGs.NDEF_MESSAGE_TLV);

                    //length
                    buffer = Helper.appendArrays(buffer, arrLength);
                }

                buffer = Helper.appendArrays(buffer, dataToWrite);

                //Append Terminator TLV
                buffer = Helper.appendArrays(buffer, (byte)NFCForum.TAGs.TERMINATOR_TLV);
                
                //Write the data to NTAG
                //Start from Block 4
                addTitleToLog(false, "Write NDEF Message To Card");

                //Get number of blocks needed
                numberOfWritesNeeded = (byte)((double)Math.Ceiling(buffer.Length / BLOCK_LENGTH));
                blockNumber = 0x04; //Start to write at Block 4

                for (int indx = 0; indx < numberOfWritesNeeded; indx++, blockNumber += (BLOCK_DATA_LENGTH/4))
                {
                    tmpArray = new byte[BLOCK_DATA_LENGTH];
                    lenToCopy = BLOCK_DATA_LENGTH;

                    //Check if remaining data to be copied is less than 16
                    if ((buffer.Length - (indx * BLOCK_DATA_LENGTH)) > BLOCK_DATA_LENGTH)
                        lenToCopy = BLOCK_DATA_LENGTH;
                    else
                        lenToCopy = buffer.Length - (indx * BLOCK_DATA_LENGTH);

                    Array.Copy(buffer, (indx * BLOCK_DATA_LENGTH), tmpArray, 0, lenToCopy);

                    addTitleToLog(false, "Write in block " + blockNumber.ToString());

                    _ntag203.updateBinary(blockNumber, tmpArray, BLOCK_DATA_LENGTH);
                }
            }
            catch (CardException cardException)
            {
                addTitleToLog(false, "[" + Helper.byteAsString(cardException.statusWord, true) +
                    "] " + cardException.Message);

                showErrorMessage(cardException.Message);
            }
            catch (PcscException pcscException)
            {
                addTitleToLog(false, "[" + pcscException.errorCode.ToString() + "] "
                    + pcscException.Message);

                showErrorMessage("[" + pcscException.errorCode.ToString() + "] "
                    + pcscException.Message);
            }
            catch (Exception generalException)
            {
                addMsgToLog(generalException.Message);
                showErrorMessage(generalException.Message);
            }
        }

        private void RichTextBoxEncodedNdefMessage_TextChanged(object sender, EventArgs e)
        {
            byte[] buff = Helper.getBytes(RichTextBoxEncodedNdefMessage.Text);

            if (buff == null)
                return;

            ButtonWriteToCard.Enabled = true;
            ButtonReadFromCard.Enabled = true;
        }

        private void ButtonReadFromCard_Click(object sender, EventArgs e)
        {
            readFromCard();
        }

        void readFromCard()
        {
            byte blockNumber = 0x04; //starting block
            int endIndex = 0;
            int temporaryLen;
            int temporaryPayLoadLen;
            int numberOfBlocks = 0;
            string payLoadTypeNameStr = "";

            byte[] buffer = new byte[0];
            byte[] tmpArray;
            string[] vCard;
            string[] vCardInfo;
            string[] delimeter = new string[] { "\r\n" };

            NdefMessage ndefMessage;
            NdefMessage temporaryNdefMessage;
            List<NdefRecord> ndefRecords;
            NdefRecord temporaryNdefRecord;

            const int BLOCK_DATA_LENGTH = 16;
            const double BLOCK_LENGTH = 16.00;
            const int THREE_BYTE_FORMAT_ADDTL = 5;
            const int NON_THREE_BYTE_FORMAT_ADDTL = 3;

            try
            {
                RichTextBoxEncodedNdefMessage.Text = "";

                buffer = new byte[0];

                tmpArray = _ntag203.readBinary(blockNumber, 0x10);
                buffer = Helper.appendArrays(buffer, tmpArray);

                if (buffer[1] == 0xFF)
                {
                    numberOfBlocks = (byte)((double)Math.Ceiling(((int)buffer[1] + THREE_BYTE_FORMAT_ADDTL) / BLOCK_LENGTH));
                }
                else
                {
                    numberOfBlocks = (byte)((double)Math.Ceiling(((int)buffer[1] + NON_THREE_BYTE_FORMAT_ADDTL) / BLOCK_LENGTH));
                }

                blockNumber += 4;

                for (int indx = 0; indx < numberOfBlocks - 1; indx++, blockNumber += 4)
                {
                    tmpArray = new byte[BLOCK_DATA_LENGTH];

                    addTitleToLog(false, "Read block " + blockNumber.ToString());

                    tmpArray = _ntag203.readBinary(blockNumber, 0x10);
                    buffer = Helper.appendArrays(buffer, tmpArray);
                }

                //Check if NDEF Message TAG is present
                if (buffer[0] != (byte)(NFCForum.TAGs.NDEF_MESSAGE_TLV))
                    throw new Exception("NDEF Message TAG is not found");

                //Find the Terminator TLV
                for (endIndex = 0; endIndex < buffer.Length; endIndex++)
                {
                    if (buffer[endIndex] == (byte)(NFCForum.TAGs.TERMINATOR_TLV))
                        break;

                    if ((endIndex + 1) >= buffer.Length)
                        throw new Exception("Invalid Card.\r\nTerminator Tag is not found");
                }

                Array.Resize(ref buffer, endIndex);

                //Remove NDEF Message Tag
                //Check if it uses one byte or 3 byte format for the length
                if (buffer[1] == 0xFF)
                {
                    //Uses 3 bytes format
                    tmpArray = new byte[buffer.Length - (THREE_BYTE_FORMAT_ADDTL - 1)];
                    Array.Copy(buffer, THREE_BYTE_FORMAT_ADDTL - 1, tmpArray, 0, tmpArray.Length);
                }
                else
                {
                    tmpArray = new byte[buffer.Length - (NON_THREE_BYTE_FORMAT_ADDTL - 1)];
                    Array.Copy(buffer, NON_THREE_BYTE_FORMAT_ADDTL - 1, tmpArray, 0, tmpArray.Length);
                }

                RichTextBoxEncodedNdefMessage.Text = Helper.byteAsString(tmpArray, false);

                ndefMessage = NdefRecord.getNestedNdefMessage(0, tmpArray);

                if (ndefMessage.getNumberOfRecords() < 0)
                {
                    showInformationMessage("No record to display");
                    return;
                }

                ndefRecords = ndefMessage.getRecords();

                payLoadTypeNameStr = ndefRecords[0].recordType.typeName;

                if (payLoadTypeNameStr == "Sp")
                {
                    temporaryNdefMessage = ndefRecords[0].getNestedNdefMessage();

                    if (temporaryNdefMessage.getNumberOfRecords() < 1)
                        throw new Exception("Invalid SmartPoster NDEF Message");

                    //Get URI Record (Mandatory)
                    temporaryNdefRecord = temporaryNdefMessage.getRecord(new NdefRecordType(TYPE_NAME_FORMAT.NFC_WELL_KNOWN_TYPE, "U"));
                    if (temporaryNdefRecord == null)
                    {
                        showErrorMessage("Mandatory URI Record is not found");
                        return;
                    }

                    ComboBoxSmartPosterPrefixURI.SelectedIndex = temporaryNdefRecord.payLoad[0];
                    TextBoxSmartPosterURI.Text = ASCIIEncoding.ASCII.GetString(temporaryNdefRecord.payLoad.Skip(1).Take(temporaryNdefRecord.payLoad.Length - 1).ToArray());

                    //Get Action Record
                    ComboBoxSmartPosterAction.SelectedIndex = 0;
                    temporaryNdefRecord = temporaryNdefMessage.getRecord(new NdefRecordType(TYPE_NAME_FORMAT.NFC_WELL_KNOWN_TYPE, "act"));
                    if (temporaryNdefRecord != null)
                    {
                        switch ((NDEF.ACTION_RECORD)temporaryNdefRecord.payLoad[0])
                        {
                            case ACTION_RECORD.DO_THE_ACTION:
                                {
                                    ComboBoxSmartPosterAction.SelectedIndex = 1;
                                    break;
                                }
                            case ACTION_RECORD.SAVE_FOR_LATER:
                                {
                                    ComboBoxSmartPosterAction.SelectedIndex = 2;
                                    break;
                                }
                            case ACTION_RECORD.OPEN_FOR_EDITING:
                                {
                                    ComboBoxSmartPosterAction.SelectedIndex = 3;
                                    break;
                                }
                            default:
                                {
                                    ComboBoxSmartPosterAction.SelectedIndex = 0;
                                    break;
                                }
                        }
                    }
                    else
                    {
                        ComboBoxSmartPosterAction.SelectedIndex = 0;
                    }

                    //Get Text Record
                    temporaryNdefRecord = temporaryNdefMessage.getRecord(new NdefRecordType(TYPE_NAME_FORMAT.NFC_WELL_KNOWN_TYPE, "T"));
                    if (temporaryNdefRecord != null)
                    {
                        temporaryLen = temporaryNdefRecord.payLoad[0] & 0x1F;
                        ComboBoxSmartPosterTitleLanguage.Text = ASCIIEncoding.ASCII.GetString(temporaryNdefRecord.payLoad.Skip(1).Take(temporaryLen).ToArray());

                        temporaryPayLoadLen = temporaryNdefRecord.payLoad.Length - (temporaryLen - 1);
                        TextBoxSmartPosterTitle.Text = ASCIIEncoding.ASCII.GetString(temporaryNdefRecord.payLoad.Skip(temporaryLen + 1).Take(temporaryPayLoadLen).ToArray());
                    }
                    else
                    {
                        TextBoxSmartPosterTitle.Text = "";
                    }

                    showInformationMessage("Smart Poster Message Found");
                    TabControlMain.SelectedIndex = 0;
                }
                else if (payLoadTypeNameStr == "T")
                {
                    //Get Text Record
                    temporaryNdefRecord = ndefRecords[0];

                    if ((temporaryNdefRecord.payLoad[0] & 0x80) != 0x00)
                        RadioButtonUtf16.Checked = true;
                    else
                        RadioButtonUtf8.Checked = true;

                    temporaryLen = temporaryNdefRecord.payLoad[0] & 0x1F;
                    ComboBoxTextLanguage.Text = ASCIIEncoding.ASCII.GetString(temporaryNdefRecord.payLoad.Skip(1).Take(temporaryLen).ToArray());

                    temporaryPayLoadLen = temporaryNdefRecord.payLoad.Length - (temporaryLen - 1);
                    TextBoxTextMessage.Text = ASCIIEncoding.ASCII.GetString(temporaryNdefRecord.payLoad.Skip(temporaryLen + 1).Take(temporaryPayLoadLen).ToArray());

                    showInformationMessage("Text Message Found");
                    TabControlMain.SelectedIndex = 2;

                }
                else if (payLoadTypeNameStr == "U")
                {
                    temporaryNdefRecord = ndefRecords[0];
                    ComboBoxUriPrefix.SelectedIndex = temporaryNdefRecord.payLoad[0];
                    TextBoxUriUri.Text = ASCIIEncoding.ASCII.GetString(temporaryNdefRecord.payLoad.Skip(1).Take(temporaryNdefRecord.payLoad.Length - 1).ToArray());

                    showInformationMessage("URI Message Found");
                    TabControlMain.SelectedIndex = 1;
                }
                else if (payLoadTypeNameStr == "text/x-vCard")
                {
                    temporaryNdefRecord = ndefRecords[0];
                    vCard = ASCIIEncoding.ASCII.GetString(temporaryNdefRecord.payLoad).Split(delimeter, StringSplitOptions.None);
                    vCardInfo = vCard[2].Split(new char[] { ';' });
                    TextBoxLastName.Text = vCardInfo[0].Substring(2, vCardInfo[0].Length - 2);
                    TextBoxGivenName.Text = vCardInfo[1];
                    TextBoxMiddleName.Text = vCardInfo[2];

                    showInformationMessage("vCard Message Found");
                    TabControlMain.SelectedIndex = 3;
                }
                else
                {
                    showInformationMessage("Unsupported payload type");
                }
            }
            catch (CardException cardException)
            {
                addTitleToLog(false, "[" + Helper.byteAsString(cardException.statusWord, true) +
                    "] " + cardException.Message);

                showErrorMessage(cardException.Message);
            }
            catch (PcscException pcscException)
            {
                addTitleToLog(false, "[" + pcscException.errorCode.ToString() + "] "
                    + pcscException.Message);

                showErrorMessage("[" + pcscException.errorCode.ToString() + "] "
                    + pcscException.Message);
            }
            catch (Exception generalException)
            {
                addMsgToLog(generalException.Message);
                showErrorMessage(generalException.Message);
            }
        }

        private void TextBoxGivenName_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.Handled = !(char.IsLetter(e.KeyChar) || e.KeyChar == (char)Keys.Back ||
                          e.KeyChar == (char)Keys.Delete || e.KeyChar == (char)Keys.OemPeriod ||
                          e.KeyChar == (char)Keys.Decimal || e.KeyChar == (char)Keys.Space ||
                          e.KeyChar == '-');
        }

        private void ComboBoxSmartPosterPrefixURI_SelectedIndexChanged(object sender, EventArgs e)
        {
            TextBoxSmartPosterURI.Text = "";
        }

        private void ComboBoxURIPrefix_SelectedIndexChanged(object sender, EventArgs e)
        {
            TextBoxUriUri.Text = "";
        }

        private void TextBoxSmartPosterURI_Leave(object sender, EventArgs e)
        {
            if (TextBoxSmartPosterURI.Text.StartsWith(ComboBoxSmartPosterPrefixURI.Text))
                TextBoxSmartPosterURI.Text = TextBoxSmartPosterURI.Text.Remove(0, ComboBoxSmartPosterPrefixURI.Text.Length);
        }

        private void TextBoxUriUri_Leave(object sender, EventArgs e)
        {
            if (TextBoxUriUri.Text.StartsWith(ComboBoxUriPrefix.Text))
                TextBoxUriUri.Text = TextBoxUriUri.Text.Remove(0, ComboBoxUriPrefix.Text.Length);
        }

        private void TextBoxSmartPosterTitle_KeyPress(object sender, KeyPressEventArgs e)
        {
           
        }

        private void TextBoxSmartPosterURI_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (ComboBoxSmartPosterPrefixURI.SelectedIndex == 5)
            {
                e.Handled = !(char.IsNumber(e.KeyChar) || e.KeyChar == (char)Keys.Back ||
                                          e.KeyChar == '-' || e.KeyChar == '+'
                                         || e.KeyChar == '(' || e.KeyChar == ')');
            }
        }

        private void ComboBoxTextLanguage_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (ComboBoxTextLanguage.SelectedIndex == 1)
                TextBoxTextMessage.MaxLength = 131;
            else 
                TextBoxTextMessage.MaxLength = 134;

            if (!string.IsNullOrEmpty(TextBoxTextMessage.Text) &&
                TextBoxTextMessage.Text.Length > TextBoxTextMessage.MaxLength)
            {
                TextBoxTextMessage.Text = TextBoxTextMessage.Text.Substring(0, TextBoxTextMessage.MaxLength);
            }

        }

        void enableControls(bool isEnable)
        {
            TabControlMain.Enabled = isEnable;
            ButtonWriteToCard.Enabled = isEnable;
            ButtonReadFromCard.Enabled = isEnable;
        }

        public bool isValidName(string name)
        {
            foreach (char c in name)
            {
                if (!Char.IsLetter(c) && c != ' ' && c != '.' && c != '-')
                    return false;
            }

            return true;
        }

        public bool isValidTel(string name)
        {
            foreach (char c in name)
            {
                if (!Char.IsNumber(c) && c != '(' && c != ')' && c != '-' && c != '+')
                    return false;
            }

            return true;
        }

        private void TextBoxUriUri_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (ComboBoxUriPrefix.SelectedIndex == 5)
            {
                e.Handled = !(char.IsNumber(e.KeyChar) || e.KeyChar == (char)Keys.Back ||
                                          e.KeyChar == '-' || e.KeyChar == '+'
                                         || e.KeyChar == '(' || e.KeyChar == ')');
            }
        }
    }
}
